#include <QString>
#include <QComboBox>
#include <QCheckBox>
#include <QPushButton>
#include <QScrollArea>
#include <QHBoxLayout>
#include <QVBoxLayout>

#include <iostream>
using namespace std;

#include "sim.h"
#include "achievementspage.h"
#include "achievemententry.h"

AchievementsPage::AchievementsPage ()  {
  simFilterBox = new QComboBox ();
  simFilterBox->setSizeAdjustPolicy (QComboBox::AdjustToContents);
  filterButton = new QPushButton ("Filter by Sim");
  filterButton->setEnabled (false);
  filterButton->setMaximumSize (filterButton->sizeHint ());
  showAllButton = new QPushButton ("Show All");
  showAllButton->setEnabled (false);
  showAllButton->setMaximumSize (showAllButton->sizeHint ());
  showHoldsBox = new QCheckBox ("Show holds");
  showHoldsBox->setChecked (true);

  topLayout = new QHBoxLayout ();
  topLayout->addWidget (filterButton);
  topLayout->addWidget (simFilterBox);
  topLayout->addWidget (showAllButton);
  topLayout->addWidget (showHoldsBox);
  
  displayArea = new QScrollArea ();
  displayWidget = new QWidget ();
  displayLayout = new QVBoxLayout ();

  displayArea->setWidget (displayWidget);
  displayWidget->setLayout (displayLayout);

  mainLayout = new QVBoxLayout ();
  mainLayout->addLayout (topLayout);
  mainLayout->setAlignment (topLayout, Qt::AlignCenter);
  mainLayout->addWidget (displayArea);
  setLayout (mainLayout);

  connect (filterButton, SIGNAL (clicked ()), this, SLOT (filterEntries ()));
  connect (showHoldsBox, SIGNAL (clicked ()), this, SLOT (filterHolds ()));
  connect (showAllButton, SIGNAL (clicked ()), this, SLOT (showAll ()));
}

void AchievementsPage::addEntry (QString message, Sim *ref1, Sim *ref2)  {
  AchievementEntry *newEntry = new AchievementEntry (message, ref1, ref2);

  displayLayout->addWidget (newEntry);
  newEntry->show ();
  displayWidget->resize (displayLayout->minimumSize ());

  if (ref1 && !simFilterList.contains (ref1))  {
    simFilterList.append (ref1);
    simFilterBox->addItem (ref1->getFullName ());
  }

  if (ref2 && !simFilterList.contains (ref2))  {
    simFilterList.append (ref2);
    simFilterBox->addItem (ref2->getFullName ());
  }

  filterButton->setEnabled (simFilterList.size () > 0);
}

QString AchievementsPage::getCSV (QList< QPointer<Sim> > list)  {
  cout << "AchievementsPage" << endl;
  cout << displayLayout << endl;
  cout << displayLayout->count () << endl;

  QString text = "";

  if (displayLayout->count () == 0)  {
    cout << "There are 0 items in the layout." << endl;
    return text;
  }

  for (int x = 0; x < displayLayout->count (); x++)  {
    AchievementEntry *entry =
         qobject_cast<AchievementEntry*>(displayLayout->itemAt (x)->widget ());

    int first = entry->getFirstSimNum (list);
    int second = entry->getSecondSimNum (list);

    text += "\"" + entry->text () + "\"," + QString::number (first) + "," +
            QString::number (second) + "\r\n"; 
  }

  return text;
}

void AchievementsPage::readCSV (QList< QPointer<Sim> > list, QString csv)  {
  QStringList slist = csv.split (',');

  QString message = slist[0];
  int currentSection = 1;

  while (!message.endsWith ("\""))
    message += "," + slist[currentSection++];

  message.chop (1);
  message.remove (0, 1);

  int first = slist[currentSection++].toInt ();
  int second = slist[currentSection++].toInt ();

  Sim *s1 = (first >= 0 ? list[first] : NULL);
  Sim *s2 = (second >= 0 ? list[second] : NULL);

  addEntry (message, s1, s2);
}

void AchievementsPage::clear ()  {
  while (displayLayout->count () > 0)  {
    QWidget *entry = displayLayout->itemAt (0)->widget ();
    displayLayout->removeWidget (entry);
    delete entry;
  }

  displayWidget->resize (displayLayout->minimumSize ());
  displayWidget->repaint ();
}

void AchievementsPage::filterEntries ()  {
  for (int x = 0; x < displayLayout->count (); x++)  {
    AchievementEntry *ptr = 
      qobject_cast<AchievementEntry*>(displayLayout->itemAt (x)->widget ());
    QRegExp regExp ("^Put .+ on hold for .+ rotations\\.");

    if (!(ptr->isReference (simFilterList[simFilterBox->currentIndex ()])))
      ptr->hide ();
    else  { 
      if (!showHoldsBox->isChecked () && regExp.exactMatch (ptr->text ()))
        ptr->hide ();
      else ptr->show ();
    }

    displayWidget->resize (displayLayout->minimumSize ());
  }

  showAllButton->setEnabled (true);
}

void AchievementsPage::filterHolds ()  {
  if (showAllButton->isEnabled ())
    filterEntries ();
  else showAll ();
}

void AchievementsPage::showAll ()  {
  QRegExp regExp ("^Put .+ on hold for .+ rotations\\.");

  for (int x = 0; x < displayLayout->count (); x++)  {
    AchievementEntry *ptr = 
         qobject_cast<AchievementEntry*>(displayLayout->itemAt (x)->widget ());

    if (!showHoldsBox->isChecked () && regExp.exactMatch (ptr->text ()))
      ptr->hide ();
    else ptr->show ();
  }

  displayWidget->resize (displayLayout->minimumSize ());
  showAllButton->setEnabled (false);
}
