#include <QLineEdit>
#include <QSpinBox>
#include <QLabel>
#include <QComboBox>
#include <QCheckBox>
#include <QPushButton>
#include <QHBoxLayout>
#include <QVBoxLayout>

#include "const.h"
#include "sim.h"
#include "addeditsimdialog.h"

AddEditSimDialog::AddEditSimDialog (int epf, QStringList houses)  {
  epFlags = epf;

  nameEdit = new QLineEdit;
  genderBox = new QComboBox;
  genderBox->addItem ("Male");
  genderBox->addItem ("Female");
  genderBox->setMaximumSize (genderBox->sizeHint ());
  QLabel *nLabel = new QLabel ("Name: ");
  nLabel->setMaximumSize (nLabel->sizeHint ());
  nameLayout = new QHBoxLayout;
  nameLayout->addWidget (nLabel);
  nameLayout->setAlignment (nLabel, Qt::AlignLeft);
  nameLayout->addWidget (nameEdit);
  nameLayout->addWidget (genderBox);

  ageBox = new QSpinBox;
  ageBox->setMinimum (0);
  ageBox->setMaximum (85);
  ageBox->setSuffix (" days old");
  ageBox->setValue (ADULT);
  ageLabel = new QLabel ("(" + ageFormat (ADULT, HUMAN) + ")");
  QLabel *aLabel = new QLabel ("Age: ");
  aLabel->setMaximumSize (aLabel->sizeHint ());
  ageLayout = new QHBoxLayout;
  ageLayout->addWidget (aLabel);
  ageLayout->setAlignment (aLabel, Qt::AlignLeft);
  ageLayout->addWidget (ageBox);
  ageLayout->addWidget (ageLabel);
  ageLayout->addStretch (1);

  connect (ageBox, SIGNAL (valueChanged (int)), this, SLOT (ageChanged (int)));

  deadBox = new QCheckBox ("Is a Ghost    Color:");
  ghostColorBox = new QComboBox;
  ghostColorBox->addItem ("Pink");
  ghostColorBox->addItem ("Green");
  ghostColorBox->addItem ("Yellow");
  ghostColorBox->addItem ("Blue");
  ghostColorBox->addItem ("Orange");
  ghostColorBox->addItem ("Purple");
  ghostColorBox->addItem ("Red");
  ghostColorBox->addItem ("White");
  ghostColorBox->addItem ("Transparent");
  ghostColorBox->addItem ("Bright Green");
  deathLayout = new QHBoxLayout;
  deathLayout->addWidget (deadBox);
  deathLayout->addWidget (ghostColorBox);
  deathLayout->addStretch (1);

  speciesBox = new QComboBox;
  speciesBox->addItem ("Human");
  speciesBox->addItem ("Large Dog");
  speciesBox->addItem ("Small Dog");
  speciesBox->addItem ("Cat");
  speciesBox->setMaximumSize (speciesBox->sizeHint ());
  if (!(epFlags & PETS)) speciesBox->setEnabled (false);
  QLabel *sLabel = new QLabel ("Species: ");
  sLabel->setMaximumSize (sLabel->sizeHint ());
  alignmentBox = new QComboBox;
  alignmentBox->addItem ("Atrociously Evil");
  alignmentBox->addItem ("Evil");
  alignmentBox->addItem ("Mean");
  alignmentBox->addItem ("Neutral");
  alignmentBox->addItem ("Nice");
  alignmentBox->addItem ("Good");
  alignmentBox->addItem ("Infallibly Good");
  alignmentBox->setMaximumSize (alignmentBox->sizeHint ());
  alignmentBox->setCurrentIndex (3);
  alignmentBox->setEnabled (false);
  QLabel *alLabel = new QLabel ("Alignment: ");
  alLabel->setMaximumSize (alLabel->sizeHint ());
  speciesLayout = new QHBoxLayout;
  speciesLayout->addWidget (sLabel);
  speciesLayout->setAlignment (sLabel, Qt::AlignLeft);
  speciesLayout->addWidget (speciesBox);
  speciesLayout->addStretch (1);
  speciesLayout->addWidget (alLabel);
  speciesLayout->addWidget (alignmentBox);

  connect (speciesBox, SIGNAL (currentIndexChanged (int)), this,
           SLOT (speciesChanged ()));

  householdBox = new QComboBox;
  householdBox->addItems (houses);
  householdBox->setEditable (true);
  QLabel *hLabel = new QLabel ("Household: ");
  hLabel->setMaximumSize (hLabel->sizeHint ());
  householdLayout = new QHBoxLayout;
  householdLayout->addWidget (hLabel);
  householdLayout->addWidget (householdBox);

  aspirationBox = new QComboBox;
  aspirationBox->addItem ("None/Unknown");
  aspirationBox->addItem ("Fortune");
  aspirationBox->addItem ("Family");
  aspirationBox->addItem ("Romance");
  aspirationBox->addItem ("Popularity");
  aspirationBox->addItem ("Knowledge");
  aspirationBox->addItem ("Pleasure");
  aspirationBox->addItem ("Grilled Cheese");
  aspirationBox->addItem ("Grow Up");
  secondaryAspirationBox = new QComboBox;
  secondaryAspirationBox->addItem ("None/Unknown");
  secondaryAspirationBox->addItem ("Fortune");
  secondaryAspirationBox->addItem ("Family");
  secondaryAspirationBox->addItem ("Romance");
  secondaryAspirationBox->addItem ("Popularity");
  secondaryAspirationBox->addItem ("Knowledge");
  secondaryAspirationBox->addItem ("Pleasure");
  secondaryAspirationBox->addItem ("Grilled Cheese");
  if (!(epFlags & FREETIME)) secondaryAspirationBox->setEnabled (false);
  QLabel *aspLabel = new QLabel ("Aspiration: ");
  QLabel *slashLabel = new QLabel ("/");
  aspirationLayout = new QHBoxLayout;
  aspirationLayout->addWidget (aspLabel);
  aspirationLayout->addWidget (aspirationBox);
  aspirationLayout->addWidget (slashLabel);
  aspirationLayout->addWidget (secondaryAspirationBox);
  aspirationLayout->addStretch (1);

  semesterBox = new QSpinBox;
  semesterBox->setMinimum (0);
  semesterBox->setMaximum (8);
  if (!(epFlags & UNIVERSITY)) semesterBox->setEnabled (false);
  majorEdit = new QLineEdit;
  majorEdit->setEnabled (false);
  QLabel *cLabel = new QLabel ("Semester in college: ");
  QLabel *mLabel = new QLabel ("Major: ");
  collegeLayout = new QHBoxLayout;
  collegeLayout->addWidget (cLabel);
  collegeLayout->addWidget (semesterBox);
  collegeLayout->addWidget (mLabel);
  collegeLayout->addWidget (majorEdit);

  connect (semesterBox, SIGNAL (valueChanged (int)), this, 
           SLOT (semesterChanged (int)));

  alienBox = new QCheckBox ("Alien");
  zombieBox = new QCheckBox ("Zombie");
  if (!(epFlags & UNIVERSITY)) zombieBox->setEnabled (false);
  vampireBox = new QCheckBox ("Vampire");
  if (!(epFlags & NIGHTLIFE)) vampireBox->setEnabled (false);
  werewolfBox = new QCheckBox ("Werewolf");
  if (!(epFlags & PETS)) werewolfBox->setEnabled (false);
  servoBox = new QCheckBox ("Servo");
  if (!(epFlags & OFB)) servoBox->setEnabled (false);
  plantsimBox = new QCheckBox ("Plantsim");
  if (!(epFlags & SEASONS)) plantsimBox->setEnabled (false);
  bigfootBox = new QCheckBox ("Bigfoot");
  if (!(epFlags & BONVOYAGE)) bigfootBox->setEnabled (false);
  witchBox = new QCheckBox ("Witch");
  if (!(epFlags & APARTMENTLIFE)) witchBox->setEnabled (false);

  connect (plantsimBox, SIGNAL (toggled (bool)), this, 
           SLOT (speciesChanged ()));
  connect (witchBox, SIGNAL (toggled (bool)), this,
           SLOT (witchChanged (bool)));

  paranormalColumnOne = new QVBoxLayout;
  paranormalColumnOne->addWidget (alienBox);
  paranormalColumnOne->addWidget (zombieBox);
  paranormalColumnOne->addWidget (vampireBox);
  paranormalColumnOne->addWidget (werewolfBox);
  paranormalColumnTwo = new QVBoxLayout;
  paranormalColumnTwo->addWidget (servoBox);
  paranormalColumnTwo->addWidget (plantsimBox);
  paranormalColumnTwo->addWidget (bigfootBox);
  paranormalColumnTwo->addWidget (witchBox);
  paranormalLayout = new QHBoxLayout;
  paranormalLayout->addLayout (paranormalColumnOne);
  paranormalLayout->addLayout (paranormalColumnTwo);

  submitButton = new QPushButton ("");

  mainLayout = new QVBoxLayout;
  mainLayout->addLayout (nameLayout);
  mainLayout->addLayout (ageLayout);
  mainLayout->addLayout (deathLayout);
  mainLayout->addLayout (speciesLayout);
  mainLayout->addLayout (householdLayout);
  mainLayout->addLayout (aspirationLayout);
  mainLayout->addLayout (collegeLayout);
  mainLayout->addLayout (paranormalLayout);
  mainLayout->addWidget (submitButton);
  mainLayout->setAlignment (submitButton, Qt::AlignCenter);

  setLayout (mainLayout);
}

AddEditSimDialog::AddEditSimDialog (int epf, QStringList houses, Sim *sim)  {
  epFlags = epf;

  nameEdit = new QLineEdit (sim->getFullName ());
  genderBox = new QComboBox;
  genderBox->addItem ("Male");
  genderBox->addItem ("Female");
  genderBox->setMaximumSize (genderBox->sizeHint ());
  genderBox->setCurrentIndex (sim->getGender () - 1);
  QLabel *nLabel = new QLabel ("Name: ");
  nLabel->setMaximumSize (nLabel->sizeHint ());
  nameLayout = new QHBoxLayout;
  nameLayout->addWidget (nLabel);
  nameLayout->setAlignment (nLabel, Qt::AlignLeft);
  nameLayout->addWidget (nameEdit);
  nameLayout->addWidget (genderBox);

  ageBox = new QSpinBox;
  ageBox->setMinimum (0);
  ageBox->setMaximum (85);
  ageBox->setSuffix (" days old");
  ageBox->setValue (sim->getAge ());

  int specials = 0;
  if (sim->isParanormal (PLANTSIM)) specials += PLANTSIM;
  if (sim->isSpecies (LARGE_DOG)) specials += LARGE_DOG;
  if (sim->isSpecies (SMALL_DOG)) specials += SMALL_DOG;
  if (sim->isSpecies (CAT)) specials += CAT;
  ageLabel = new QLabel ("(" + ageFormat (sim->getAge (), specials) + ")");

  QLabel *aLabel = new QLabel ("Age: ");
  aLabel->setMaximumSize (aLabel->sizeHint ());
  ageLayout = new QHBoxLayout;
  ageLayout->addWidget (aLabel);
  ageLayout->addWidget (ageBox);
  ageLayout->addWidget (ageLabel);
  ageLayout->setAlignment (aLabel, Qt::AlignLeft);
  ageLayout->addStretch (1);

  connect (ageBox, SIGNAL (valueChanged (int)), this, SLOT (ageChanged (int)));

  deadBox = new QCheckBox ("Is a Ghost    Color:");
  deadBox->setChecked (!sim->isAlive ());
  ghostColorBox = new QComboBox;
  ghostColorBox->addItem ("Pink");
  ghostColorBox->addItem ("Green");
  ghostColorBox->addItem ("Yellow");
  ghostColorBox->addItem ("Blue");
  ghostColorBox->addItem ("Orange");
  ghostColorBox->addItem ("Purple");
  ghostColorBox->addItem ("Red");
  ghostColorBox->addItem ("White");
  ghostColorBox->addItem ("Transparent");
  ghostColorBox->addItem ("Bright Green");
  ghostColorBox->setCurrentIndex (sim->getGhostColor ());
  deathLayout = new QHBoxLayout;
  deathLayout->addWidget (deadBox);
  deathLayout->addWidget (ghostColorBox);
  deathLayout->addStretch (1);

  speciesBox = new QComboBox;
  speciesBox->addItem ("Human");
  speciesBox->addItem ("Large Dog");
  speciesBox->addItem ("Small Dog");
  speciesBox->addItem ("Cat");
  speciesBox->setMaximumSize (speciesBox->sizeHint ());
  if (!(epFlags & PETS)) speciesBox->setEnabled (false);

  if (sim->isSpecies (LARGE_DOG)) speciesBox->setCurrentIndex (1);
  else if (sim->isSpecies (SMALL_DOG)) speciesBox->setCurrentIndex (2);
  else if (sim->isSpecies (CAT)) speciesBox->setCurrentIndex (3);

  QLabel *sLabel = new QLabel ("Species: ");
  sLabel->setMaximumSize (sLabel->sizeHint ());
  alignmentBox = new QComboBox;
  alignmentBox->addItem ("Atrociously Evil");
  alignmentBox->addItem ("Evil");
  alignmentBox->addItem ("Mean");
  alignmentBox->addItem ("Neutral");
  alignmentBox->addItem ("Nice");
  alignmentBox->addItem ("Good");
  alignmentBox->addItem ("Infallibly Good");
  alignmentBox->setMaximumSize (alignmentBox->sizeHint ());
  alignmentBox->setCurrentIndex (sim->getAlignment () + 3);
  alignmentBox->setEnabled (false);
  QLabel *alLabel = new QLabel ("Alignment: ");
  alLabel->setMaximumSize (alLabel->sizeHint ());
  speciesLayout = new QHBoxLayout;
  speciesLayout->addWidget (sLabel);
  speciesLayout->setAlignment (sLabel, Qt::AlignLeft);
  speciesLayout->addWidget (speciesBox);
  speciesLayout->addStretch (1);
  speciesLayout->addWidget (alLabel);
  speciesLayout->addWidget (alignmentBox);

  connect (speciesBox, SIGNAL (currentIndexChanged (int)), this,
           SLOT (speciesChanged ()));

  householdBox = new QComboBox;
  householdBox->addItems (houses);
  householdBox->setEditable (true);

  int index = 0;
  for (int x = 0; x < houses.size (); x++)
    if (houses[x] == sim->getHousehold ())
      index = x;

  householdBox->setCurrentIndex (index);
  QLabel *hLabel = new QLabel ("Household: ");
  hLabel->setMaximumSize (hLabel->sizeHint ());
  householdLayout = new QHBoxLayout;
  householdLayout->addWidget (hLabel);
  householdLayout->addWidget (householdBox);

  aspirationBox = new QComboBox;
  aspirationBox->addItem ("None/Unknown");
  aspirationBox->addItem ("Fortune");
  aspirationBox->addItem ("Family");
  aspirationBox->addItem ("Romance");
  aspirationBox->addItem ("Popularity");
  aspirationBox->addItem ("Knowledge");
  aspirationBox->addItem ("Pleasure");
  aspirationBox->addItem ("Grilled Cheese");
  aspirationBox->addItem ("Grow Up");
  aspirationBox->setCurrentIndex (sim->getAspirationCode ());
  secondaryAspirationBox = new QComboBox;
  secondaryAspirationBox->addItem ("None/Unknown");
  secondaryAspirationBox->addItem ("Fortune");
  secondaryAspirationBox->addItem ("Family");
  secondaryAspirationBox->addItem ("Romance");
  secondaryAspirationBox->addItem ("Popularity");
  secondaryAspirationBox->addItem ("Knowledge");
  secondaryAspirationBox->addItem ("Pleasure");
  secondaryAspirationBox->addItem ("Grilled Cheese");
  secondaryAspirationBox->setCurrentIndex (sim->getSecondaryAspirationCode ());
  if (!(epFlags & FREETIME)) secondaryAspirationBox->setEnabled (false);
  QLabel *aspLabel = new QLabel ("Aspiration: ");
  QLabel *slashLabel = new QLabel ("/");
  aspirationLayout = new QHBoxLayout;
  aspirationLayout->addWidget (aspLabel);
  aspirationLayout->addWidget (aspirationBox);
  aspirationLayout->addWidget (slashLabel);
  aspirationLayout->addWidget (secondaryAspirationBox);
  aspirationLayout->addStretch (1);

  semesterBox = new QSpinBox;
  semesterBox->setMinimum (0);
  semesterBox->setMaximum (8);
  semesterBox->setValue (sim->getSemester ());
  if (!(epFlags & UNIVERSITY)) semesterBox->setEnabled (false);
  majorEdit = new QLineEdit (sim->getMajor ());
  majorEdit->setEnabled (false);
  QLabel *cLabel = new QLabel ("Semester in college: ");
  QLabel *mLabel = new QLabel ("Major: ");
  collegeLayout = new QHBoxLayout;
  collegeLayout->addWidget (cLabel);
  collegeLayout->addWidget (semesterBox);
  collegeLayout->addWidget (mLabel);
  collegeLayout->addWidget (majorEdit);

  connect (semesterBox, SIGNAL (valueChanged (int)), this, 
           SLOT (semesterChanged (int)));

  alienBox = new QCheckBox ("Alien");
  alienBox->setChecked (sim->isParanormal (ALIEN));
  zombieBox = new QCheckBox ("Zombie");
  zombieBox->setChecked (sim->isParanormal (ZOMBIE));
  if (!(epFlags & UNIVERSITY)) zombieBox->setEnabled (false);
  vampireBox = new QCheckBox ("Vampire");
  vampireBox->setChecked (sim->isParanormal (VAMPIRE));
  if (!(epFlags & NIGHTLIFE)) vampireBox->setEnabled (false);
  werewolfBox = new QCheckBox ("Werewolf");
  werewolfBox->setChecked (sim->isParanormal (WEREWOLF));
  if (!(epFlags & PETS)) werewolfBox->setEnabled (false);
  servoBox = new QCheckBox ("Servo");
  servoBox->setChecked (sim->isParanormal (SERVO));
  if (!(epFlags & OFB)) servoBox->setEnabled (false);
  plantsimBox = new QCheckBox ("Plantsim");
  plantsimBox->setChecked (sim->isParanormal (PLANTSIM));
  if (!(epFlags & SEASONS)) plantsimBox->setEnabled (false);
  bigfootBox = new QCheckBox ("Bigfoot");
  bigfootBox->setChecked (sim->isParanormal (BIGFOOT));
  if (!(epFlags & BONVOYAGE)) bigfootBox->setEnabled (false);
  witchBox = new QCheckBox ("Witch");
  witchBox->setChecked (sim->isParanormal (WITCH));
  if (!(epFlags & APARTMENTLIFE)) witchBox->setEnabled (false);
  if (witchBox->isChecked ()) alignmentBox->setEnabled (true);

  connect (plantsimBox, SIGNAL (toggled (bool)), this, 
           SLOT (speciesChanged ()));
  connect (witchBox, SIGNAL (toggled (bool)), this,
           SLOT (witchChanged (bool)));

  paranormalColumnOne = new QVBoxLayout;
  paranormalColumnOne->addWidget (alienBox);
  paranormalColumnOne->addWidget (zombieBox);
  paranormalColumnOne->addWidget (vampireBox);
  paranormalColumnOne->addWidget (werewolfBox);
  paranormalColumnTwo = new QVBoxLayout;
  paranormalColumnTwo->addWidget (servoBox);
  paranormalColumnTwo->addWidget (plantsimBox);
  paranormalColumnTwo->addWidget (bigfootBox);
  paranormalColumnTwo->addWidget (witchBox);
  paranormalLayout = new QHBoxLayout;

  paranormalLayout->addLayout (paranormalColumnOne);
  paranormalLayout->addLayout (paranormalColumnTwo);

  submitButton = new QPushButton ("");

  mainLayout = new QVBoxLayout;
  mainLayout->addLayout (nameLayout);
  mainLayout->addLayout (ageLayout);
  mainLayout->addLayout (deathLayout);
  mainLayout->addLayout (speciesLayout);
  mainLayout->addLayout (householdLayout);
  mainLayout->addLayout (aspirationLayout);
  mainLayout->addLayout (collegeLayout);
  mainLayout->addLayout (paranormalLayout);
  mainLayout->addWidget (submitButton);
  mainLayout->setAlignment (submitButton, Qt::AlignCenter);

  setLayout (mainLayout);

  ageChanged (sim->getAge ());
  speciesChanged ();
  semesterChanged (sim->getSemester ());
}

void AddEditSimDialog::ageChanged (int age)  {
  int specials = 0;
  if (plantsimBox->checkState () == Qt::Checked) specials = PLANTSIM;
  if (speciesBox->currentText () == "Large Dog") specials = LARGE_DOG;
  if (speciesBox->currentText () == "Small Dog") specials = SMALL_DOG;
  if (speciesBox->currentText () == "Cat") specials = CAT;
  ageLabel->setText ("(" + ageFormat (age, specials) + ")");

  alienBox->setEnabled (true);
  if (epFlags & OFB) servoBox->setEnabled (true);
  if (epFlags & SEASONS) plantsimBox->setEnabled (true);
  if (epFlags & BONVOYAGE) bigfootBox->setEnabled (true);

  if ((age < TEEN && plantsimBox->checkState () == Qt::Unchecked) ||
      (age < PLANTSIM_ADULT && plantsimBox->checkState () == Qt::Checked))  {
    aspirationBox->setCurrentIndex (GROW_UP);
    aspirationBox->setEnabled (false);
    secondaryAspirationBox->setCurrentIndex (NONE);
    secondaryAspirationBox->setEnabled (false);

    zombieBox->setChecked (false);
    zombieBox->setEnabled (false);
    vampireBox->setChecked (false);
    vampireBox->setEnabled (false);
    werewolfBox->setChecked (false);
    werewolfBox->setEnabled (false);
    witchBox->setChecked (false);
    witchBox->setEnabled (false);

    if (age < TODDLER)
      aspirationBox->setCurrentIndex (NONE);
  }

  else  {
    aspirationBox->setEnabled (true);
    secondaryAspirationBox->setEnabled (true);

    if (epFlags & UNIVERSITY) zombieBox->setEnabled (true);
    if (epFlags & NIGHTLIFE) vampireBox->setEnabled (true);
    if (epFlags & PETS) werewolfBox->setEnabled (true);
    if (epFlags & APARTMENTLIFE) witchBox->setEnabled (true);
  }

  if ((age < ADULT && !plantsimBox->isChecked ()) ||
      (age < PLANTSIM_ADULT && plantsimBox->isChecked ()))  {
    bigfootBox->setChecked (false);
    bigfootBox->setEnabled (false);
  }

  else if (epFlags & BONVOYAGE) bigfootBox->setEnabled (true);

  if (age != ADULT || plantsimBox->isChecked () ||
      speciesBox->currentIndex () != 0)  {
    semesterBox->setValue (0);
    semesterBox->setEnabled (false);
    majorEdit->setEnabled (false);
  }

  else if (epFlags & UNIVERSITY) {
    semesterBox->setEnabled (true);
    majorEdit->setEnabled (true);
  }

  if (speciesBox->currentIndex () != 0)  {
    aspirationBox->setCurrentIndex (NONE);
    aspirationBox->setEnabled (false);
    secondaryAspirationBox->setCurrentIndex (NONE);
    secondaryAspirationBox->setEnabled (false);

    alienBox->setChecked (false);
    alienBox->setEnabled (false);
    zombieBox->setChecked (false);
    zombieBox->setEnabled (false);
    vampireBox->setChecked (false);
    vampireBox->setEnabled (false);
    werewolfBox->setChecked (false);
    werewolfBox->setEnabled (false);
    servoBox->setChecked (false);
    servoBox->setEnabled (false);
    plantsimBox->setChecked (false);
    plantsimBox->setEnabled (false);
    bigfootBox->setChecked (false);
    bigfootBox->setEnabled (false);
    witchBox->setChecked (false);
    witchBox->setEnabled (false);
  }
}

void AddEditSimDialog::speciesChanged ()  {
  ageChanged (ageBox->value ());
}

void AddEditSimDialog::semesterChanged (int semester)  {
  if (semester > 0 && (epFlags & UNIVERSITY)) majorEdit->setEnabled (true);
  else majorEdit->setEnabled (false);
}

void AddEditSimDialog::witchChanged (bool w)  {
  if (w) alignmentBox->setEnabled (true);
  else alignmentBox->setEnabled (false);
}

QString AddEditSimDialog::ageFormat (int age, int special)  {
  QString text = "";

  if (special == PLANTSIM)  {
    if (age >= PLANTSIM_ELDER)  {
      text.setNum (age);
      return text + " days old";
    }

    if (age >= PLANTSIM_ADULT)  {
      text.setNum (PLANTSIM_ELDER - age);
      return text + " days until Elder";
    }

    else  {
      text.setNum (PLANTSIM_ADULT - age);
      return text + " days until Adult";
    }
  }

  else if (special == LARGE_DOG)  {
    if (age >= ELDER_LARGE_DOG)  {
      text.setNum (age);
      return text + " days old";
    }

    if (age >= ADULT_DOG)  {
      text.setNum (ELDER_LARGE_DOG - age);
      return text + " days until Elder";
    }

    else {
      text.setNum (ADULT_DOG - age);
      return text + " days until Adult";
    }
  }

  else if (special == SMALL_DOG)  {
    if (age >= ELDER_SMALL_DOG)  {
      text.setNum (age);
      return text + " days old";
    }

    if (age >= ADULT_DOG)  {
      text.setNum (ELDER_SMALL_DOG - age);
      return text + " days until Elder";
    }

    else  {
      text.setNum (ADULT_DOG - age);
      return text + " days until Adult";
    }
  }

  else if (special == CAT)  {
    if (age >= ELDER_CAT)  {
      text.setNum (age);
      return text + " days old";
    }

    if (age >= ADULT_CAT)  {
      text.setNum (ELDER_CAT - age);
      return text + " days until Elder";
    }

    else  {
      text.setNum (ADULT_CAT - age);
      return text + " days until Adult";
    }
  }

  else  {
    if (age >= ELDER)  {
      text.setNum (age - 5);
      return text + " days old";
    }

    if (age >= ADULT)  {
      text.setNum (ELDER - age);
      return text + " days until Elder";
    }

    if (age >= TEEN)  {
      text.setNum (ADULT - age);
      return text + " days until Adult";
    }

    if (age >= CHILD)  {
      text.setNum (TEEN - age);
      return text + " days until Teen";
    }

    if (age >= TODDLER)  {
      text.setNum (CHILD - age);
      return text + " days until Child";
    }

    else  {
      text.setNum (TODDLER - age);
      return text + " days until Toddler";
    }
  }
}
