#include <QString>
#include <QLabel>
#include <QComboBox>
#include <QSpinBox>
#include <QLineEdit>
#include <QPushButton>
#include <QHBoxLayout>
#include <QVBoxLayout>

#include "const.h"
#include "aspirationdisplay.h"

AspirationDisplay::AspirationDisplay (int flags)  {
  epFlags = flags;

  aspirationViewLabel = new QLabel ("");
  if (flags & FREETIME)
    aspirationViewLabel->setText ((QString)"Primary Aspiraton: <b>None</b>. " +
                                 (QString)"Secondary Aspiration: <b>None</b>.");
  else aspirationViewLabel->setText ("Aspiration: <b>None</b>.");
  aspirationViewLayout = new QHBoxLayout;
  aspirationViewLayout->addWidget (aspirationViewLabel);

  ltwViewLabel = new QLabel ("Lifetime Wants:");
  editButton = new QPushButton ("Edit");
  editButton->setMaximumSize (editButton->sizeHint ());
  ltwTopViewLayout = new QHBoxLayout;
  ltwTopViewLayout->addWidget (ltwViewLabel);
  ltwTopViewLayout->addWidget (editButton);

  ltwViewLayout = new QVBoxLayout;
  ltwViewLayout->addLayout (ltwTopViewLayout);

  connect (editButton, SIGNAL (clicked ()), this, SLOT (changeToEditPage ()));

  mainViewLayout = new QVBoxLayout;
  mainViewLayout->addLayout (aspirationViewLayout);
  mainViewLayout->addLayout (ltwViewLayout);

  viewPage = new QWidget;
  viewPage->setLayout (mainViewLayout);

  primaryAspirationLabel = new QLabel ("");
  if (flags & FREETIME)
    primaryAspirationLabel->setText ("Primary Aspiration:");
  else primaryAspirationLabel->setText ("Aspiration:");
  secondaryAspirationLabel = new QLabel ("SecondaryAspiration:");
  primaryAspirationBox = new QComboBox;
  primaryAspirationBox->addItem ("None/Unknown");
  primaryAspirationBox->addItem ("Fortune");
  primaryAspirationBox->addItem ("Family");
  primaryAspirationBox->addItem ("Romance");
  primaryAspirationBox->addItem ("Popularity");
  primaryAspirationBox->addItem ("Knowledge");
  primaryAspirationBox->addItem ("Pleasure");
  primaryAspirationBox->addItem ("Grilled Cheese");
  secondaryAspirationBox = new QComboBox;
  secondaryAspirationBox->addItem ("None/Unknown");
  secondaryAspirationBox->addItem ("Fortune");
  secondaryAspirationBox->addItem ("Family");
  secondaryAspirationBox->addItem ("Romance");
  secondaryAspirationBox->addItem ("Popularity");
  secondaryAspirationBox->addItem ("Knowledge");
  secondaryAspirationBox->addItem ("Pleasure");
  secondaryAspirationBox->addItem ("Grilled Cheese");

  aspirationEditLayout = new QHBoxLayout;
  aspirationEditLayout->addWidget (primaryAspirationLabel);
  aspirationEditLayout->addWidget (primaryAspirationBox);
  if (flags & FREETIME)  {
    aspirationEditLayout->addWidget (secondaryAspirationLabel);
    aspirationEditLayout->addWidget (secondaryAspirationBox);
  }

  ltwEditLabel = new QLabel ("Lifetime Wants:");
  addLTWButton = new QPushButton ("Add New Lifetime Want");
  deleteLastLTWButton = new QPushButton ("Delete Last Lifetime Want");
  ltwTopEditLayout = new QHBoxLayout;
  ltwTopEditLayout->addWidget (ltwEditLabel);
  ltwTopEditLayout->addWidget (addLTWButton);
  ltwTopEditLayout->addWidget (deleteLastLTWButton);

  connect (addLTWButton, SIGNAL (clicked ()), this, SLOT (addLTW ()));
  connect (deleteLastLTWButton, SIGNAL (clicked ()), this, 
           SLOT (deleteLastLTW ()));

  ltwEditLayout = new QVBoxLayout;
  ltwEditLayout->addLayout (ltwTopEditLayout);

  centralEditLayout = new QVBoxLayout;
  centralEditLayout->addLayout (aspirationEditLayout);
  centralEditLayout->addLayout (ltwEditLayout);

  viewButton = new QPushButton ("Save and View");
  deleteButton = new QPushButton ("Delete");
  deleteButton->setEnabled (false);
  buttonLayout = new QVBoxLayout;
  buttonLayout->addWidget (viewButton);
  buttonLayout->addWidget (deleteButton);

  connect (viewButton, SIGNAL (clicked ()), this, SLOT (changeToViewPage ()));
  connect (deleteButton, SIGNAL (clicked ()), this, SLOT (deleteMe ()));

  mainEditLayout = new QHBoxLayout;
  mainEditLayout->addLayout (centralEditLayout);
  mainEditLayout->addLayout (buttonLayout);

  editPage = new QWidget;
  editPage->setLayout (mainEditLayout);

  addWidget (viewPage);
  addWidget (editPage);
}

void AspirationDisplay::setEPFlags (int flags)  {
  epFlags = flags;
}

void AspirationDisplay::setAspiration (int asp, int sasp)  {
  if (asp > GRILLED_CHEESE || sasp > GRILLED_CHEESE)
    return;

  if (asp < 0) asp = 0;
  if (sasp < 0) sasp = 0;

  QString aspName = "";

  switch (asp)  {
    case NONE: aspName = "None"; break;
    case FORTUNE: aspName = "Fortune"; break;
    case FAMILY: aspName = "Family"; break;
    case ROMANCE: aspName = "Romance"; break;
    case POPULARITY: aspName = "Popularity"; break;
    case KNOWLEDGE: aspName = "Knowledge"; break;
    case PLEASURE: aspName = "Pleasure"; break;
    case GRILLED_CHEESE: aspName = "Grilled Cheese"; break;
    default: break;
  }

  QString saspName = "";

  switch (sasp)  {
    case NONE: saspName = "None"; break;
    case FORTUNE: saspName = "Fortune"; break;
    case FAMILY: saspName = "Family"; break;
    case ROMANCE: saspName = "Romance"; break;
    case POPULARITY: saspName = "Popularity"; break;
    case KNOWLEDGE: saspName = "Knowledge"; break;
    case PLEASURE: saspName = "Pleasure"; break;
    case GRILLED_CHEESE: saspName = "Grilled Cheese"; break;
    default: break;
  }

  if (epFlags & FREETIME)
    aspirationViewLabel->setText ("Primary Aspiration: <b>" + aspName + 
                                  "</b>. Secondary Aspiration: <b>" + saspName 
                                  + "</b>.");
  else aspirationViewLabel->setText ("Aspiration: <b>" + aspName + "</b>.");
  primaryAspirationBox->setCurrentIndex (asp);
  secondaryAspirationBox->setCurrentIndex (sasp);
}

void AspirationDisplay::addLTW (QString name, int done, int steps)  {
  if (done > steps)  {
    int temp = done;
    done = steps;
    steps = temp;
  }

  int next = ltwDescriptionLabel.size ();
  double percentage;
  if (steps == 0) percentage = 0.0;
  else if (steps == done) percentage = 100.0;
  else percentage = (double)done/(double)steps * 100;

  ltwDescriptionLabel.append (new QLabel 
      (name + " (" + QString::number (percentage, 'g', 3) + "\% done)"));
  ltwViewLayout->addWidget (ltwDescriptionLabel[next]);

  ltwDescriptionEdit.append (new QLineEdit (name));
  ltwCompletedStepsBox.append (new QSpinBox);
  ltwStepsBox.append (new QSpinBox);
  ltwCompletedStepsBox[next]->setRange (0, steps);
  ltwCompletedStepsBox[next]->setValue (done);
  ltwStepsBox[next]->setRange (0, 100);
  ltwStepsBox[next]->setValue (steps);

  connect (ltwStepsBox[next], SIGNAL (valueChanged (int)), this,
           SLOT (recalcSteps ()));

  ofLabel.append (new QLabel (" of "));
  stepsDoneLabel.append (new QLabel (" steps done."));

  ltwLineLayout.append (new QHBoxLayout);
  ltwLineLayout[next]->addWidget (ltwDescriptionEdit[next]);
  ltwLineLayout[next]->addWidget (ltwCompletedStepsBox[next]);
  ltwLineLayout[next]->addWidget (ofLabel[next]);
  ltwLineLayout[next]->addWidget (ltwStepsBox[next]);
  ltwLineLayout[next]->addWidget (stepsDoneLabel[next]);

  ltwEditLayout->addLayout (ltwLineLayout[next]);
}

void AspirationDisplay::changeToViewPage ()  {
  setAspiration (primaryAspirationBox->currentIndex (),
                 secondaryAspirationBox->currentIndex ());
  emit updateAspiration (this, primaryAspirationBox->currentIndex (),
                         secondaryAspirationBox->currentIndex ());

  for (int x = 0; x < ltwDescriptionEdit.size (); x++)  {
    if (ltwDescriptionLabel.size () <= x)
      ltwDescriptionLabel.append (new QLabel (""));

    double percentage;
    if (ltwStepsBox[x]->value () == 0) percentage = 0.0;
    else if (ltwStepsBox[x]->value () == ltwCompletedStepsBox[x]->value ())
      percentage = 100.0;
    else percentage = (double)ltwCompletedStepsBox[x]->value ()/
                      (double)ltwStepsBox[x]->value () * 100;
    ltwDescriptionLabel[x]->setText (ltwDescriptionEdit[x]->text () + " (" +
                                     QString::number (percentage, 'g', 3) + 
                                     "\% done)");

    ltwViewLayout->addWidget (ltwDescriptionLabel[x]);

    emit updateLTW (this, x, ltwDescriptionEdit[x]->text (), 
                    ltwCompletedStepsBox[x]->value (), 
                    ltwStepsBox[x]->value ());
  }

  setCurrentWidget (viewPage);
}

void AspirationDisplay::changeToEditPage ()  {
  setCurrentWidget (editPage);
}

void AspirationDisplay::addLTW ()  {
  addLTW ("", 0, 0);
}

void AspirationDisplay::deleteLastLTW ()  {
  int last = ltwLineLayout.size () - 1;

  while (QLayoutItem *child = ltwLineLayout[last]->takeAt (0))
    delete child;

  delete ltwLineLayout[last];
  delete ltwDescriptionLabel[last];
  delete ltwDescriptionEdit[last];
  delete ltwCompletedStepsBox[last];
  delete ltwStepsBox[last];
  delete ofLabel[last];
  delete stepsDoneLabel[last];

  ltwLineLayout.removeLast ();
  ltwDescriptionLabel.removeLast ();
  ltwDescriptionEdit.removeLast ();
  ltwCompletedStepsBox.removeLast ();
  ltwStepsBox.removeLast ();
  ofLabel.removeLast ();
  stepsDoneLabel.removeLast ();
}

void AspirationDisplay::recalcSteps ()  {
  for (int x = 0; x < ltwLineLayout.size (); x++)
    ltwCompletedStepsBox[x]->setRange (0, ltwStepsBox[x]->value ());
}

void AspirationDisplay::deleteMe ()  {
  emit deleteMe (this);
}
