#ifndef ASPIRATIONDISPLAY_H
#define ASPIRATIONDISPLAY_H

#include <QStackedWidget>
#include <QList>

class QString;
class QLabel;
class QComboBox;
class QSpinBox;
class QLineEdit;
class QPushButton;
class QHBoxLayout;
class QVBoxLayout;

class AspirationDisplay : public QStackedWidget  {
  Q_OBJECT

  public:
    AspirationDisplay (int);

    void setEPFlags (int);
    void setAspiration (int, int);
    void addLTW (QString, int, int);

  signals:
    void updateAspiration (AspirationDisplay*, int, int);
    void updateLTW (AspirationDisplay*, int, QString, int, int);
    void deleteMe (AspirationDisplay*);

  private slots:
    void changeToViewPage ();
    void changeToEditPage ();
    void addLTW ();
    void deleteLastLTW ();
    void recalcSteps ();

    void deleteMe ();

  private:
    int epFlags;

    QWidget *viewPage;
    QWidget *editPage;

    QPushButton *editButton;
    QPushButton *viewButton;
    QPushButton *deleteButton;

    QLabel *aspirationViewLabel;
    QLabel *primaryAspirationLabel;
    QLabel *secondaryAspirationLabel;
    QComboBox *primaryAspirationBox;
    QComboBox *secondaryAspirationBox;

    QLabel *ltwViewLabel;
    QLabel *ltwEditLabel;
    QPushButton *addLTWButton;
    QList<QLabel*> ltwDescriptionLabel;
    QList<QLineEdit*> ltwDescriptionEdit;
    QList<QSpinBox*> ltwCompletedStepsBox;
    QList<QSpinBox*> ltwStepsBox;
    QList<QLabel*> ofLabel;
    QList<QLabel*> stepsDoneLabel;
    QPushButton* deleteLastLTWButton;

    QVBoxLayout *buttonLayout;
    QHBoxLayout *aspirationViewLayout;
    QHBoxLayout *aspirationEditLayout;
    QHBoxLayout *ltwTopViewLayout;
    QVBoxLayout *ltwViewLayout;
    QHBoxLayout *ltwTopEditLayout;
    QList<QHBoxLayout*> ltwLineLayout;
    QVBoxLayout *ltwEditLayout;
    QVBoxLayout *mainViewLayout;
    QVBoxLayout *centralEditLayout;
    QHBoxLayout *mainEditLayout;
};

#endif
