#include <QLabel>
#include <QPushButton>
#include <QComboBox>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QGroupBox>
#include <QString>
#include <QStringList>
#include <QMessageBox>

#include "sim.h"
#include "familybrowser.h"

FamilyBrowser::FamilyBrowser (QPointer<Sim> s, QList< QPointer<Sim> > list)
  : QDialog ()
{
  sim = s;
  simList = list;
  sortList (simList);

  setMinimumWidth (500);

  nameLabel = new QLabel ("Immediate Family of " + sim->getFullName () + ":");

  spouseButton = new QPushButton;
  noSpouseLabel = new QLabel ("None");
  setSpouseButton = new QPushButton ("Set Spouse:");
  setSpouseBox = new QComboBox;
  setSpouseBox->setSizeAdjustPolicy (QComboBox::AdjustToContents);

  spouseLayout = new QHBoxLayout;
  QLabel *spouseLabel = new QLabel ("Spouse: ");
  spouseLayout->addWidget (spouseLabel);
  spouseLayout->addWidget (noSpouseLabel);

  addSpouseLayout = new QHBoxLayout;
  addSpouseLayout->addWidget (setSpouseButton);
  addSpouseLayout->addWidget (setSpouseBox);

  parent1Button = new QPushButton;
  parent2Button = new QPushButton;
  noParent1Label = new QLabel ("None");
  noParent2Label = new QLabel ("None");
  setParent1Button = new QPushButton ("Set First Parent:");
  setParent2Button = new QPushButton ("Set Second Parent:");
  setParent1Box = new QComboBox;
  setParent1Box->setSizeAdjustPolicy (QComboBox::AdjustToContents);
  setParent2Box = new QComboBox;
  setParent2Box->setSizeAdjustPolicy (QComboBox::AdjustToContents);

  parent1Layout = new QHBoxLayout;
  QLabel *parent1Label = new QLabel ("First Parent: ");
  parent1Layout->addWidget (parent1Label);
  parent1Layout->addWidget (noParent1Label);

  parent2Layout = new QHBoxLayout;
  QLabel *parent2Label = new QLabel ("Second Parent: ");
  parent2Layout->addWidget (parent2Label);
  parent2Layout->addWidget (noParent2Label);

  addParent1Layout = new QHBoxLayout;
  addParent1Layout->addWidget (setParent1Button);
  addParent1Layout->addWidget (setParent1Box);

  addParent2Layout = new QHBoxLayout;
  addParent2Layout->addWidget (setParent2Button);
  addParent2Layout->addWidget (setParent2Box);

  noChildrenLabel = new QLabel ("None");
  childButton = new QPushButton ("View Child:");
  childrenBox = new QComboBox;
  childrenBox->setSizeAdjustPolicy (QComboBox::AdjustToContents);
  addChildButton = new QPushButton ("Add Child:");
  addChildBox = new QComboBox;
  addChildBox->setSizeAdjustPolicy (QComboBox::AdjustToContents);
  removeChildButton = new QPushButton ("Remove Child:");
  removeChildButton->setEnabled (false);
  removeChildBox = new QComboBox;

  childrenLayout = new QHBoxLayout;
  QLabel *childrenLabel = new QLabel ("Children: ");
  childrenLayout->addWidget (childrenLabel);
  childrenLayout->addWidget (noChildrenLabel);

  addChildLayout = new QHBoxLayout;
  addChildLayout->addWidget (addChildButton);
  addChildLayout->addWidget (addChildBox);

  removeChildLayout = new QHBoxLayout;
  removeChildLayout->addWidget (removeChildButton);
  removeChildLayout->addWidget (removeChildBox);

  noSiblingsLabel = new QLabel ("None");
  siblingButton = new QPushButton ("View Sibling:");
  siblingButton->setMinimumSize (siblingButton->sizeHint ());
  siblingBox = new QComboBox;
  siblingBox->setSizeAdjustPolicy (QComboBox::AdjustToContents);
  siblingBox->setMinimumSize (siblingBox->sizeHint ());
  addSiblingButton = new QPushButton ("Add Full Sibling:");
  addSiblingButton->setMinimumSize (addSiblingButton->sizeHint ());
  addSiblingBox = new QComboBox;
  addSiblingBox->setSizeAdjustPolicy (QComboBox::AdjustToContents);

  siblingLayout = new QHBoxLayout;
  QLabel *siblingLabel = new QLabel ("Full and Partial Siblings: ");
  siblingLabel->setMinimumSize (siblingLabel->sizeHint ());
  siblingLayout->addWidget (siblingLabel);
  siblingLayout->addWidget (noSiblingsLabel);

  addSiblingLayout = new QHBoxLayout;
  addSiblingLayout->addWidget (addSiblingButton);
  addSiblingLayout->addWidget (addSiblingBox);

  closeButton = new QPushButton ("Close and Refresh Family List");

  navigationLayout = new QVBoxLayout;
  navigationLayout->addLayout (spouseLayout);
  navigationLayout->addLayout (parent1Layout);
  navigationLayout->addLayout (parent2Layout);
  navigationLayout->addLayout (childrenLayout);
  navigationLayout->addLayout (siblingLayout);
  navigationGroupBox = new QGroupBox ("Navigation");
  navigationGroupBox->setLayout (navigationLayout);

  editingLayout = new QVBoxLayout;
  editingLayout->addLayout (addSpouseLayout);
  editingLayout->addLayout (addParent1Layout);
  editingLayout->addLayout (addParent2Layout);
  editingLayout->addLayout (addChildLayout);
  editingLayout->addLayout (removeChildLayout);
  editingLayout->addLayout (addSiblingLayout);
  editingGroupBox = new QGroupBox ("Edit Family");
  editingGroupBox->setLayout (editingLayout);

  boxLayout = new QHBoxLayout;
  boxLayout->addWidget (navigationGroupBox);
  boxLayout->addWidget (editingGroupBox);

  mainLayout = new QVBoxLayout;
  mainLayout->addWidget (nameLabel);
  mainLayout->setAlignment (nameLabel, Qt::AlignCenter);
  mainLayout->addLayout (boxLayout);
  mainLayout->addWidget (closeButton);
  mainLayout->setAlignment (closeButton, Qt::AlignCenter);

  setLayout (mainLayout);

  connect (spouseButton, SIGNAL (clicked ()), this, SLOT (viewSpouse ()));
  connect (parent1Button, SIGNAL (clicked ()), this, SLOT (viewParent1 ()));
  connect (parent2Button, SIGNAL (clicked ()), this, SLOT (viewParent2 ()));
  connect (childButton, SIGNAL (clicked ()), this, SLOT (viewChild ()));
  connect (siblingButton, SIGNAL (clicked ()), this, SLOT (viewSibling ()));

  connect (setSpouseButton, SIGNAL (clicked ()), this, SLOT (setSpouse ()));
  connect (setParent1Button, SIGNAL (clicked ()), this, SLOT (setParent1 ()));
  connect (setParent2Button, SIGNAL (clicked ()), this, SLOT (setParent2 ()));
  connect (addChildButton, SIGNAL (clicked ()), this, SLOT (addChild ()));
  connect (removeChildButton, SIGNAL (clicked ()), this, SLOT (removeChild ()));
  connect (addSiblingButton, SIGNAL (clicked ()), this, SLOT (addSibling ()));

  connect (closeButton, SIGNAL (clicked ()), this, SIGNAL (closeAndUpdate ()));
  connect (closeButton, SIGNAL (clicked ()), this, SLOT (close ()));

  updateDisplay ();
}

void FamilyBrowser::setSim (QPointer<Sim> s)  {
  sim = s;
  updateDisplay ();

  navigationGroupBox->setMinimumSize (navigationLayout->sizeHint ());
  editingGroupBox->setMinimumSize (editingGroupBox->sizeHint ());
  setMinimumSize (mainLayout->sizeHint ());

  navigationGroupBox->resize (navigationLayout->sizeHint ());
  editingGroupBox->resize (editingLayout->sizeHint ());
  resize (mainLayout->sizeHint ());
}

void FamilyBrowser::setSimList (QList< QPointer<Sim> > list)  {
  simList = list;
  sortList (simList);
  updateDisplay ();
}

void FamilyBrowser::viewSpouse ()  {
  if (sim->getSpouse ())
    setSim (sim->getSpouse ());
}

void FamilyBrowser::viewParent1 ()  {
  if (sim->getParent1 ())
    setSim (sim->getParent1 ());
}

void FamilyBrowser::viewParent2 ()  {
  if (sim->getParent2 ())
    setSim (sim->getParent2 ());
}

void FamilyBrowser::viewChild ()  {
  if (sim->getChild (childrenBox->currentIndex ()))
    setSim (sim->getChild (childrenBox->currentIndex ()));
}

void FamilyBrowser::viewSibling ()  {
  QPointer<Sim> p1 = sim->getParent1 ();
  QPointer<Sim> p2 = sim->getParent2 ();

  QList< QPointer<Sim> > siblings;

  int currentChild = -1;

  if (!p1.isNull ())  
    do  {
      currentChild++;
      if (p1->getChild (currentChild) == sim) continue;
      siblings.append (p1->getChild (currentChild));
    } while (p1->getChild (currentChild));

  currentChild = -1;

  if (!p2.isNull ())
    do  {
      currentChild++;
      if (p2->getChild (currentChild) == sim ||
          siblings.contains (p2->getChild (currentChild))) continue;
      siblings.append (p2->getChild (currentChild));
    } while (p2->getChild (currentChild));

  sortList (siblings);

  if (siblings.size () > siblingBox->currentIndex ())
    setSim (siblings[siblingBox->currentIndex ()]);
}

void FamilyBrowser::setSpouse ()  {
  if (setSpouseBox->currentIndex () >= simList.size ())
    sim->clearSpouse ();
  else sim->setSpouse (simList[setSpouseBox->currentIndex ()]);

  updateDisplay ();
}

void FamilyBrowser::setParent1 ()  {
  if (setParent1Box->currentIndex () >= simList.size ())
    sim->clearParent1 ();
  else sim->setParents (simList[setParent1Box->currentIndex ()], NULL);

  updateDisplay ();
}

void FamilyBrowser::setParent2 ()  {
  if (setParent2Box->currentIndex () >= simList.size ())
    sim->clearParent2 ();
  else sim->setParents (NULL, simList[setParent2Box->currentIndex ()]);

  updateDisplay ();
}

void FamilyBrowser::addChild ()  {
  sim->addChild (simList[addChildBox->currentIndex ()]);

  updateDisplay ();
}

void FamilyBrowser::removeChild ()  {
  sim->clearChild (removeChildBox->currentIndex ());

  updateDisplay ();
}

void FamilyBrowser::sortList (QList< QPointer<Sim> > &list)  {
  for (int x = 0; x < list.size (); x++)  {
    int smallest = x;

    for (int y = x + 1; y < list.size (); y++)
      if (!(list[smallest].isNull ()) && !(list[y].isNull ()))
        if (QString::localeAwareCompare (list[smallest]->getFullName (),
                                         list[y]->getFullName ()) > 0)
          smallest = y;

    list.swap (smallest, x);
  }
}

void FamilyBrowser::addSibling ()  {
  simList[addSiblingBox->currentIndex ()]->setParents (sim->getParent1 (),
                                                       sim->getParent2 ());

  updateDisplay ();
}

void FamilyBrowser::updateDisplay ()  {
  nameLabel->setText ("Immediate Family of " + sim->getFullName () + ":");

  if (!(sim->getSpouse ().isNull ()))  {
    spouseButton->setText (sim->getSpouse ()->getFullName ());
    spouseLayout->removeWidget (noSpouseLabel);
    noSpouseLabel->hide ();
    spouseLayout->addWidget (spouseButton);
    spouseButton->show ();
  }

  else  {
    spouseButton->setText ("");
    spouseLayout->removeWidget (spouseButton);
    spouseButton->hide ();
    spouseLayout->addWidget (noSpouseLabel);
    noSpouseLabel->show ();
  }

  if (!(sim->getParent1 ().isNull ()))  {
    parent1Button->setText (sim->getParent1 ()->getFullName ());
    parent1Layout->removeWidget (noParent1Label);
    noParent1Label->hide ();
    parent1Layout->addWidget (parent1Button);
    parent1Button->show ();
  }

  else  {
    parent1Button->setText ("");
    parent1Layout->removeWidget (parent1Button);
    parent1Button->hide ();
    parent1Layout->addWidget (noParent1Label);
    noParent1Label->show ();
  }

  if (!(sim->getParent2 ().isNull ()))  {
    parent2Button->setText (sim->getParent2 ()->getFullName ());
    parent2Layout->removeWidget (noParent2Label);
    noParent2Label->hide ();
    parent2Layout->addWidget (parent2Button);
    parent2Button->show ();
  }

  else  {
    parent2Button->setText ("");
    parent2Layout->removeWidget (parent2Button);
    parent2Button->hide ();
    parent2Layout->addWidget (noParent2Label);
    noParent2Label->show ();
  }

  childrenBox->clear ();
  removeChildBox->clear ();

  if (sim->getChild (0).isNull ())  {
    childrenLayout->removeWidget (childButton);
    childButton->hide ();
    childrenLayout->removeWidget (childrenBox);
    childrenBox->hide ();
    childrenLayout->addWidget (noChildrenLabel);
    noChildrenLabel->show ();
    removeChildButton->setEnabled (false);
  }

  else  {
    childrenLayout->removeWidget (noChildrenLabel);
    noChildrenLabel->hide ();
    childrenLayout->addWidget (childButton);
    childButton->show ();

    QStringList childNames;
    int currentChild = 0;

    do
      childNames << sim->getChild (currentChild++)->getFullName ();
    while (sim->getChild (currentChild));

    childrenBox->addItems (childNames);
    removeChildBox->addItems (childNames);
    removeChildButton->setEnabled (true);

    childrenLayout->addWidget (childrenBox);
    childrenBox->show ();
  }

  QPointer<Sim> p1 = sim->getParent1 ();
  QPointer<Sim> p2 = sim->getParent2 ();

  QList< QPointer<Sim> > siblings;

  siblingBox->clear ();

  int currentChild = -1;
    
  if (!(p1.isNull ()))
    do  {
      currentChild++;
      if (p1->getChild (currentChild) == sim) continue;
      siblings.append (p1->getChild (currentChild));
    } while (p1->getChild (currentChild));

  currentChild = -1;

  if (!(p2.isNull ()))
    do  {
      currentChild++;
      if (p2->getChild (currentChild) == sim) continue;
      if (siblings.contains (p2->getChild (currentChild))) continue;
      siblings.append (p2->getChild (currentChild));
    } while (p2->getChild (currentChild));

  sortList (siblings);

  if (siblings.isEmpty ())  {
    siblingLayout->removeWidget (siblingButton);
    siblingButton->hide ();
    siblingLayout->removeWidget (siblingBox);
    siblingBox->hide ();
    siblingLayout->addWidget (noSiblingsLabel);
    noSiblingsLabel->show ();
  }

  else  {
    siblingLayout->removeWidget (noSiblingsLabel);
    noSiblingsLabel->hide ();
    siblingLayout->addWidget (siblingButton);
    siblingButton->show ();
    siblingLayout->addWidget (siblingBox);
    siblingBox->show ();

    QStringList sibNames;
    for (int x = 0; x < siblings.size (); x++)
      if (!((siblings[x]).isNull ()))
        sibNames.append (siblings[x]->getFullName ());

    siblingBox->addItems (sibNames);
  }

  QStringList simNames;
  for (int x = 0; x < simList.size (); x++)
    if (!(simList[x].isNull ()))
      simNames.append (simList[x]->getFullName ());
  simNames.append ("None");

  setSpouseBox->clear ();
  setSpouseBox->addItems (simNames);
  setParent1Box->clear ();
  setParent1Box->addItems (simNames);
  setParent2Box->clear ();
  setParent2Box->addItems (simNames);

  simNames.removeLast ();

  addChildBox->clear ();
  addChildBox->addItems (simNames);
  addSiblingBox->clear ();
  addSiblingBox->addItems (simNames);

  navigationGroupBox->setMinimumSize (navigationLayout->sizeHint ());
  editingGroupBox->setMinimumSize (editingGroupBox->sizeHint ());
  setMinimumSize (mainLayout->sizeHint ());

  navigationGroupBox->resize (navigationLayout->sizeHint ());
  editingGroupBox->resize (editingLayout->sizeHint ());
  resize (mainLayout->sizeHint ());
}
