#include <QCheckBox>
#include <QGroupBox>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QGridLayout>

#include "const.h"
#include "filterdialog.h"
#include "simproxymodel.h"

FilterDialog::FilterDialog ()  {
  // Living Group
  aliveBox = new QCheckBox ("Living Sims");
  aliveBox->setChecked (true);
  connect (aliveBox, SIGNAL (toggled (bool)), this, 
           SIGNAL (setAliveFilter (bool)));

  deadBox = new QCheckBox ("Dead Sims");
  deadBox->setChecked (true);
  connect (deadBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setDeadFilter (bool)));

  livingLayout = new QHBoxLayout;
  livingLayout->addWidget (aliveBox);
  livingLayout->addWidget (deadBox);
  livingGroup = new QGroupBox ("Life");
  livingGroup->setLayout (livingLayout);

  // Type Group
  playableBox = new QCheckBox ("Playable Sims");
  playableBox->setChecked (true);
  connect (playableBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setPlayableFilter (bool)));

  townieBox = new QCheckBox ("Townies");
  townieBox->setChecked (true);
  connect (townieBox, SIGNAL (toggled (bool)), this, 
           SIGNAL (setTownieFilter (bool)));

  npcBox = new QCheckBox ("Service NPCs");
  npcBox->setChecked (true);
  connect (npcBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setNPCFilter (bool)));

  runawayBox = new QCheckBox ("Runaways");
  runawayBox->setChecked (true);
  connect (runawayBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setRunawayFilter (bool)));

  adoptionBox = new QCheckBox ("Adoptables");
  adoptionBox->setChecked (true);
  connect (adoptionBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setAdoptionFilter (bool)));

  downtownieBox = new QCheckBox ("Downtownies");
  downtownieBox->setChecked (true);
  connect (downtownieBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setDowntownieFilter (bool)));

  gardenClubBox = new QCheckBox ("Garden Club Members");
  gardenClubBox->setChecked (true);
  connect (gardenClubBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setGardenClubFilter (bool)));

  islandBox = new QCheckBox ("Island Locals");
  islandBox->setChecked (true);
  connect (islandBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setIslandFilter (bool)));

  mountainBox = new QCheckBox ("Mountain Locals");
  mountainBox->setChecked (true);
  connect (mountainBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setMountainFilter (bool)));

  asianBox = new QCheckBox ("Asian Locals");
  asianBox->setChecked (true);
  connect (asianBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setAsianFilter (bool)));

  touristBox = new QCheckBox ("Tourists");
  touristBox->setChecked (true);
  connect (touristBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setTouristFiler (bool)));

  hobbyBox = new QCheckBox ("Hobby Mavens");
  hobbyBox->setChecked (true);
  connect (hobbyBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setHobbyFilter (bool)));

  socialiteBox = new QCheckBox ("Socialites");
  socialiteBox->setChecked (true);
  connect (socialiteBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setSocialiteFilter (bool)));

  bohemianBox = new QCheckBox ("Bohemians");
  bohemianBox->setChecked (true);
  connect (bohemianBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setBohemianFilter (bool)));

  jockBox = new QCheckBox ("Jocks");
  jockBox->setChecked (true);
  connect (jockBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setJockFilter (bool)));

  techieBox = new QCheckBox ("Techies");
  techieBox->setChecked (true);
  connect (techieBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setTechieFilter (bool)));

  gearheadBox = new QCheckBox ("Gearheads");
  gearheadBox->setChecked (true);
  connect (gearheadBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setGearheadFilter (bool)));

  typeLayout = new QGridLayout;
  typeLayout->addWidget (playableBox, 0, 0);
  typeLayout->addWidget (townieBox, 0, 1);
  typeLayout->addWidget (npcBox, 0, 2);
  typeLayout->addWidget (runawayBox, 1, 0);
  typeLayout->addWidget (adoptionBox, 1, 1);
  typeLayout->addWidget (downtownieBox, 1, 2);
  typeLayout->addWidget (gardenClubBox, 2, 0);
  typeLayout->addWidget (islandBox, 2, 1);
  typeLayout->addWidget (mountainBox, 2, 2);
  typeLayout->addWidget (asianBox, 3, 0);
  typeLayout->addWidget (touristBox, 3, 1);
  typeLayout->addWidget (hobbyBox, 3, 2);
  typeLayout->addWidget (socialiteBox, 4, 0);
  typeLayout->addWidget (bohemianBox, 4, 1);
  typeLayout->addWidget (jockBox, 4, 2);
  typeLayout->addWidget (techieBox, 5, 0);
  typeLayout->addWidget (gearheadBox, 5, 1);
  typeGroup = new QGroupBox ("Type");
  typeGroup->setLayout (typeLayout);

  // Species Group
  humanBox = new QCheckBox ("Humans");
  humanBox->setChecked (true);
  connect (humanBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setHumanFilter (bool)));

  largeDogBox = new QCheckBox ("Large Dogs");
  largeDogBox->setChecked (true);
  connect (largeDogBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setLargeDogFilter (bool)));

  smallDogBox = new QCheckBox ("Small Dogs");
  smallDogBox->setChecked (true);
  connect (smallDogBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setSmallDogFilter (bool)));

  catBox = new QCheckBox ("Cats");
  catBox->setChecked (true);
  connect (catBox, SIGNAL (toggled (bool)), this, 
           SIGNAL (setCatFilter (bool)));

  speciesLayout = new QGridLayout;
  speciesLayout->addWidget (humanBox, 0, 0);
  speciesLayout->addWidget (largeDogBox, 0, 1);
  speciesLayout->addWidget (smallDogBox, 1, 0);
  speciesLayout->addWidget (catBox, 1, 1);
  speciesGroup = new QGroupBox ("Species");
  speciesGroup->setLayout (speciesLayout);

  // Gender Group
  maleBox = new QCheckBox ("Males");
  maleBox->setChecked (true);
  connect (maleBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setMaleFilter (bool)));

  femaleBox = new QCheckBox ("Females");
  femaleBox->setChecked (true);
  connect (femaleBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setFemaleFilter (bool)));

  genderLayout = new QHBoxLayout;
  genderLayout->addWidget (maleBox);
  genderLayout->addWidget (femaleBox);
  genderGroup = new QGroupBox ("Gender");
  genderGroup->setLayout (genderLayout);

  // Age Group
  babyBox = new QCheckBox ("Babies");
  babyBox->setChecked (true);
  connect (babyBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setBabyFilter (bool)));

  toddlerBox = new QCheckBox ("Toddlers");
  toddlerBox->setChecked (true);
  connect (toddlerBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setToddlerFilter (bool)));

  childBox = new QCheckBox ("Children");
  childBox->setChecked (true);
  connect (childBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setChildFilter (bool)));

  teenBox = new QCheckBox ("Teens");
  teenBox->setChecked (true);
  connect (teenBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setTeenFilter (bool)));

  yaBox = new QCheckBox ("Young Adults");
  yaBox->setChecked (true);
  connect (yaBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setYAFilter (bool)));

  adultBox = new QCheckBox ("Adults");
  adultBox->setChecked (true);
  connect (adultBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setAdultFilter (bool)));

  elderBox = new QCheckBox ("Elders");
  elderBox->setChecked (true);
  connect (elderBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setElderFilter (bool)));

  ageLayout = new QGridLayout;
  ageLayout->addWidget (babyBox, 0, 0);
  ageLayout->addWidget (toddlerBox, 0, 1);
  ageLayout->addWidget (childBox, 1, 0);
  ageLayout->addWidget (teenBox, 1, 1);
  ageLayout->addWidget (yaBox, 2, 0);
  ageLayout->addWidget (adultBox, 2, 1);
  ageLayout->addWidget (elderBox, 3, 0);
  ageGroup = new QGroupBox ("Life Stage");
  ageGroup->setLayout (ageLayout);

  // Aspiration Group
  noneBox = new QCheckBox ("None/Unknown");
  noneBox->setChecked (true);
  connect (noneBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setNoneFilter (bool)));

  growUpBox = new QCheckBox ("Grow Up");
  growUpBox->setChecked (true);
  connect (growUpBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setGrowUpFilter (bool)));

  fortuneBox = new QCheckBox ("Fortune");
  fortuneBox->setChecked (true);
  connect (fortuneBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setFortuneFilter (bool)));

  familyBox = new QCheckBox ("Family");
  familyBox->setChecked (true);
  connect (familyBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setFamilyFilter (bool)));

  popularityBox = new QCheckBox ("Popularity");
  popularityBox->setChecked (true);
  connect (popularityBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setPopularityFilter (bool)));

  romanceBox = new QCheckBox ("Romance");
  romanceBox->setChecked (true);
  connect (romanceBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setRomanceFilter (bool)));

  knowledgeBox = new QCheckBox ("Knowledge");
  knowledgeBox->setChecked (true);
  connect (knowledgeBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setKnowledgeFilter (bool)));

  pleasureBox = new QCheckBox ("Pleasure");
  pleasureBox->setChecked (true);
  connect (pleasureBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setPleasureFilter (bool)));

  grilledCheeseBox = new QCheckBox ("Grilled Cheese");
  grilledCheeseBox->setChecked (true);
  connect (grilledCheeseBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setGrilledCheeseFilter (bool)));

  aspirationLayout = new QGridLayout;
  aspirationLayout->addWidget (noneBox, 0, 0);
  aspirationLayout->addWidget (growUpBox, 0, 1);
  aspirationLayout->addWidget (fortuneBox, 0, 2);
  aspirationLayout->addWidget (familyBox, 1, 0);
  aspirationLayout->addWidget (popularityBox, 1, 1);
  aspirationLayout->addWidget (romanceBox, 1, 2);
  aspirationLayout->addWidget (knowledgeBox, 2, 0);
  aspirationLayout->addWidget (pleasureBox, 2, 1);
  aspirationLayout->addWidget (grilledCheeseBox, 2, 2);
  aspirationGroup = new QGroupBox ("Aspiration");
  aspirationGroup->setLayout (aspirationLayout);

  // Paranormal Group
  normalBox = new QCheckBox ("Non-Paranormal");
  normalBox->setChecked (true);
  connect (normalBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setNormalFilter (bool)));

  alienBox = new QCheckBox ("Aliens");
  alienBox->setChecked (true);
  connect (alienBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setAlienFilter (bool)));

  zombieBox = new QCheckBox ("Zombies");
  zombieBox->setChecked (true);
  connect (zombieBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setZombieFilter (bool)));

  vampireBox = new QCheckBox ("Vampires");
  vampireBox->setChecked (true);
  connect (vampireBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setVampireFilter (bool)));

  werewolfBox = new QCheckBox ("Werewolves");
  werewolfBox->setChecked (true);
  connect (werewolfBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setWerewolfFilter (bool)));

  servoBox = new QCheckBox ("Servos");
  servoBox->setChecked (true);
  connect (servoBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setServoFilter (bool)));

  plantsimBox = new QCheckBox ("Plantsims");
  plantsimBox->setChecked (true);
  connect (plantsimBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setPlantsimFilter (bool)));

  bigfootBox = new QCheckBox ("Bigfoots");
  bigfootBox->setChecked (true);
  connect (bigfootBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setBigfootFilter (bool)));

  witchBox = new QCheckBox ("Witches");
  witchBox->setChecked (true);
  connect (witchBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setWitchFilter (bool)));

  paranormalLayout = new QGridLayout;
  paranormalLayout->addWidget (normalBox, 0, 0);
  paranormalLayout->addWidget (alienBox, 0, 1);
  paranormalLayout->addWidget (zombieBox, 0, 2);
  paranormalLayout->addWidget (vampireBox, 1, 0);
  paranormalLayout->addWidget (werewolfBox, 1, 1);
  paranormalLayout->addWidget (servoBox, 1, 2);
  paranormalLayout->addWidget (plantsimBox, 2, 0);
  paranormalLayout->addWidget (bigfootBox, 2, 1);
  paranormalLayout->addWidget (witchBox, 2, 2);
  paranormalGroup = new QGroupBox ("Paranormal");
  paranormalGroup->setLayout (paranormalLayout);

  mainLayout = new QVBoxLayout;
  mainLayout->addWidget (livingGroup);
  mainLayout->addWidget (typeGroup);
  mainLayout->addWidget (speciesGroup);
  mainLayout->addWidget (genderGroup);
  mainLayout->addWidget (ageGroup);
  mainLayout->addWidget (aspirationGroup);
  mainLayout->addWidget (paranormalGroup);

  setLayout (mainLayout);
}
