#include <QStandardItemModel>
#include <QAbstractItemModel>
#include <QString>
#include <QStringList>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QSortFilterProxyModel>
#include <QModelIndex>
#include <QTreeView>
#include <QLabel>
#include <QPushButton>
#include <QScrollArea>
#include <QCheckBox>

#include "householdpage.h"
#include "edithouseholddialog.h"

HouseholdPage::HouseholdPage ()  {
  editHouseholdDialog = NULL;

  proxyModel = new QSortFilterProxyModel (this);
  proxyModel->setDynamicSortFilter (true);

  householdView = new QTreeView;
  householdView->setRootIsDecorated (false);
  householdView->setAlternatingRowColors (true);
  householdView->setModel (proxyModel);
  householdView->setSortingEnabled (true);
  householdView->setSelectionBehavior (QAbstractItemView::SelectRows);

  editNameButton = new QPushButton ("Edit Selected Household");
  editNameButton->setMaximumSize (editNameButton->sizeHint ());
  editNameButton->setEnabled (false);
  alreadyPlayedBox = new QCheckBox 
                          ("Current household has been played this rotation.");
  alreadyPlayedBox->setEnabled (false);

  topLayout = new QHBoxLayout ();
  topLayout->addWidget (editNameButton);
  topLayout->addWidget (alreadyPlayedBox);
  topLayout->setAlignment (alreadyPlayedBox, Qt::AlignRight);

  displayArea = new QScrollArea ();
  displayLabel = new QLabel ();
  displayArea->setWidget (displayLabel);
  displayArea->show ();
  displayLabel->show ();

  mainLayout = new QVBoxLayout ();
  mainLayout->addLayout (topLayout);
  mainLayout->addWidget (householdView);
  mainLayout->addWidget (displayArea);

  setLayout (mainLayout);

  connect (householdView->selectionModel (), 
             SIGNAL (currentChanged (const QModelIndex&, const QModelIndex&)),
           this, SLOT (updateDisplay ()));

  connect (alreadyPlayedBox, SIGNAL (stateChanged (int)), this,
           SLOT (updateAlreadyPlayed (int)));

  connect (editNameButton, SIGNAL (clicked ()), this,
           SLOT (createEditHouseholdDialog ()));
}

void HouseholdPage::setSourceModel (QAbstractItemModel *model)  {
  proxyModel->setSourceModel (model);
  proxyModel->sort (0);

  householdView->resizeColumnToContents (0);
  householdView->resizeColumnToContents (1);
}

void HouseholdPage::displayNames (QStringList list)  {
  QString text = "";

  for (int x = 0; x < list.size (); x++)  {
    text += list[x];
    if (x < (list.size () - 1)) text += "\n";
  }

  displayLabel->setText (text);
  displayLabel->resize (displayLabel->sizeHint ());
}

void HouseholdPage::setNotPlayed ()  {
  for (int x = 0; x < proxyModel->rowCount (); x++)  {
    QString currentHouse = proxyModel->index (x, 0).data ().toString ();

    if (currentHouse != "Runaways" && currentHouse != "Adoption Queue" &&
        currentHouse != "Townies and Dormies" && currentHouse != "Downtown" &&
        currentHouse != "Garden Club" && currentHouse != "Service NPCs" &&
        currentHouse != "Island Locals" && currentHouse != "Mountain Locals" &&
        currentHouse != "Asian Locals" && currentHouse != "Tourists" &&
        currentHouse != "Socialites" && currentHouse != "Gearheads" &&
        currentHouse != "Techies" && currentHouse != "Jocks" &&
        currentHouse != "Bohemians" && currentHouse != "Hobby Mavens")
      proxyModel->setData (proxyModel->index (x, 1), "No", Qt::DisplayRole);
  }
}

void HouseholdPage::reset ()  {
  editNameButton->setEnabled (false);
  alreadyPlayedBox->setEnabled (false);
  displayLabel->setText ("");
  householdView->selectionModel ()->reset ();
}

QString HouseholdPage::getCSV ()  {
  QString csv = "";

  for (int x = 0; x < proxyModel->rowCount (); x++)  {
    csv += proxyModel->index (x, 0).data ().toString ();
    csv += ",";
    csv += proxyModel->index (x, 1).data ().toString ();

    if (x < (proxyModel->rowCount () - 1)) csv += "\r\n";
  }

  return csv;
}

void HouseholdPage::readCSV (QString csv)  {
  QStringList list = csv.split (',');

  for (int x = 0; x < proxyModel->rowCount (); x++)
    if (proxyModel->index (x, 0).data ().toString () == list[0])
      proxyModel->setData (proxyModel->index (x, 1), list[1], Qt::DisplayRole);
}

QAbstractItemModel *HouseholdPage::sourceModel ()  {
  return proxyModel->sourceModel ();
}

void HouseholdPage::updateDisplay ()  {
  editNameButton->setEnabled (true);
  alreadyPlayedBox->setEnabled (true);

  QAbstractItemModel *model = proxyModel->sourceModel ();
  QModelIndex current = householdView->selectionModel ()->currentIndex ();
  QModelIndex source = proxyModel->mapToSource (current);
  int currentRow = source.row ();

  QString currentPlayed = model->index (currentRow, 1).data ().toString ();
  alreadyPlayedBox->setChecked (currentPlayed == "Yes");

  QString currentHouse = model->index (currentRow, 0).data ().toString ();

  if (currentHouse == "Runaways" || currentHouse == "Adoption Queue" ||
      currentHouse == "Townies and Dormies" || currentHouse == "Downtown" || 
      currentHouse == "Garden Club" || currentHouse == "Service NPCs" || 
      currentHouse == "Island Locals" || currentHouse == "Mountain Locals" || 
      currentHouse == "Asian Locals" || currentHouse == "Tourists" || 
      currentHouse == "Socialites" || currentHouse == "Gearheads" || 
      currentHouse == "Techies" || currentHouse == "Jocks" || 
      currentHouse == "Bohemians" || currentHouse == "Hobby Mavens")  {
    editNameButton->setEnabled (false);
    alreadyPlayedBox->setEnabled (false);
    model->setData (model->index (currentRow, 1), "", Qt::DisplayRole);
  }

  emit showHousehold (currentHouse);
}

void HouseholdPage::updateAlreadyPlayed (int state)  {
  QAbstractItemModel *model = proxyModel->sourceModel ();
  QModelIndex current = householdView->selectionModel ()->currentIndex ();
  QModelIndex source = proxyModel->mapToSource (current);
  int currentRow = source.row ();

  if (state == Qt::Checked)
    model->setData (model->index (currentRow, 1), "Yes", Qt::DisplayRole);
  else
    model->setData (model->index (currentRow, 1), "No", Qt::DisplayRole);
}

void HouseholdPage::createEditHouseholdDialog ()  {
  QAbstractItemModel *model = proxyModel->sourceModel ();
  QModelIndex current = householdView->selectionModel ()->currentIndex ();
  QModelIndex source = proxyModel->mapToSource (current);
  int currentRow = source.row ();
  QString currentHousehold = model->index (currentRow, 0).data ().toString ();

  if (editHouseholdDialog) 
    editHouseholdDialog->changeName (currentHousehold);
  else  {
    editHouseholdDialog = new EditHouseholdDialog (currentHousehold);
    connect (editHouseholdDialog, SIGNAL (replaceName (QString, QString)),
             this, SIGNAL (replaceHouseholdName (QString, QString)));
  }

  editHouseholdDialog->show ();
}
