#include <QString>
#include <QLabel>
#include <QPushButton>
#include <QComboBox>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QStringList>
#include <QDir>
#include <QFile>
#include <QTextStream>
#include <QMessageBox>
#include <QProgressBar>
#include <QDialog>

#include <windows.h>
#include <tchar.h>
#include <cmath>
#include <string>
#include <cstdio>
#include <cstdlib>
using namespace std;

#include "importdialog.h"
#include "importsimspage.h"
#include "const.h"
#include "sim.h"
#include "csv.h"

void csvFieldDone (void*, size_t, void*);
void csvRecordDone (int, void*);

ImportDialog::ImportDialog (QString path, int flags)  {
  currentPage = 0;
  currentSection = PLAYABLE;
  epFlags = flags;

  topLabel = new QLabel ("");

  topLayout = new QHBoxLayout;
  topLayout->addStretch (1);
  topLayout->addWidget (topLabel);
  topLayout->addStretch (1);

  selectAllPlayablesButton = new QPushButton ("Select All Playables");
  selectAllTowniesButton = new QPushButton ("Select All Townies");
  selectAllNPCsButton = new QPushButton ("Select All NPCs");

  connect (selectAllPlayablesButton, SIGNAL (clicked ()), this,
           SLOT (selectAllPlayables ()));
  connect (selectAllTowniesButton, SIGNAL (clicked ()), this,
           SLOT (selectAllTownies ()));
  connect (selectAllNPCsButton, SIGNAL (clicked ()), this,
           SLOT (selectAllNPCs ()));

  selectAllLayout = new QHBoxLayout;
  selectAllLayout->addStretch (1);
  selectAllLayout->addWidget (selectAllPlayablesButton);
  selectAllLayout->addWidget (selectAllTowniesButton);
  selectAllLayout->addWidget (selectAllNPCsButton);
  selectAllLayout->addStretch (1);

  deselectPlayablesButton = new QPushButton ("Deselect All Playables");
  deselectTowniesButton = new QPushButton ("Deselect All Townies");
  deselectNPCsButton = new QPushButton ("Deselect All NPCs");

  connect (deselectPlayablesButton, SIGNAL (clicked ()), this,
           SLOT (deselectPlayables ()));
  connect (deselectTowniesButton, SIGNAL (clicked ()), this,
           SLOT (deselectTownies ()));
  connect (deselectNPCsButton, SIGNAL (clicked ()), this,
           SLOT (deselectNPCs ()));

  deselectLayout = new QHBoxLayout;
  deselectLayout->addStretch (1);
  deselectLayout->addWidget (deselectPlayablesButton);
  deselectLayout->addWidget (deselectTowniesButton);
  deselectLayout->addWidget (deselectNPCsButton);
  deselectLayout->addStretch (1);

  nextButton = new QPushButton ("Next Page");
  nextButton->setMaximumSize (nextButton->sizeHint ());
  nextButton->setEnabled (false);
  previousButton = new QPushButton ("Previous Page");
  previousButton->setMaximumSize (previousButton->sizeHint ());
  previousButton->setEnabled (false);
  pageLabel = new QLabel ("no page");

  navigationLayout = new QHBoxLayout;
  navigationLayout->addWidget (previousButton);
  navigationLayout->addStretch (1);
  navigationLayout->addWidget (pageLabel);
  navigationLayout->addStretch (1);
  navigationLayout->addWidget (nextButton);

  jumpButton = new QPushButton ("Jump to: ");
  jumpToSectionBox = new QComboBox;
  jumpToSectionBox->addItem ("Playables");
  jumpToSectionBox->addItem ("Townies");
  jumpToSectionBox->addItem ("NPCs");
  jumpToPageBox = new QComboBox;

  connect (jumpToSectionBox, SIGNAL (currentIndexChanged (int)), this,
           SLOT (updateJumpBoxes ()));
  connect (jumpButton, SIGNAL (clicked ()), this, SLOT (jump ()));

  jumpLayout = new QHBoxLayout;
  jumpLayout->addStretch (1);
  jumpLayout->addWidget (jumpButton);
  jumpLayout->addWidget (jumpToSectionBox);
  jumpLayout->addWidget (jumpToPageBox);
  jumpLayout->addStretch (1);

  importPage = new ImportSimsPage;
  importButton = new QPushButton ("Import");
  importButton->setMaximumSize (importButton->sizeHint ());
  importButton->setDefault (true);

  bottomLayout = new QHBoxLayout;
  bottomLayout->addStretch (1);
  bottomLayout->addWidget (importButton);
  bottomLayout->addStretch (1);

  mainLayout = new QVBoxLayout;
  mainLayout->addLayout (topLayout);
  mainLayout->addLayout (selectAllLayout);
  mainLayout->addLayout (deselectLayout);
  mainLayout->addLayout (navigationLayout);
  mainLayout->addLayout (jumpLayout);
  mainLayout->addWidget (importPage);
  mainLayout->addLayout (bottomLayout);

  setLayout (mainLayout);

  connect (nextButton, SIGNAL (clicked ()), this, SLOT (nextPage ()));
  connect (previousButton, SIGNAL (clicked ()), this, SLOT (previousPage ()));
  connect (importButton, SIGNAL (clicked ()), this, SLOT (import ()));

  loadSimInfo (path);

  resize (minimumSize ());
}

void ImportDialog::addFieldToInput (QString field)  {
  if (inputList.size () == 0)
    inputList.append (QStringList ());

  inputList[inputList.size () - 1].append (field);
}

void ImportDialog::addRecordToInput ()  {
  inputList.append (QStringList ());
}

void ImportDialog::nextPage ()  {
  saveData ();
  
  if (currentSection == PLAYABLE && currentPage*5+5 >= playablesList.size ()) {
    currentPage = 0;
    currentSection = TOWNIE;
  }

  else if (currentSection == TOWNIE && currentPage*5+5 >= towniesList.size ()){
    currentPage = 0;
    currentSection = NPC;
  }

  else if (currentSection == NPC && currentPage * 5 + 5 >= npcsList.size ())
    return;

  else currentPage++;

  updateDisplay ();
}

void ImportDialog::previousPage ()  {
  saveData ();
  
  if (currentSection == NPC && currentPage == 0)  {
    currentPage = (int)floor (towniesList.size () / 5.00);
    if (towniesList.size () % 5 == 0) currentPage--;
    currentSection = TOWNIE;
  }

  else if (currentSection == TOWNIE && currentPage == 0)  {
    currentPage = (int)floor (playablesList.size () / 5.00);
    if (playablesList.size () % 5 == 0) currentPage--;
    currentSection = PLAYABLE;
  }

  else if (currentSection == PLAYABLE && currentPage == 0)
    return;

  else currentPage--;

  updateDisplay ();
}

void ImportDialog::jump ()  {
  currentSection = jumpToSectionBox->currentIndex ();
  currentPage = jumpToPageBox->currentIndex ();

  updateDisplay ();
}

void ImportDialog::updateJumpBoxes ()  {
  jumpToPageBox->clear ();

  int numPages = 0;
  if (jumpToSectionBox->currentIndex () == PLAYABLE)  {
    numPages = (int)floor (playablesList.size () / 5.00 + 1);
    if (playablesList.size () % 5 == 0) numPages--;
  }

  if (jumpToSectionBox->currentIndex () == TOWNIE)  {
    numPages = (int)floor (towniesList.size () / 5.00 + 1);
    if (towniesList.size () % 5 == 0) numPages--;
  }

  if (jumpToSectionBox->currentIndex () == NPC)  {
    numPages = (int)floor (npcsList.size () / 5.00 + 1);
    if (npcsList.size () % 5 == 0) numPages--;
  }

  for (int x = 0; x < numPages; x++)
    jumpToPageBox->addItem ("Page " + QString::number (x + 1));

  jumpToPageBox->setMinimumWidth (jumpToPageBox->sizeHint ().width () + 10);
}

void ImportDialog::selectAllPlayables ()  {
  for (int x = 0; x < playablesList.size (); x++)
    playablesList[x][21] = "Y";

  updateDisplay ();
}

void ImportDialog::selectAllTownies ()  {
  for (int x = 0; x < towniesList.size (); x++)
    towniesList[x][21] = "Y";

  updateDisplay ();
}

void ImportDialog::selectAllNPCs ()  {
  for (int x = 0; x < npcsList.size (); x++)
    npcsList[x][21] = "Y";

  updateDisplay ();
}

void ImportDialog::deselectPlayables ()  {
  for (int x = 0; x < playablesList.size (); x++)
    playablesList[x][21] = "N";

  updateDisplay ();
}

void ImportDialog::deselectTownies ()  {
  for (int x = 0; x < towniesList.size (); x++)
    towniesList[x][21] = "N";

  updateDisplay ();
}

void ImportDialog::deselectNPCs ()  {
  for (int x = 0; x < npcsList.size (); x++)
    npcsList[x][21] = "N";

  updateDisplay ();
}

void ImportDialog::import ()  {
  saveData ();

  int simsToImport = 0;
  for (int x = 0; x < playablesList.size (); x++)
    if (playablesList[x][21] == "Y")
      simsToImport++;
  for (int x = 0; x < towniesList.size (); x++)
    if (towniesList[x][21] == "Y")
      simsToImport++;
  for (int x = 0; x < npcsList.size (); x++)
    if (npcsList[x][21] == "Y")
      simsToImport++;

  QDialog *progressDialog = new QDialog (this);
  QVBoxLayout *progressLayout = new QVBoxLayout;
  progressDialog->setLayout (progressLayout);
  QProgressBar *progressBar = new QProgressBar ();
  progressBar->setMaximum (simsToImport);
  progressBar->setValue (0);
  progressBar->setFormat ("%v of %m");
  progressLayout->addWidget (progressBar);
  progressDialog->setMinimumWidth (500);
  progressDialog->show ();

  QList<Sim*> s;
  QList< QList<int> > familyInfo;
  int currentSim = 0;

  for (int x = 0; x < playablesList.size (); x++)
    if (playablesList[x][21] == "Y")  {
      playablesList[x][13] = QString::number (playablesList[x][13].toInt ()+1);

      s.append (new Sim ());
      familyInfo.append (QList<int> ());

      s[currentSim]->setName (playablesList[x][3] + " " + playablesList[x][4]);
      s[currentSim]->setHousehold (playablesList[x][7]);
      s[currentSim]->setGender (playablesList[x][11] == "Male" ? MALE : FEMALE);
      s[currentSim]->setNID (playablesList[x][2].toInt ());
      s[currentSim]->setNotes (playablesList[x][5]);
      s[currentSim]->setSpecies (playablesList[x][17]);
      s[currentSim]->setParaFlags (playablesList[x][22].toInt ());
      s[currentSim]->setAlignment (playablesList[x][23].toInt ());

      int age = ELDER;

      if (playablesList[x][17] == "Human" && playablesList[x][6].toInt ())  {
        if ((playablesList[x][22].toInt () & PLANTSIM) &&
            playablesList[x][12] == "Toddler")
          age = PLANTSIM_ADULT - playablesList[x][13].toInt ();
        else if ((playablesList[x][22].toInt () & PLANTSIM) &&
                 playablesList[x][12] == "Adult")
          age = PLANTSIM_ELDER - playablesList[x][13].toInt ();

        else if (playablesList[x][12] == "Baby")
          age = TODDLER - playablesList[x][13].toInt ();
        else if (playablesList[x][12] == "Toddler")
          age = CHILD - playablesList[x][13].toInt ();
        else if (playablesList[x][12] == "Child")
          age = TEEN - playablesList[x][13].toInt ();
        else if (playablesList[x][12] == "Teen")
          age = ADULT - playablesList[x][13].toInt ();
        else if (playablesList[x][12] == "Adult")
          age = ELDER - playablesList[x][13].toInt ();
        else if (playablesList[x][12] == "Elder")
          age = 84 - playablesList[x][13].toInt ();
      }

      else if (playablesList[x][17] == "LargeDog" && 
               playablesList[x][6].toInt ())  {
        if (playablesList[x][12] == "Baby")
          age = ADULT_DOG - playablesList[x][13].toInt ();
        else if (playablesList[x][12] == "Adult")
          age = ELDER_LARGE_DOG - playablesList[x][13].toInt ();
      }

      else if (playablesList[x][17] == "SmallDog" && 
               playablesList[x][6].toInt ())  {
        if (playablesList[x][12] == "Baby")
          age = ADULT_DOG - playablesList[x][13].toInt ();
        else if (playablesList[x][12] == "Adult")
          age = ELDER_SMALL_DOG - playablesList[x][13].toInt ();
      }

      else if (playablesList[x][17] == "Cat" && playablesList[x][6].toInt ()) {
        if (playablesList[x][12] == "Baby")
          age = ADULT_CAT - playablesList[x][13].toInt ();
        else if (playablesList[x][12] == "Adult")
          age = ELDER_CAT - playablesList[x][13].toInt ();
      }

      s[currentSim]->setAge (age);

      if (playablesList[x][6].toInt () == 0)
        s[currentSim]->makeDead (WHITE);

      if (playablesList[x][14] == "Y")  {
        s[currentSim]->setCollege (true);
        s[currentSim]->setSemester (playablesList[x][15].toInt ());
      }
      s[currentSim]->setMajor (playablesList[x][16]);

      s[currentSim]->updateAspiration (playablesList[x][18], 
                                      playablesList[x][19]);
      s[currentSim]->addLTW (playablesList[x][20]);

      bool fieldIsNumber;
      int num = playablesList[x][10].toInt (&fieldIsNumber);
      if (fieldIsNumber) familyInfo[currentSim].append (num);
      else familyInfo[currentSim].append (-1);

      num = playablesList[x][8].toInt (&fieldIsNumber);
      if (fieldIsNumber) familyInfo[currentSim].append (num);
      else familyInfo[currentSim].append (-1);

      num = playablesList[x][9].toInt (&fieldIsNumber);
      if (fieldIsNumber) familyInfo[currentSim].append (num);
      else familyInfo[currentSim].append (-1);

      progressBar->setValue (progressBar->value () + 1);
      currentSim++;
    }

  for (int x = 0; x < towniesList.size (); x++)
    if (towniesList[x][21] == "Y")  {
      towniesList[x][13] = QString::number (towniesList[x][13].toInt () + 1);

      s.append (new Sim ());
      familyInfo.append (QList<int> ());

      s[currentSim]->setName (towniesList[x][3] + " " + towniesList[x][4]);
      s[currentSim]->setHousehold (towniesList[x][7]);
      s[currentSim]->setGender (towniesList[x][11] == "Male" ? MALE : FEMALE);
      s[currentSim]->setNID (towniesList[x][2].toInt ());
      s[currentSim]->setNotes (towniesList[x][5]);
      s[currentSim]->setSpecies (towniesList[x][17]);
      s[currentSim]->setParaFlags (towniesList[x][22].toInt ());
      s[currentSim]->setAlignment (towniesList[x][23].toInt ());

      int age = ELDER;

      if (towniesList[x][17] == "Human" && towniesList[x][6].toInt ())  {
        if ((towniesList[x][22].toInt () & PLANTSIM) &&
            towniesList[x][12] == "Toddler")
          age = PLANTSIM_ADULT - towniesList[x][13].toInt ();
        else if ((towniesList[x][22].toInt () & PLANTSIM) &&
                 towniesList[x][12] == "Adult")
          age = PLANTSIM_ELDER - towniesList[x][13].toInt ();

        else if (towniesList[x][12] == "Baby")
          age = TODDLER - towniesList[x][13].toInt ();
        else if (towniesList[x][12] == "Toddler")
          age = CHILD - towniesList[x][13].toInt ();
        else if (towniesList[x][12] == "Child")
          age = TEEN - towniesList[x][13].toInt ();
        else if (towniesList[x][12] == "Teen")
          age = ADULT - towniesList[x][13].toInt ();
        else if (towniesList[x][12] == "Adult")
         age = ELDER - towniesList[x][13].toInt ();
        else if (towniesList[x][12] == "Elder")
         age = 84 - towniesList[x][13].toInt ();
      }

      else if (towniesList[x][17] == "LargeDog" && towniesList[x][6].toInt ()){
        if (towniesList[x][12] == "Baby")
          age = ADULT_DOG - towniesList[x][13].toInt ();
        else if (towniesList[x][12] == "Adult")
          age = ELDER_LARGE_DOG - towniesList[x][13].toInt ();
      }

      else if (towniesList[x][17] == "SmallDog" && towniesList[x][6].toInt ()){
        if (towniesList[x][12] == "Baby")
          age = ADULT_DOG - towniesList[x][13].toInt ();
        else if (towniesList[x][12] == "Adult")
          age = ELDER_SMALL_DOG - towniesList[x][13].toInt ();
      }

      else if (towniesList[x][17] == "Cat" && towniesList[x][6].toInt ())  {
        if (towniesList[x][12] == "Baby")
          age = ADULT_CAT - towniesList[x][13].toInt ();
        else if (towniesList[x][12] == "Adult")
          age = ELDER_CAT - towniesList[x][13].toInt ();
      }

      s[currentSim]->setAge (age);

      if (towniesList[x][6].toInt () == 0)
        s[currentSim]->makeDead (WHITE);

      if (towniesList[x][14] == "Y")
        s[currentSim]->setCollege (true);
      s[currentSim]->setMajor (towniesList[x][16]);

      s[currentSim]->updateAspiration (towniesList[x][18], towniesList[x][19]);
      s[currentSim]->addLTW (towniesList[x][20]);

      bool fieldIsNumber;
      int num = towniesList[x][10].toInt (&fieldIsNumber);
      if (fieldIsNumber) familyInfo[currentSim].append (num);
      else familyInfo[currentSim].append (-1);

      num = towniesList[x][8].toInt (&fieldIsNumber);
      if (fieldIsNumber) familyInfo[currentSim].append (num);
      else familyInfo[currentSim].append (-1);

      num = towniesList[x][9].toInt (&fieldIsNumber);
      if (fieldIsNumber) familyInfo[currentSim].append (num);
      else familyInfo[currentSim].append (-1);

      progressBar->setValue (progressBar->value () + 1);
      currentSim++;
    }

  for (int x = 0; x < npcsList.size (); x++)
    if (npcsList[x][21] == "Y")  {
      npcsList[x][13] = QString::number (npcsList[x][13].toInt () + 1);

      s.append (new Sim ());
      familyInfo.append (QList<int> ());

      s[currentSim]->setName (npcsList[x][3] + " " + npcsList[x][4]);
      s[currentSim]->setHousehold (npcsList[x][7]);
      s[currentSim]->setGender (npcsList[x][11] == "Male" ? MALE : FEMALE);
      s[currentSim]->setNID (npcsList[x][2].toInt ());
      s[currentSim]->setNotes (npcsList[x][5]);
      s[currentSim]->setSpecies (npcsList[x][17]);
      s[currentSim]->setParaFlags (npcsList[x][22].toInt ());
      s[currentSim]->setAlignment (npcsList[x][23].toInt ());

      int age = ELDER;

      if (npcsList[x][17] == "Human" && npcsList[x][6].toInt ())  {
        if ((npcsList[x][22].toInt () & PLANTSIM) &&
            npcsList[x][12] == "Toddler")
          age = PLANTSIM_ADULT - npcsList[x][13].toInt ();
        else if ((npcsList[x][22].toInt () & PLANTSIM) &&
                 npcsList[x][12] == "Adult")
          age = PLANTSIM_ELDER - npcsList[x][13].toInt ();

        else if (npcsList[x][12] == "Baby")
          age = TODDLER - npcsList[x][13].toInt ();
        else if (npcsList[x][12] == "Toddler")
          age = CHILD - npcsList[x][13].toInt ();
        else if (npcsList[x][12] == "Child")
          age = TEEN - npcsList[x][13].toInt ();
        else if (npcsList[x][12] == "Teen")
          age = ADULT - npcsList[x][13].toInt ();
        else if (npcsList[x][12] == "Adult")
          age = ELDER - npcsList[x][13].toInt ();
        else if (npcsList[x][12] == "Elder")
          age = 84 - npcsList[x][13].toInt ();
      }

      else if (npcsList[x][17] == "LargeDog" && npcsList[x][6].toInt ())  {
        if (npcsList[x][12] == "Baby")
          age = ADULT_DOG - npcsList[x][13].toInt ();
        else if (npcsList[x][12] == "Adult")
          age = ELDER_LARGE_DOG - npcsList[x][13].toInt ();
      }

      else if (npcsList[x][17] == "SmallDog" && npcsList[x][6].toInt ())  {
        if (npcsList[x][12] == "Baby")
          age = ADULT_DOG - npcsList[x][13].toInt ();
        else if (npcsList[x][12] == "Adult")
          age = ELDER_SMALL_DOG - npcsList[x][13].toInt ();
      }

      else if (npcsList[x][17] == "Cat" && npcsList[x][6].toInt ())  {
        if (npcsList[x][12] == "Baby")
          age = ADULT_CAT - npcsList[x][13].toInt ();
        else if (npcsList[x][12] == "Adult")
          age = ELDER_CAT - npcsList[x][13].toInt ();
      }

      s[currentSim]->setAge (age);

      if (npcsList[x][6].toInt () == 0)
        s[currentSim]->makeDead (WHITE);

      if (npcsList[x][14] == "Y")
        s[currentSim]->setCollege (true);
      s[currentSim]->setMajor (npcsList[x][16]);

      s[currentSim]->updateAspiration (npcsList[x][18], npcsList[x][19]);
      s[currentSim]->addLTW (npcsList[x][20]);

      bool fieldIsNumber;
      int num = npcsList[x][10].toInt (&fieldIsNumber);
      if (fieldIsNumber) familyInfo[currentSim].append (num);
      else familyInfo[currentSim].append (-1);

      num = npcsList[x][8].toInt (&fieldIsNumber);
      if (fieldIsNumber) familyInfo[currentSim].append (num);
      else familyInfo[currentSim].append (-1);

      num = npcsList[x][9].toInt (&fieldIsNumber);
      if (fieldIsNumber) familyInfo[currentSim].append (num);
      else familyInfo[currentSim].append (-1);

      progressBar->setValue (progressBar->value () + 1);
      currentSim++;
    }

  progressBar->reset ();
  progressBar->setFormat ("%v of %m (Family Info)");

  for (int x = 0; x < s.size (); x++)  {
    if (familyInfo[x][0] != -1)
      for (int y = 0; y < s.size (); y++)
        if (s[y]->getNID () == familyInfo[x][0])  {
          s[x]->setSpouse (s[y]);
          break;
        }

    if (familyInfo[x][1] != -1)
      for (int y = 0; y < s.size (); y++)
        if (s[y]->getNID () == familyInfo[x][1])  {
          s[x]->setParents (s[y], NULL);
          break;
        }

    if (familyInfo[x][2] != -1)
      for (int y = 0; y < s.size (); y++)
        if (s[y]->getNID () == familyInfo[x][2])  {
          s[x]->setParents (NULL, s[y]);
          break;
        }

    progressBar->setValue (progressBar->value () + 1);
  }

  progressBar->reset ();
  progressBar->setFormat ("%v of %m (Adding to List)");

  for (int x = 0; x < s.size (); x++)  {
    emit addSim (s[x]);
    progressBar->setValue (progressBar->value () + 1);
  }

  emit updateModels ();

  delete progressDialog;
  close ();
}

void ImportDialog::loadSimInfo (QString path)  {
  QDir simsDir (path);

  QFile file (path + "/Rufio/ExportedSims.txt");
  QTextStream in (&file);

  if (!file.open (QFile::ReadOnly | QFile::Text))  {
    QMessageBox::warning (this, "", "Cannot read SimPE output.");
    return;
  }

  QString line = in.readLine ();

  int hoodIndex = -1;
  int hoodNameIndex = -1;
  int nidIndex = -1;
  int firstNameIndex = -1;
  int lastNameIndex = -1;
  int descriptionIndex = -1;
  int familyIndex = -1;
  int householdIndex = -1;
  int parentAIndex = -1;
  int parentBIndex = -1;
  int spouseIndex = -1;
  int genderIndex = -1;
  int ageIndex = -1;
  int daysLeftIndex = -1;
  int yaIndex = -1;
  int semesterIndex = -1;
  int majorIndex = -1;
  int speciesIndex = -1;
  int paspIndex = -1;
  int saspIndex = -1;
  int ltwIndex = -1;

  QStringList splitLine = line.split (',');

  for (int x = 0; x < splitLine.size (); x++)  {
    if (splitLine[x] == "hood") hoodIndex = x;
    if (splitLine[x] == "HoodName") hoodNameIndex = x;
    if (splitLine[x] == "NID") nidIndex = x;
    if (splitLine[x] == "FirstName") firstNameIndex = x;
    if (splitLine[x] == "LastName") lastNameIndex = x;
    if (splitLine[x] == "SimDescription") descriptionIndex = x;
    if (splitLine[x] == "FamilyInstance") familyIndex = x;
    if (splitLine[x] == "HouseholdName") householdIndex = x;
    if (splitLine[x] == "ParentA") parentAIndex = x;
    if (splitLine[x] == "ParentB") parentBIndex = x;
    if (splitLine[x] == "Spouse") spouseIndex = x;
    if (splitLine[x] == "Gender") genderIndex = x;
    if (splitLine[x] == "LifeSection") ageIndex = x;
    if (splitLine[x] == "AgeDaysLeft") daysLeftIndex = x;
    if (splitLine[x] == "IsAtUniversity") yaIndex = x;
    if (splitLine[x] == "UniSemester") semesterIndex = x;
    if (splitLine[x] == "UniMajor") majorIndex = x;
    if (splitLine[x] == "Species") speciesIndex = x;
    if (splitLine[x] == "PrimaryAspiration") paspIndex = x;
    if (splitLine[x] == "SecondaryAspiration") saspIndex = x;
    if (splitLine[x] == "LifetimeWant") ltwIndex = x;
  }

  QString fileContents = "";

  while (!in.atEnd ())
    fileContents += in.readLine () + "\n";

  QFile otherFile ("ExportedSims.txt");
  QTextStream otherStream (&otherFile);
  if (!otherFile.open (QFile::WriteOnly | QFile::Text))
    QMessageBox::warning (this, "", "Could not open file.");
  otherStream << fileContents;
  otherFile.close ();
  file.close ();

  size_t i;
  char buf[1024];

  FILE *inFile = fopen ("ExportedSims.txt", "rb");
  if (inFile == NULL) QMessageBox::warning (this, "", "Could not open file.");

  struct csv_parser parser;
  csv_init (&parser, 0);

  while ((i = fread (buf, 1, 1024, inFile)) > 0)
    if (csv_parse (&parser, buf, i, csvFieldDone, csvRecordDone, this) != i) {
      QMessageBox::warning (this, "", "Error parsing file: " + 
                            QString (csv_strerror (csv_error (&parser))));
      fclose (inFile);
    }

  csv_fini (&parser, csvFieldDone, csvRecordDone, this);
  csv_free (&parser);
/*
  bool truncatedLine = false;

  while (line != "" || truncatedLine)  {
    if (truncatedLine)  {
      line += in.readLine ();
      truncatedLine = false;
    }
    else line = in.readLine ();

    if (line != "")  {
      QStringList splitLine = line.split (',');

      for (int x = 0; x < splitLine.size (); x++)  {
        if (splitLine[x].startsWith ("\"") && !splitLine[x].endsWith ("\""))  {
          if (splitLine.size () <= x + 1)  {
            splitLine[x].append ("\r\n");
            truncatedLine = true;
            break;
          }
          
          splitLine[x] += "," + splitLine[x + 1];
          splitLine.removeAt (x + 1);
          x--;
        }

        else if (splitLine[x].startsWith ("\""))  {
          int last = splitLine[x].size () - 1;
          int quotes = 0;

          while (last >= 0)  {
            if (splitLine[x][last--] == '\"')
              quotes++;
            else break;
          }

          if (quotes % 2 == 0)  {
            if (splitLine.size () <= x + 1)  {
              splitLine[x].append ("\r\n");
              truncatedLine = true;
              break;
            }
          
            splitLine[x] += "," + splitLine[x + 1];
            splitLine.removeAt (x + 1);
            x--;
          }
        }
      }

      if (truncatedLine) continue;

      for (int x = 0; x < splitLine.size (); x++)
        splitLine[x].replace ("\"\"", "\"");
*/

  for (int x = 0; x < inputList.size (); x++)  {
    QStringList newLine;
    newLine.append (hoodIndex >= 0 && inputList[x].size () > hoodIndex ? 
                    inputList[x][hoodIndex] : "");
    newLine.append (hoodNameIndex >= 0 && inputList[x].size () > hoodNameIndex ?
                    inputList[x][hoodNameIndex] : "");
    newLine.append (nidIndex >= 0 && inputList[x].size () > nidIndex ? 
                    inputList[x][nidIndex] : "0");
    newLine.append (firstNameIndex >= 0 && inputList[x].size() > firstNameIndex?
                    inputList[x][firstNameIndex] : "");
    newLine.append (lastNameIndex >= 0 && inputList[x].size () > lastNameIndex ?
                    inputList[x][lastNameIndex] : "");
    newLine.append(descriptionIndex >= 0 && 
                   inputList[x].size () > descriptionIndex ? 
                   inputList[x][descriptionIndex] : "");
    newLine.append (familyIndex >= 0 && inputList[x].size () > familyIndex ? 
                    inputList[x][familyIndex] : "0");
    newLine.append (householdIndex >= 0 && inputList[x].size() > householdIndex?
                    inputList[x][householdIndex] : "");
    newLine.append (parentAIndex >= 0 && inputList[x].size () > parentAIndex ? 
                    inputList[x][parentAIndex] : "");
    newLine.append (parentBIndex >= 0 && inputList[x].size () > parentBIndex ? 
                    inputList[x][parentBIndex] : "");
    newLine.append (spouseIndex >= 0 && inputList[x].size () > spouseIndex ? 
                    inputList[x][spouseIndex] : "");
    newLine.append (genderIndex >= 0 && inputList[x].size () > genderIndex ? 
                    inputList[x][genderIndex] : "");
    newLine.append (ageIndex >= 0 && inputList[x].size () > ageIndex ? 
                    inputList[x][ageIndex] : "");
    newLine.append (daysLeftIndex >= 0 && inputList[x].size () > daysLeftIndex ?
                    inputList[x][daysLeftIndex] : "0");
    newLine.append (yaIndex >= 0 && inputList[x].size () > yaIndex ? 
                    inputList[x][yaIndex] : "");
    newLine.append (semesterIndex >= 0 && inputList[x].size () > semesterIndex ?
                    inputList[x][semesterIndex] : "0");
    newLine.append (majorIndex >= 0 && inputList[x].size () > majorIndex ? 
                    inputList[x][majorIndex] : "");
    newLine.append (speciesIndex >= 0 && inputList[x].size () > speciesIndex ? 
                    inputList[x][speciesIndex] : "");
    newLine.append (paspIndex >= 0 && inputList[x].size () > paspIndex ? 
                    inputList[x][paspIndex] : "");
    newLine.append (saspIndex >= 0 && inputList[x].size () > saspIndex ? 
                    inputList[x][saspIndex] : "");
    newLine.append (ltwIndex >= 0 && inputList[x].size () > ltwIndex ? 
                    inputList[x][ltwIndex] : "");

    if (newLine[6].toInt () == 0x7FFF)  {
      newLine[7] = "Service NPCs";
      npcsList.append (newLine);
    }

    else if (newLine[6].toInt () >= 0x7F00 && 
             newLine[6].toInt () <= 0x7FFE)  {
      if (newLine[6].toInt () == 0x7FF1) newLine[7] = "Island Locals";
      if (newLine[6].toInt () == 0x7FF2) newLine[7] = "Mountain Locals";
      if (newLine[6].toInt () == 0x7FF3) newLine[7] = "Asian Locals";
      if (newLine[6].toInt () >= 0x7FE5 && newLine[6].toInt () <= 0x7FF0)
        newLine[7] = "Tourists";
      if (newLine[6].toInt () == 0x7FE4) newLine[7] = "Hobby Mavens";
      if (newLine[6].toInt () == 0x7FDF) newLine[7] = "Socialites";
      if (newLine[6].toInt () == 0x7FE0) newLine[7] = "Techies";
      if (newLine[6].toInt () == 0x7FE1) newLine[7] = "Jocks";
      if (newLine[6].toInt () == 0x7FE2) newLine[7] = "Bohemians";
      if (newLine[6].toInt () == 0x7FE3) newLine[7] = "Gearheads";
      if (newLine[6].toInt () == 0x7FF6) newLine[7] = "Garden Club";
      if (newLine[6].toInt () == 0x7FF9 || 
          newLine[6].toInt () == 0x7FFD ||
          newLine[6].toInt () == 0x7FF7 ||
          newLine[6].toInt () == 0x7FF8) newLine[7] = "Adoption Queue";
      if (newLine[6].toInt () == 0x7FFC) newLine[7] = "Downtown";
      if (newLine[6].toInt () == 0x7FFE || newLine[6].toInt () == 0x7FFA) 
        newLine[7] = "Townies and Dormies";
       towniesList.append (newLine);
    }

    else if (newLine[6].toInt () != 0 || !newLine[4].contains ("(NPC)"))
      playablesList.append (newLine);
  }

  if (playablesList.size () > 0)
    topLabel->setText (playablesList[0][0] + ": " + playablesList[0][1]);

  for (int x = 0; x < playablesList.size (); x++)
    if (playablesList[x][3] == "Unknown" || playablesList[x][3] == "")  {
      playablesList.removeAt (x);
      x--;
    }

  for (int x = 0; x < towniesList.size (); x++)
    if (towniesList[x][3] == "Unknown" || towniesList[x][3] == "")  {
      towniesList.removeAt (x);
      x--;
    }

  for (int x = 0; x < npcsList.size (); x++)
    if (npcsList[x][3] == "Unknown" || npcsList[x][3] == "")  {
      npcsList.removeAt (x);
      x--;
    }

  for (int x = 0; x < playablesList.size (); x++)  {
    int smallest = x;

    for (int y = x + 1; y < playablesList.size (); y++)
      if (QString::localeAwareCompare (playablesList[smallest][3] + " " +
                                       playablesList[smallest][5], 
                                       playablesList[y][3] + " " +
                                       playablesList[y][5]) > 0)
        smallest = y;

    playablesList.swap (smallest, x);
  }

  for (int x = 0; x < towniesList.size (); x++)  {
    int smallest = x;

    for (int y = x + 1; y < towniesList.size (); y++)
      if (QString::localeAwareCompare (towniesList[smallest][3] + " " +
                                       towniesList[smallest][5],
                                       towniesList[y][3] + " " +
                                       towniesList[y][5]) > 0)
        smallest = y;
    
    towniesList.swap (smallest, x);
  }

  for (int x = 0; x < npcsList.size (); x++)  {
    int smallest = x;

    for (int y = x + 1; y < npcsList.size (); y++)
      if (QString::localeAwareCompare (npcsList[smallest][3] + " " +
                                       npcsList[smallest][5],
                                       npcsList[y][3] + " " + 
                                       npcsList[y][5]) > 0)
        smallest = y;

    npcsList.swap (smallest, x);
  }

  for (int x = 0; x < playablesList.size (); x++)  {
    playablesList[x].append ("N");
    playablesList[x].append ("0");
    playablesList[x].append ("0");
  }

  for (int x = 0; x < towniesList.size (); x++)  {
    towniesList[x].append ("N");
    towniesList[x].append ("0");
    towniesList[x].append ("0");
  }

  for (int x = 0; x < npcsList.size (); x++)  {
    npcsList[x].append ("N");
    npcsList[x].append ("0");
    npcsList[x].append ("0");
  }

  int total = playablesList.size () + towniesList.size () + npcsList.size ();

  QMessageBox::information (this, "", "Loaded data for " + 
                                      QString::number (total) + " sims");
  
  currentPage = 0;
  currentSection = PLAYABLE;

  updateDisplay ();
  updateJumpBoxes ();
  inputList.clear ();
}

void csvFieldDone (void *d, size_t len, void *l)  {
  const QByteArray data ((const char*)d, (int)len);
  QString *str = new QString (data);

  ((ImportDialog*)(l))->addFieldToInput (*str);
}

void csvRecordDone (int i, void *l)  {
  ((ImportDialog*)(l))->addRecordToInput ();
}

void ImportDialog::saveData ()  {
  if (currentSection == PLAYABLE) savePlayableData ();
  if (currentSection == TOWNIE) saveTownieData ();
  if (currentSection == NPC) saveNPCData ();
}

void ImportDialog::savePlayableData ()  {
  for (int x = 0; x < 5; x++)
    if (x + currentPage * 5 < playablesList.size ())  {
      if (importPage->isChecked (x))
        playablesList[x + currentPage * 5][21] = "Y";
      else playablesList[x + currentPage * 5][21] = "N";

      playablesList[x + currentPage * 5][7] = importPage->getHouseholdName (x);
      playablesList[x + currentPage * 5][22] = 
        QString::number (importPage->getParaFlags (x));
      playablesList[x + currentPage * 5][23] = 
        QString::number (importPage->getAlignment (x));
    }
}

void ImportDialog::saveTownieData ()  {
  for (int x = 0; x < 5; x++)
    if (x + currentPage * 5 < towniesList.size ())  {
      if (importPage->isChecked (x))
        towniesList[x + currentPage * 5][21] = "Y";
      else towniesList[x + currentPage * 5][21] = "N";

      towniesList[x + currentPage * 5][7] = importPage->getHouseholdName (x);
      towniesList[x + currentPage * 5][22] = 
        QString::number (importPage->getParaFlags (x));
      towniesList[x + currentPage * 5][23] = 
        QString::number (importPage->getAlignment (x));
    }
}

void ImportDialog::saveNPCData ()  {
  for (int x = 0; x < 5; x++)
    if (x + currentPage * 5 < npcsList.size ())  {
      if (importPage->isChecked (x))
        npcsList[x + currentPage * 5][21] = "Y";
      else npcsList[x + currentPage * 5][21] = "N";

      npcsList[x + currentPage * 5][7] = importPage->getHouseholdName (x);
      npcsList[x + currentPage * 5][22] = 
        QString::number (importPage->getParaFlags (x));
      npcsList[x + currentPage * 5][23] = 
        QString::number (importPage->getAlignment (x));
    }
}

void ImportDialog::updateDisplay ()  {
  QList<QStringList> pageData;
  for (int x = currentPage * 5; x < currentPage * 5 + 5; x++)  {
    if (currentSection == PLAYABLE && x < playablesList.size ())
      pageData.append (playablesList[x]);
    if (currentSection == TOWNIE && x < towniesList.size ())
      pageData.append (towniesList[x]);
    if (currentSection == NPC && x < npcsList.size ())
      pageData.append (npcsList[x]);
  }

  importPage->setEPFlags (epFlags);
  importPage->setData (pageData);
  importPage->setType (currentSection);
  importPage->setNumber (currentPage + 1);

  if (currentSection == NPC && currentPage * 5 + 5 >= npcsList.size ())
    nextButton->setEnabled (false);
  else nextButton->setEnabled (true);

  if (currentSection == PLAYABLE && currentPage == 0)
    previousButton->setEnabled (false);
  else previousButton->setEnabled (true);

  pageLabel->setText (importPage->getTitle ());
}
