#ifndef IMPORTDIALOG_H
#define IMPORTDIALOG_H

#include <QDialog>
#include <QList>

class QLabel;
class QString;
class QPushButton;
class QComboBox;
class QVBoxLayout;
class QHBoxLayout;
class QStringList;
class ImportSimsPage;
class Sim;

class ImportDialog : public QDialog  {
  Q_OBJECT

  public:
    ImportDialog (QString, int);

    void addFieldToInput (QString);
    void addRecordToInput ();

  signals:
    void addSim (Sim*);
    void updateModels ();

  private slots:
    void nextPage ();
    void previousPage ();
    void jump ();
    void updateJumpBoxes ();

    void selectAllPlayables ();
    void selectAllTownies ();
    void selectAllNPCs ();
    void deselectPlayables ();
    void deselectTownies ();
    void deselectNPCs ();

    void import ();

  private:
    void loadSimInfo (QString);
    void saveData ();
    void savePlayableData ();
    void saveTownieData ();
    void saveNPCData ();
    void updateDisplay ();

    int currentPage;
    int currentSection;
    int epFlags;

    QList<QStringList> inputList;
    QList<QStringList> playablesList;
    QList<QStringList> towniesList;
    QList<QStringList> npcsList;

    QLabel *topLabel;
    QPushButton *selectAllPlayablesButton;
    QPushButton *selectAllTowniesButton;
    QPushButton *selectAllNPCsButton;
    QPushButton *deselectPlayablesButton;
    QPushButton *deselectTowniesButton;
    QPushButton *deselectNPCsButton;
    QPushButton *nextButton;
    QPushButton *previousButton;
    QComboBox *jumpToSectionBox;
    QComboBox *jumpToPageBox;
    QPushButton *jumpButton;
    QLabel *pageLabel;
    QPushButton *importButton;

    ImportSimsPage *importPage;

    QHBoxLayout *topLayout;
    QHBoxLayout *selectAllLayout;
    QHBoxLayout *deselectLayout;
    QHBoxLayout *navigationLayout;
    QHBoxLayout *jumpLayout;
    QHBoxLayout *bottomLayout;
    QVBoxLayout *mainLayout;
};

#endif
