#include <QCheckBox>
#include <QComboBox>
#include <QString>
#include <QLineEdit>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QLabel>

#include "importsimdisplay.h"
#include "const.h"

ImportSimDisplay::ImportSimDisplay ()  {
  name = "";
  gender = "";
  age = "";
  species = "";
  aspiration = "";
  saspiration = "";
  epFlags = NONE;

  simBox = new QCheckBox ("");
  alienBox = new QCheckBox ("Alien");
  zombieBox = new QCheckBox ("Zombie");
  vampireBox = new QCheckBox ("Vampire");
  servoBox = new QCheckBox ("Servo");
  werewolfBox = new QCheckBox ("Werewolf");
  plantsimBox = new QCheckBox ("Plantsim");
  bigfootBox = new QCheckBox ("Bigfoot");
  witchBox = new QCheckBox ("Witch");

  alignmentBox = new QComboBox;
  alignmentBox->addItem ("Atrociously Evil");
  alignmentBox->addItem ("Evil");
  alignmentBox->addItem ("Mean");
  alignmentBox->addItem ("Neutral");
  alignmentBox->addItem ("Nice");
  alignmentBox->addItem ("Good");
  alignmentBox->addItem ("Infallibly Good");
  alignmentBox->setCurrentIndex (3);
  alignmentBox->setMaximumSize (alignmentBox->sizeHint ());
  alignmentBox->setEnabled (false);

  connect (witchBox, SIGNAL (toggled (bool)), alignmentBox, 
           SLOT (setEnabled (bool)));

  householdEdit = new QLineEdit;
  householdEdit->setMinimumWidth (50);

  QLabel *alignmentLabel = new QLabel ("Alignment: ");
  QLabel *householdLabel = new QLabel ("Household: ");

  topLayout = new QHBoxLayout;
  topLayout->addWidget (simBox);
  topLayout->addWidget (alignmentLabel);
  topLayout->addWidget (alignmentBox);
  topLayout->addWidget (householdLabel);
  topLayout->addWidget (householdEdit);

  paraLayout = new QHBoxLayout;
  paraLayout->addStretch (1);
  paraLayout->addWidget (alienBox);
  paraLayout->addWidget (zombieBox);
  paraLayout->addWidget (vampireBox);
  paraLayout->addWidget (servoBox);
  paraLayout->addWidget (werewolfBox);
  paraLayout->addWidget (plantsimBox);
  paraLayout->addWidget (bigfootBox);
  paraLayout->addWidget (witchBox);
  paraLayout->addStretch (1);

  mainLayout = new QVBoxLayout;
  mainLayout->addLayout (topLayout);
  mainLayout->addLayout (paraLayout);

  setLayout (mainLayout);
}

void ImportSimDisplay::setName (QString n)  {
  name = n;
  updateDisplay ();
}

void ImportSimDisplay::setHousehold (QString h)  {
  householdEdit->setText (h);
  householdEdit->setMinimumSize (householdEdit->sizeHint ());
}

void ImportSimDisplay::setGender (QString g)  {
  gender = g;
  updateDisplay ();
}

void ImportSimDisplay::setLifeStage (QString l)  {
  age = l;
  updateDisplay ();
}

void ImportSimDisplay::setAspiration (QString p, QString s)  {
  aspiration = p;
  saspiration = s;
  updateDisplay ();
}

void ImportSimDisplay::setSpecies (QString s)  {
  species = s;
  updateDisplay ();
}

void ImportSimDisplay::setEPFlags (int flags)  {
  epFlags = flags;
  updateDisplay ();
}

void ImportSimDisplay::setChecked (bool b)  {
  simBox->setChecked (b);
}

void ImportSimDisplay::setParaFlags (int pf)  {
  alienBox->setChecked (pf & ALIEN);
  zombieBox->setChecked (pf & ZOMBIE);
  vampireBox->setChecked (pf & VAMPIRE);
  servoBox->setChecked (pf & SERVO);
  werewolfBox->setChecked (pf & WEREWOLF);
  plantsimBox->setChecked (pf & PLANTSIM);
  bigfootBox->setChecked (pf & BIGFOOT);
  witchBox->setChecked (pf & WITCH);
}

void ImportSimDisplay::setAlignment (int a)  {
  alignmentBox->setCurrentIndex (a + 3);
}

bool ImportSimDisplay::isChecked ()  {
  return simBox->isChecked ();
}

int ImportSimDisplay::getParaFlags ()  {
  int flags = NONE;
  if (alienBox->isChecked ()) flags |= ALIEN;
  if (zombieBox->isChecked ()) flags |= ZOMBIE;
  if (vampireBox->isChecked ()) flags |= VAMPIRE;
  if (servoBox->isChecked ()) flags |= SERVO;
  if (werewolfBox->isChecked ()) flags |= WEREWOLF;
  if (plantsimBox->isChecked ()) flags |= PLANTSIM;
  if (bigfootBox->isChecked ()) flags |= BIGFOOT;
  if (witchBox->isChecked ()) flags |= WITCH;

  return flags;
}

QString ImportSimDisplay::getHouseholdName ()  {
  return householdEdit->text ();
}

int ImportSimDisplay::getAlignment ()  {
  return alignmentBox->currentIndex () - 3;
}

void ImportSimDisplay::updateDisplay ()  {
  QString text = name + " (" + gender + "), " + aspiration;
  if (epFlags & FREETIME) text += "/" + saspiration;
  text += " " + age;
  if (epFlags & PETS) text += " " + species;

  simBox->setText (text);

  if (species != "Human")  {
    alienBox->setEnabled (false);
    zombieBox->setEnabled (false);
    vampireBox->setEnabled (false);
    servoBox->setEnabled (false);
    werewolfBox->setEnabled (false);
    plantsimBox->setEnabled (false);
    bigfootBox->setEnabled (false);
    witchBox->setEnabled (false);
  }

  else if (age != "Adult" && age != "Ghost")  {
    alienBox->setEnabled (true);
    servoBox->setEnabled (false);

    if (age != "Toddler" && age != "Elder")
      plantsimBox->setEnabled (false);
    else if (epFlags & SEASONS) plantsimBox->setEnabled (true);

    if (age != "Elder")
      bigfootBox->setEnabled (false);
    else if (epFlags & BONVOYAGE) bigfootBox->setEnabled (true);

    if (age != "Teen" && age != "Elder" && age != "Young Adult")  {
      zombieBox->setEnabled (false);
      vampireBox->setEnabled (false);
      werewolfBox->setEnabled (false);
      witchBox->setEnabled (false);
    }

    else  {
      if (epFlags & UNIVERSITY) zombieBox->setEnabled (true);
      if (epFlags & NIGHTLIFE) vampireBox->setEnabled (true);
      if (epFlags & PETS) werewolfBox->setEnabled (true);
      if (epFlags & APARTMENTLIFE) witchBox->setEnabled (true);
    }
  }

  else  {
    alienBox->setEnabled (true);
    if (epFlags & UNIVERSITY) zombieBox->setEnabled (true);
    if (epFlags & NIGHTLIFE) vampireBox->setEnabled (true);
    if (epFlags & OFB) servoBox->setEnabled (true);
    if (epFlags & PETS) werewolfBox->setEnabled (true);
    if (epFlags & SEASONS) plantsimBox->setEnabled (true);
    if (epFlags & BONVOYAGE) bigfootBox->setEnabled (true);
    if (epFlags & APARTMENTLIFE) witchBox->setEnabled (true);
  }
}

