#include <QStringList>
#include <QString>
#include <QVBoxLayout>
#include <QMessageBox>

#include "importsimspage.h"
#include "importsimdisplay.h"
#include "const.h"

ImportSimsPage::ImportSimsPage ()  {
  type = PLAYABLE;
  pageNumber = 0;
  epFlags = NONE;

  for (int x = 0; x < 5; x++)
    simDisplay.append (new ImportSimDisplay ());

  mainLayout = new QVBoxLayout;
  for (int x = 0; x < 5; x++)
    mainLayout->addWidget (simDisplay[x]);

  setLayout (mainLayout);
}

void ImportSimsPage::setType (int t)  {
  type = t;
}

void ImportSimsPage::setNumber (int n)  {
  pageNumber = n;
}

void ImportSimsPage::setData (QList<QStringList> data)  {
  for (int x = 0; x < 5; x++)
    simDisplay[x]->show ();

  for (int x = 0; x < data.size (); x++)  {
    simDisplay[x]->setEPFlags (epFlags);
    simDisplay[x]->setName (data[x][3] + " " +  data[x][4]);
    simDisplay[x]->setHousehold (data[x][7]);
    simDisplay[x]->setGender (data[x][11]);
    if (data[x][14] == "Y") simDisplay[x]->setLifeStage ("Young Adult");
    else if (data[x][6].toInt () == 0) simDisplay[x]->setLifeStage ("Ghost");
    else simDisplay[x]->setLifeStage (data[x][12]);
    simDisplay[x]->setAspiration (data[x][18], data[x][19]);
    simDisplay[x]->setSpecies (data[x][17]);

    simDisplay[x]->setChecked (data[x][21] == "Y");
    simDisplay[x]->setParaFlags (data[x][22].toInt ());
    simDisplay[x]->setAlignment (data[x][23].toInt ());
  }

  if (data.size () < 5)
    for (int x = data.size (); x < 5; x++)
      simDisplay[x]->hide ();

  mainLayout->addStretch (1);
}

void ImportSimsPage::setEPFlags (int flags)  {
  epFlags = flags;
}

int ImportSimsPage::getType ()  {
  return type;
}

int ImportSimsPage::getNumber ()  {
  return pageNumber;
}

QString ImportSimsPage::getTitle ()  {
  QString text = "";

  if (type == PLAYABLE) text += "Playables: ";
  else if (type == TOWNIE) text += "Townies: ";
  else text += "NPCs: ";

  text += "page " + QString::number (pageNumber);

  return text;
}

bool ImportSimsPage::isChecked (int n)  {
  if (n >= 0 && n < simDisplay.size ())
    return simDisplay[n]->isChecked ();
  else return false;
}

int ImportSimsPage::getParaFlags (int n)  {
  if (n >= 0 && n < simDisplay.size ())
    return simDisplay[n]->getParaFlags ();
  else return NONE;
}

QString ImportSimsPage::getHouseholdName (int n)  {
  if (n >= 0 && n < simDisplay.size ())
    return simDisplay[n]->getHouseholdName ();
  else return "";
}

int ImportSimsPage::getAlignment (int n)  {
  if (n >= 0 && n < simDisplay.size ())
    return simDisplay[n]->getAlignment ();
  else return NEUTRAL;
}
