#include <QString>
#include <QStringList>
#include <QLabel>
#include <QPushButton>
#include <QComboBox>
#include <QLineEdit>
#include <QCheckBox>
#include <QGroupBox>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QGridLayout>
#include <QDir>
#include <QFileDialog>
#include <QMessageBox>

#include <string>
#include <windows.h>
#include <tchar.h>
using namespace std;

#include "const.h"
#include "listmanagementdialog.h"
#include "importdialog.h"

ListManagementDialog::ListManagementDialog ()  {
  importDialog = NULL;
  epFlags = NONE;

  QDir dir (".");
  dir.setNameFilters (QStringList ("*.sls"));
  QStringList files = dir.entryList ();

  for (int x = 0; x < files.size (); x++)
    files[x].chop (4);

  files.sort ();

  openGroupBox = new QGroupBox ("Open a list");
  openTopLayout = new QHBoxLayout ();
  openBottomLayout = new QHBoxLayout ();
  openMainLayout = new QVBoxLayout ();
  openMainLayout->addLayout (openTopLayout);
  openMainLayout->addLayout (openBottomLayout);
  openGroupBox->setLayout (openMainLayout);

  openButton = new QPushButton ("Open");
  openButton->setMaximumSize (openButton->sizeHint ());
  openButton->setEnabled (files.size () > 0);
  openComboBox = new QComboBox ();
  openComboBox->setEditable (false);
  openComboBox->addItems (files);
  QLabel *fileFormatLabel = new QLabel ("File format: ");
  fileFormatComboBox = new QComboBox ();
  fileFormatComboBox->addItem ("Version 0.1");
  fileFormatComboBox->addItem ("Version 0.2");
  fileFormatComboBox->addItem ("Version 0.3");
  fileFormatComboBox->addItem ("Version 0.4");
  fileFormatComboBox->addItem ("This version (0.5)");
  fileFormatComboBox->setCurrentIndex (V0_5);
  fileFormatComboBox->setMaximumSize (fileFormatComboBox->sizeHint ());

  openTopLayout->addWidget (openButton);
  openTopLayout->addWidget (openComboBox);
  openBottomLayout->addWidget (fileFormatLabel);
  openBottomLayout->addWidget (fileFormatComboBox);

  saveGroupBox = new QGroupBox ("Save the current list");
  saveLayout = new QHBoxLayout ();
  saveGroupBox->setLayout (saveLayout);

  saveButton = new QPushButton ("Save As");
  saveButton->setMaximumSize (saveButton->sizeHint ());
  saveButton->setEnabled (files.size () > 0);
  saveComboBox = new QComboBox ();
  saveComboBox->setEditable (true);
  saveComboBox->addItems (files);

  saveLayout->addWidget (saveButton);
  saveLayout->addWidget (saveComboBox);

  setDebugGroupBox = new QGroupBox ("Set a premade debug list");
  setDebugLayout = new QHBoxLayout ();
  setDebugGroupBox->setLayout (setDebugLayout);

  setDebugButton = new QPushButton ("Set to");
  setDebugButton->setMaximumSize (setDebugButton->sizeHint ()); 
  setDebugButton->setEnabled (true);
  setDebugComboBox = new QComboBox ();
  setDebugComboBox->setEditable (false);
  setDebugComboBox->addItem ("50 Random Sims");
  setDebugComboBox->addItem ("One of Each Type");

  setDebugLayout->addWidget (setDebugButton);
  setDebugLayout->addWidget (setDebugComboBox);

  simPEImportGroupBox = new QGroupBox ("Import Sims from SimPE");
  simPELocationLayout = new QHBoxLayout;
  simPEImportLayout = new QHBoxLayout;
  simPELayout = new QVBoxLayout;
  simPELayout->addLayout (simPELocationLayout);
  simPELayout->addLayout (simPEImportLayout);
  simPEImportGroupBox->setLayout (simPELayout);

  simPEBrowseButton = new QPushButton ("Browse to SimPE");
  simPEBrowseButton->setMaximumSize (simPEBrowseButton->sizeHint ());
  simPEImportButton = new QPushButton ("Import Sims");
  simPEImportButton->setMaximumSize (simPEImportButton->sizeHint ());
  simPEImportButton->setEnabled (false);
  simPELocationEdit = new QLineEdit;
  simPELocationEdit->setText ("C:/Program Files/SimPE/");
  simPELocationEdit->setCursorPosition (0);
  QLabel *nLabel = new QLabel ("Neighborhood: ");
  simPENeighborhoodsBox = new QComboBox;
  importFromFileBox = new QCheckBox ("Import from previously created output");

  WCHAR wstr[32767];
  DWORD buffSize = 32767;
  GetUserName (wstr, &buffSize);

  QFileInfoList driveList = QDir::drives ();

  QDir neighborhoodsDir;
  for (int x = 0; x < driveList.size (); x++)  {
    neighborhoodsDir.setPath (driveList[x].absoluteFilePath () + 
        "/Documents and Settings/" + QString::fromStdWString (wstr) + 
        "/My Documents/EA Games/The Sims 2/Neighborhoods/");
    if (neighborhoodsDir.exists ()) break;
  }

  QRegExp nRegExp ("\\w{4}");
  QStringList nList = neighborhoodsDir.entryList ();
  for (int x = 0; x < nList.size (); x++)
    if (!nRegExp.exactMatch (nList[x]))  {
      nList.removeAt (x);
      x--;
    }

  simPENeighborhoodsBox->addItems (nList);
  simPENeighborhoodsBox->setEditable (true);

  QDir simPEDir (simPELocationEdit->text ());
  if (simPEDir.exists ("SimPE.exe"))
    simPEImportButton->setEnabled (true);

  simPELocationLayout->addWidget (simPELocationEdit);
  simPELocationLayout->addWidget (simPEBrowseButton);
  simPEImportLayout->addWidget (nLabel);
  simPEImportLayout->addWidget (simPENeighborhoodsBox);
  simPEImportLayout->addWidget (simPEImportButton);
  simPELayout->addWidget (importFromFileBox);

  clearButton = new QPushButton ("Clear current list");

  closeButton = new QPushButton ("Close");
  closeButton->setMaximumSize (closeButton->sizeHint ());

  mainLayout = new QGridLayout ();
  mainLayout->addWidget (openGroupBox, 0, 0);
  mainLayout->addWidget (saveGroupBox, 0, 1);
  mainLayout->addWidget (setDebugGroupBox, 1, 0);
  mainLayout->addWidget (simPEImportGroupBox, 1, 1);
  mainLayout->addWidget (clearButton, 2, 0);
  mainLayout->setAlignment (clearButton, Qt::AlignVCenter | Qt::AlignRight);
  mainLayout->addWidget (closeButton, 2, 1);
  mainLayout->setAlignment (closeButton, Qt::AlignVCenter | Qt::AlignLeft);

  setLayout (mainLayout);

  connect (openButton, SIGNAL (clicked ()), this, SLOT (open ()));
  connect (saveButton, SIGNAL (clicked ()), this, SLOT (save ()));
  connect (saveButton, SIGNAL (clicked ()), this, SLOT (refreshFileList ()));
  connect (setDebugButton, SIGNAL (clicked ()), this, SLOT (setDebug ()));
  connect (simPEBrowseButton, SIGNAL (clicked ()), this, 
           SLOT (browseForSimPE ()));
  connect (simPEImportButton, SIGNAL (clicked ()), this,
           SLOT (importSims ()));
  connect (clearButton, SIGNAL (clicked ()), this, SIGNAL (clearList ()));
  connect (closeButton, SIGNAL (clicked ()), this, SLOT (hide ()));

  connect (openComboBox, SIGNAL (currentIndexChanged (QString)), this,
           SLOT (validate ()));
  connect (saveComboBox, SIGNAL (currentIndexChanged (QString)), this, 
           SLOT (validate ()));
  connect (saveComboBox, SIGNAL (editTextChanged (QString)), this,
           SLOT (validate ()));
  connect (setDebugComboBox, SIGNAL (currentIndexChanged (QString)), this,
           SLOT (validate ()));
  connect (simPELocationEdit, SIGNAL (textChanged (QString)), this,
           SLOT (validate ()));
}

void ListManagementDialog::setEPFlags (int flags)  {
  epFlags = flags;
}

void ListManagementDialog::setSimPELocation (QString loc)  {
  simPELocationEdit->setText (loc);
}

void ListManagementDialog::open ()  {
  emit openList (openComboBox->currentText (), 
                 fileFormatComboBox->currentIndex ());
  clear ();
}

void ListManagementDialog::save ()  {
  emit saveList (saveComboBox->currentText ());
  QMessageBox::information (this, "Save Completed", "List has been saved as " +
                            saveComboBox->currentText () + ".sls");
  clear ();
}

void ListManagementDialog::setDebug ()  {
  emit setDebugList (setDebugComboBox->currentIndex ());
  clear ();
}

void ListManagementDialog::validate ()  {
  if (openComboBox->currentText () != "")
    openButton->setEnabled (true);
  else openButton->setEnabled (false);

  if (saveComboBox->currentText () != "")
    saveButton->setEnabled (true);
  else saveButton->setEnabled (false);

  if (setDebugComboBox->currentText () != "")
    setDebugButton->setEnabled (true);
  else setDebugButton->setEnabled (false);

  QDir simPEDir (simPELocationEdit->text ());
  if (simPEDir.exists ("SimPE.exe"))
    simPEImportButton->setEnabled (true);
  else simPEImportButton->setEnabled (false);
}

void ListManagementDialog::refreshFileList ()  {
  openComboBox->clear ();
  saveComboBox->clear ();

  QDir dir (".");
  dir.setNameFilters (QStringList ("*.sls"));
  QStringList files = dir.entryList ();

  for (int x = 0; x < files.size (); x++)
    files[x].chop (4);

  files.sort ();

  openComboBox->addItems (files);
  saveComboBox->addItems (files);
}

void ListManagementDialog::browseForSimPE ()  {
  simPELocationEdit->setText (QFileDialog::getExistingDirectory (this, 
      "Browse to the location of SimPE.exe", "C:\\") + "/");

  simPELocationEdit->setCursorPosition (0);
}

void ListManagementDialog::importSims ()  {
  QString path = simPELocationEdit->text ();
  path.replace ("/", "\\");
  path.prepend ("extract.bat \"");
  path += "SimPE.exe\" ";
//  path += "-rufio -hood " + simPENeighborhoodsBox->currentText ();

  QDir simSheetDir (".");
  QString simSheetPath = simSheetDir.absolutePath ();
  simSheetPath.replace ("/", "\\");
  path += "\"" + simSheetPath + "\"";
  path += " " + simPENeighborhoodsBox->currentText ();

//  QMessageBox::information (this, "", path);

  string pathStr = path.toStdString ();

  if (!importFromFileBox->isChecked ())
    system (pathStr.c_str ());

/*
  WCHAR wstr[32767];
  DWORD buffSize = 32767;
  GetUserName (wstr, &buffSize);

  QDir rootDir ("/");
  QFileInfoList driveList = QDir::drives ();

  QDir simsDir;
  for (int x = 0; x < driveList.size (); x++)  {
    simsDir.setPath (driveList[x].absoluteFilePath () + 
        "/Documents and Settings/" + QString::fromStdWString (wstr) + 
        "/My Documents/EA Games/The Sims 2/");
    if (simsDir.exists ("Rufio/ExportedSims.txt")) break;
  }
*/
  if (!simSheetDir.exists ("Rufio/ExportedSims.txt"))
    QMessageBox::information (this, "", (QString)"Cannot find output file.  " +
                                        (QString)"Check that you are using " +
                                        (QString)"the AL version of SimPE.");

  else  {
    QMessageBox::information (this, "", "Generated list of sims.");
    if (importDialog) delete importDialog;
    importDialog = new ImportDialog (simSheetDir.absolutePath (), epFlags);

    connect (importDialog, SIGNAL (addSim (Sim*)), this, 
             SIGNAL (addSim (Sim*)));
    connect (importDialog, SIGNAL (updateModels ()), this, 
             SIGNAL (updateModels ()));
    importDialog->show ();
  }
}

void ListManagementDialog::clear ()  {
  openComboBox->setCurrentIndex (0);
  saveComboBox->setCurrentIndex (0);
  saveComboBox->clearEditText ();
  setDebugComboBox->setCurrentIndex (0);
}
