#ifndef LISTMANAGEMENTDIALOG_H
#define LISTMANAGEMENTDIALOG_H

#include <QDialog>

class QString;
class QLabel;
class QComboBox;
class QLineEdit;
class QCheckBox;
class QGroupBox;
class QHBoxLayout;
class QVBoxLayout;
class QGridLayout;
class ImportDialog;
class Sim;

class ListManagementDialog : public QDialog {
  Q_OBJECT

  public:
    ListManagementDialog ();

    void setEPFlags (int);
    void setSimPELocation (QString);

  signals:
    void openList (QString, int);
    void saveList (QString);
    void setDebugList (int);
    void clearList ();

    void addSim (Sim*);
    void updateModels ();

  private slots:
    void open ();
    void save ();
    void setDebug ();

    void validate ();
    void refreshFileList ();

    void browseForSimPE ();
    void importSims ();

  private:
    void clear ();

    int epFlags;

    ImportDialog *importDialog;

    QGroupBox *openGroupBox;
    QGroupBox *saveGroupBox;
    QGroupBox *setDebugGroupBox;
    QGroupBox *simPEImportGroupBox;

    QHBoxLayout *openTopLayout;
    QHBoxLayout *openBottomLayout;
    QVBoxLayout *openMainLayout;
    QHBoxLayout *saveLayout;
    QHBoxLayout *setDebugLayout;
    QHBoxLayout *simPELocationLayout;
    QHBoxLayout *simPEImportLayout;
    QVBoxLayout *simPELayout;

    QPushButton *openButton;
    QPushButton *saveButton;
    QPushButton *setDebugButton;
    QPushButton *clearButton;
    QPushButton *closeButton;
    QPushButton *simPEBrowseButton;
    QPushButton *simPEImportButton;

    QComboBox *openComboBox;
    QComboBox *fileFormatComboBox;
    QComboBox *saveComboBox;
    QComboBox *setDebugComboBox;
    QLineEdit *simPELocationEdit;
    QComboBox *simPENeighborhoodsBox;

    QCheckBox *importFromFileBox;

    QGridLayout *mainLayout;
};

#endif
