#include <QLabel>
#include <QSpinBox>
#include <QCheckBox>
#include <QComboBox>
#include <QPushButton>
#include <QLineEdit>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QGridLayout>
#include <QMessageBox>
#include <QFileDialog>

#include <iostream>
using namespace std;

#include "const.h"
#include "settingsdialog.h"

SettingsDialog::SettingsDialog (int rt, bool cr, int crt, bool ac, int act,
                                bool ab, int abt, int ad, int ep, QString sl) {
  QLabel *rotationLabel = new QLabel ("Every update, increase time by ");
  rotationSpinBox = new QSpinBox ();
  rotationSpinBox->setRange (1, 14);
  rotationSpinBox->setSuffix (" sim days");
  rotationSpinBox->setValue (rt);

  rotationLayout = new QHBoxLayout ();
  rotationLayout->addWidget (rotationLabel);
  rotationLayout->addWidget (rotationSpinBox);

  collegeRotationCheckBox = new QCheckBox
                                ("Every update, increase time in college by ");
  collegeRotationCheckBox->setChecked (cr);
  collegeRotationCheckBox->setEnabled (ep & UNIVERSITY);
  collegeRotationSpinBox = new QSpinBox ();
  collegeRotationSpinBox->setRange (1, 8);
  collegeRotationSpinBox->setSuffix (" semesters");
  collegeRotationSpinBox->setValue (crt);
  collegeRotationSpinBox->setEnabled (ep & UNIVERSITY);

  collegeRotationLayout = new QHBoxLayout ();
  collegeRotationLayout->addWidget (collegeRotationCheckBox);
  collegeRotationLayout->addWidget (collegeRotationSpinBox);

  connect (collegeRotationCheckBox, SIGNAL (toggled (bool)),
           collegeRotationSpinBox, SLOT (setEnabled (bool)));

  autoGoToCollegeCheckBox = new QCheckBox 
                      ("Automatically send teens to college when they reach ");
  autoGoToCollegeCheckBox->setChecked (ac);
  autoGoToCollegeCheckBox->setEnabled (ep & UNIVERSITY);
  autoGoToCollegeSpinBox = new QSpinBox ();
  autoGoToCollegeSpinBox->setRange (1, 14);
  autoGoToCollegeSpinBox->setSuffix (" sim days until adulthood");
  autoGoToCollegeSpinBox->setEnabled (ep & UNIVERSITY);
  autoGoToCollegeSpinBox->setValue (act);
  autoGoToCollegeSpinBox->setEnabled (false);

  autoGoToCollegeLayout = new QHBoxLayout ();
  autoGoToCollegeLayout->addWidget (autoGoToCollegeCheckBox);
  autoGoToCollegeLayout->addWidget (autoGoToCollegeSpinBox);

  connect (autoGoToCollegeCheckBox, SIGNAL (toggled (bool)),
           autoGoToCollegeSpinBox, SLOT (setEnabled (bool)));

  autoBirthdayCheckBox = new QCheckBox 
       ("Automatically add age transitions as updates (by default occurring ");
  autoBirthdayCheckBox->setChecked (ab);
  autoBirthdayComboBox = new QComboBox ();
  autoBirthdayComboBox->addItem ("before");
  autoBirthdayComboBox->addItem ("at or after");
  autoBirthdayComboBox->setCurrentIndex (abt);
  QLabel *autoBirthdayLabel = new QLabel ("six PM).");

  autoBirthdayLayout = new QHBoxLayout ();
  autoBirthdayLayout->addWidget (autoBirthdayCheckBox);
  autoBirthdayLayout->addWidget (autoBirthdayComboBox);
  autoBirthdayLayout->addWidget (autoBirthdayLabel);

  connect (autoBirthdayCheckBox, SIGNAL (toggled  (bool)),
           autoBirthdayComboBox, SLOT (setEnabled (bool)));

  QLabel *ageDisplayLabel = new QLabel ("Age display format: ");
  ageDisplayComboBox = new QComboBox ();
  ageDisplayComboBox->addItem ("## days old");
  ageDisplayComboBox->addItem ("## days until Lifestage");
  ageDisplayComboBox->setCurrentIndex (ad);

  ageDisplayLayout = new QHBoxLayout ();
  ageDisplayLayout->addWidget (ageDisplayLabel);
  ageDisplayLayout->addWidget (ageDisplayComboBox);

  QLabel *simPELocationLabel = new QLabel ("Default location of SimPE: ");
  simPELocationEdit = new QLineEdit (sl);
  findSimPEButton = new QPushButton ("Browse...");

  simPELocationLayout = new QHBoxLayout;
  simPELocationLayout->addWidget (simPELocationLabel);
  simPELocationLayout->addWidget (simPELocationEdit);
  simPELocationLayout->addWidget (findSimPEButton);

  QLabel *expansionPackLabel = 
    new QLabel ("Use features from the following expansion packs:");
  uniBox = new QCheckBox ("University");
  uniBox->setChecked (ep & UNIVERSITY);
  nightlifeBox = new QCheckBox ("Nightlife");
  nightlifeBox->setChecked (ep & NIGHTLIFE);
  ofbBox = new QCheckBox ("Open for Business");
  ofbBox->setChecked (ep & OFB);
  petsBox = new QCheckBox ("Pets");
  petsBox->setChecked (ep & PETS);
  seasonsBox = new QCheckBox ("Seasons");
  seasonsBox->setChecked (ep & SEASONS);
  bvBox = new QCheckBox ("Bon Voyage");
  bvBox->setChecked (ep & BONVOYAGE);
  freetimeBox = new QCheckBox ("Free Time");
  freetimeBox->setChecked (ep & FREETIME);
  alBox = new QCheckBox ("Apartment Life");
  alBox->setChecked (ep & APARTMENTLIFE);

  expansionPackLayout = new QGridLayout;
  expansionPackLayout->addWidget (expansionPackLabel, 0, 0, 1, 2);
  expansionPackLayout->addWidget (uniBox, 1, 0);
  expansionPackLayout->addWidget (nightlifeBox, 1, 1);
  expansionPackLayout->addWidget (ofbBox, 2, 0);
  expansionPackLayout->addWidget (petsBox, 2, 1);
  expansionPackLayout->addWidget (seasonsBox, 3, 0);
  expansionPackLayout->addWidget (bvBox, 3, 1);
  expansionPackLayout->addWidget (freetimeBox, 4, 0);
  expansionPackLayout->addWidget (alBox, 4, 1);

  closeButton = new QPushButton ("Close");
  closeButton->setDefault (true);

  mainLayout = new QVBoxLayout ();
  mainLayout->addLayout (rotationLayout);
  mainLayout->addLayout (collegeRotationLayout);
  mainLayout->addLayout (autoGoToCollegeLayout);
  mainLayout->addLayout (autoBirthdayLayout);
  mainLayout->addLayout (ageDisplayLayout);
  mainLayout->addLayout (simPELocationLayout);
  mainLayout->addLayout (expansionPackLayout);
  mainLayout->addWidget (closeButton);
  mainLayout->setAlignment (closeButton, Qt::AlignCenter);

  setLayout (mainLayout);

  connect (rotationSpinBox, SIGNAL (valueChanged (int)), this,
           SIGNAL (setRotation (int)));
  connect (collegeRotationCheckBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setAutoCollegeRotation (bool)));
  connect (collegeRotationSpinBox, SIGNAL (valueChanged (int)), this,
           SIGNAL (setCollegeRotation (int)));
  connect (autoGoToCollegeCheckBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setAutoGoToCollege (bool)));
  connect (autoGoToCollegeSpinBox, SIGNAL (valueChanged (int)), this,
           SIGNAL (setGoToCollegeTime (int)));
  connect (autoBirthdayCheckBox, SIGNAL (toggled (bool)), this,
           SIGNAL (setAutoBirthday (bool)));
  connect (autoBirthdayComboBox, SIGNAL (currentIndexChanged (int)), this,
           SIGNAL (setAutoBirthdayTime (int)));
  connect (ageDisplayComboBox, SIGNAL (currentIndexChanged (int)), this,
           SIGNAL (setAgeDisplay (int)));
  connect (simPELocationEdit, SIGNAL (textChanged (QString)), this,
           SIGNAL (setSimPELocation (QString)));
  connect (findSimPEButton, SIGNAL (clicked ()), this, 
           SLOT (browseForSimPE ()));

  connect (uniBox, SIGNAL (toggled (bool)), this, SLOT (updateEPFlags ()));
  connect (nightlifeBox, SIGNAL (toggled (bool)), this, SLOT (updateEPFlags()));
  connect (ofbBox, SIGNAL (toggled (bool)), this, SLOT (updateEPFlags ()));
  connect (petsBox, SIGNAL (toggled (bool)), this, SLOT (updateEPFlags ()));
  connect (seasonsBox, SIGNAL (toggled (bool)), this, SLOT (updateEPFlags ()));
  connect (bvBox, SIGNAL (toggled (bool)), this, SLOT (updateEPFlags ()));
  connect (freetimeBox, SIGNAL (toggled (bool)), this, SLOT (updateEPFlags ()));
  connect (alBox, SIGNAL (toggled (bool)), this, SLOT (updateEPFlags ()));

  connect (closeButton, SIGNAL (clicked ()), this, SLOT (hide ()));
}

QString SettingsDialog::getCSV ()  {
  QString text = "";

  text += QString::number (rotationSpinBox->value ()) + "," + 
          QString::number ((int)collegeRotationCheckBox->isChecked ()) + "," +
          QString::number (collegeRotationSpinBox->value ()) + "," +
          QString::number ((int)autoGoToCollegeCheckBox->isChecked ()) + "," +
          QString::number (autoGoToCollegeSpinBox->value ()) + "," +
          QString::number ((int)autoBirthdayCheckBox->isChecked ()) + "," +
          QString::number (autoBirthdayComboBox->currentIndex ()) + "," +
          QString::number (ageDisplayComboBox->currentIndex ()) + "," +
          simPELocationEdit->text () + "," +
          QString::number ((int)uniBox->isChecked ()) + "," +
          QString::number ((int)nightlifeBox->isChecked ()) + "," +
          QString::number ((int)ofbBox->isChecked ()) + "," +
          QString::number ((int)petsBox->isChecked ()) + "," +
          QString::number ((int)seasonsBox->isChecked ()) + "," +
          QString::number ((int)bvBox->isChecked ()) + "," +
          QString::number ((int)freetimeBox->isChecked ()) + "," +
          QString::number ((int)alBox->isChecked ()) + ",";

  return text;
}

void SettingsDialog::readCSV (QString csv, int format)  {
  cout << "Settings dialog" << endl;

  if (format == V0_1 || format == V0_2)  {
    readCSV0_2 (csv);
    return;
  }

  if (format == V0_3 || format == V0_4)  {
    readCSV0_4 (csv);
    return;
  }

  QStringList list = csv.split (',');
  cout << list.size () << endl;

  rotationSpinBox->setValue (list[0].toInt ());
  collegeRotationCheckBox->setChecked (list[1].toInt ());
  collegeRotationSpinBox->setValue (list[2].toInt ());
  autoGoToCollegeCheckBox->setChecked (list[3].toInt ());
  autoGoToCollegeSpinBox->setValue (list[4].toInt ());
  autoBirthdayCheckBox->setChecked (list[5].toInt ());
  autoBirthdayComboBox->setCurrentIndex (list[6].toInt ());
  ageDisplayComboBox->setCurrentIndex (list[7].toInt ());
  simPELocationEdit->setText (list[8]);
  uniBox->setChecked ((list[9].toInt ()));
  nightlifeBox->setChecked (list[10].toInt ());
  ofbBox->setChecked (list[11].toInt ());
  petsBox->setChecked (list[12].toInt ());
  seasonsBox->setChecked (list[13].toInt ());
  bvBox->setChecked (list[14].toInt ());
  freetimeBox->setChecked (list[15].toInt ());
  alBox->setChecked (list[16].toInt ());
}

void SettingsDialog::readCSV0_4 (QString csv)  {
  QStringList list = csv.split (',');
  cout << list.size () << endl;

  rotationSpinBox->setValue (list[0].toInt ());
  collegeRotationCheckBox->setChecked (list[1].toInt ());
  collegeRotationSpinBox->setValue (list[2].toInt ());
  autoGoToCollegeCheckBox->setChecked (list[3].toInt ());
  autoGoToCollegeSpinBox->setValue (list[4].toInt ());
  autoBirthdayCheckBox->setChecked (list[5].toInt ());
  autoBirthdayComboBox->setCurrentIndex (list[6].toInt ());
  ageDisplayComboBox->setCurrentIndex (list[7].toInt ());
  uniBox->setChecked ((list[8].toInt ()));
  nightlifeBox->setChecked (list[9].toInt ());
  ofbBox->setChecked (list[10].toInt ());
  petsBox->setChecked (list[11].toInt ());
  seasonsBox->setChecked (list[12].toInt ());
  bvBox->setChecked (list[13].toInt ());
  freetimeBox->setChecked (list[14].toInt ());
  alBox->setChecked (list[15].toInt ());
}

void SettingsDialog::readCSV0_2 (QString csv)  {
  QStringList list = csv.split (',');
  cout << list.size () << endl;

  rotationSpinBox->setValue (list[0].toInt ());
  collegeRotationCheckBox->setChecked (list[1].toInt ());
  collegeRotationSpinBox->setValue (list[2].toInt ());
  autoGoToCollegeCheckBox->setChecked (list[3].toInt ());
  autoGoToCollegeSpinBox->setValue (list[4].toInt ());
  autoBirthdayCheckBox->setChecked (list[5].toInt ());
  autoBirthdayComboBox->setCurrentIndex (list[6].toInt ());
  ageDisplayComboBox->setCurrentIndex (list[7].toInt ());
}

void SettingsDialog::updateEPFlags ()  {
  int flags = 0;

  if (uniBox->isChecked ()) flags += UNIVERSITY;
  if (nightlifeBox->isChecked ()) flags += NIGHTLIFE;
  if (ofbBox->isChecked ()) flags += OFB;
  if (petsBox->isChecked ()) flags += PETS;
  if (seasonsBox->isChecked ()) flags += SEASONS;
  if (bvBox->isChecked ()) flags += BONVOYAGE;
  if (freetimeBox->isChecked ()) flags += FREETIME;
  if (alBox->isChecked ()) flags += APARTMENTLIFE;

  if (uniBox->isChecked ())  {
    if (collegeRotationCheckBox->isChecked ())
      collegeRotationSpinBox->setEnabled (true);
    if (autoGoToCollegeCheckBox->isChecked ())
      autoGoToCollegeSpinBox->setEnabled (true);

    collegeRotationCheckBox->setEnabled (true);
    autoGoToCollegeCheckBox->setEnabled (true);
  }

  else  {
    collegeRotationCheckBox->setChecked (false);
    collegeRotationCheckBox->setEnabled (false);
    autoGoToCollegeCheckBox->setChecked (false);
    autoGoToCollegeCheckBox->setEnabled (false);
  }

  emit setEPFlags (flags);
}

void SettingsDialog::browseForSimPE ()  {
  simPELocationEdit->setText (QFileDialog::getExistingDirectory (this,
        "Browse to the location of SimPE.exe", "C:\\") + "/");

  simPELocationEdit->setCursorPosition (0);
}
