#include <QString>
#include <QStringList>
#include <QTextStream>
#include <QMessageBox>

#include <iostream>
using namespace std;

#include "const.h"
#include "sim.h"

Sim::Sim (QObject *parent)  
 : QObject (parent), parent1 (), parent2 (), spouse ()
{
  name = "";
  nid = -1;
  species = HUMAN;
  age = 0;
  hold = 0;
  household = "At Large";
  semester = 0;
  gender = UNKNOWN;
  youngAdult = false;
  major = "";
  aspiration.append (NONE);
  secondaryAspiration.append (NONE);
  ltwDescriptions.append (QStringList ());
  ltwDescriptions[0].clear ();
  ltwSteps.append (QList<int> ());
  ltwSteps[0].clear ();
  ltwCompleted.append (QList<int> ());
  ltwCompleted[0].clear ();

  zombie = false;
  vampire = false;
  servo = false;
  werewolf = false;
  plantsim = false;
  bigfoot = false;
  witch = false;
  alignment = NEUTRAL;

  alive = true;
  ghostColor = WHITE;
  notes = "";
  imageFile = "";
}

// This is silly.  I shouldn't be passing the alive value - every sim that gets
// created is going to start out alive.  But if I change it then I'll have to
// go through and changin approximately 500000000000000000 signals and slots in
// about as many GUI classes, and I'm *really* not looking forward to that.
// Come to that, I should note that I probably don't need this anymore, which
// is why it's so out of date.
Sim::Sim (QString n, int a, int g, int asp, int sasp, bool al, int para, 
          QString nt, QString fn)  
{
  name = n;
  nid = -1;
  age = a;
  hold = 0;
  household = "At Large";
  semester = 0;
  gender = g;
  aspiration.append (asp);
  secondaryAspiration.append (sasp);
  ltwDescriptions.append (QStringList ());
  ltwDescriptions[0].clear ();
  ltwSteps.append (QList<int> ());
  ltwSteps[0].clear ();
  ltwCompleted.append (QList<int> ());
  ltwCompleted[0].clear ();
  alive = al;
  ghostColor = WHITE;
  notes = nt;
  imageFile = fn;
  major = "";

  if (age == YOUNG_ADULT)  {
    age = ADULT;
    youngAdult = true;
    major = "Undeclared";
    semester = 1;
  }

  else youngAdult = false;

  alien = para & ALIEN;
  zombie = para & ZOMBIE;
  vampire = para & VAMPIRE;
  servo = para & SERVO;
  werewolf = para & WEREWOLF;
  plantsim = para & PLANTSIM;
  bigfoot = para & BIGFOOT;
  witch = para & WITCH;
  alignment = NEUTRAL;

  if (para & LARGE_DOG) species = LARGE_DOG;
  else if (para & SMALL_DOG) species = SMALL_DOG;
  else if (para & CAT) species = CAT;
  else species = HUMAN;
}

// Same with this one.
void Sim::setInitial (QString n, int a, int g, int asp, bool al, int para,
                      QString nt)  {
  name = n;
  age = a;
  gender = g;
  aspiration.append (asp);
  secondaryAspiration.append (NONE);
  ltwDescriptions.append (QStringList ());
  ltwSteps.append (QList<int> ());
  ltwCompleted.append (QList<int> ());
  alive = al;
  notes = nt;

  if (age == YOUNG_ADULT)  {
    age = ADULT;
    youngAdult = true;
    semester = 1;
  }

  else youngAdult = false;

  alien = para & ALIEN;
  zombie = para & ZOMBIE;
  vampire = para & VAMPIRE;
  servo = para & SERVO;
  werewolf = para & WEREWOLF;
  plantsim = para & PLANTSIM;
  bigfoot = para & BIGFOOT;
  witch = para & WITCH;
}

void Sim::readCSV (QString csv, int format)  {
  if (format == V0_1)  {
    household = "";
    readCSV0_1 (csv);
    return;
  }

  if (format == V0_2)  {
    readCSV0_2 (csv);
    return;
  }

  if (format == V0_3 || format == V0_4)  {
    readCSV0_30_4 (csv);
    return;
  }

  QStringList list = csv.split (',');
  int currentItem = 0;

  name = list[currentItem++];

  while (!name.endsWith ("\""))
    name += "," + list[currentItem++];

  name.chop (1);
  name.remove (0, 1);

  nid = list[currentItem++].toInt ();
  age = list[currentItem++].toInt ();
  gender = list[currentItem++].toInt ();

  while (list[currentItem].startsWith ("("))  {
    list[currentItem].remove (0, 1);
    aspiration.append (list[currentItem++].toInt ());
    ltwDescriptions.append (QStringList ());
    ltwSteps.append (QList<int> ());
    ltwCompleted.append (QList<int> ());

    if (!list[currentItem].endsWith (")"))  {
      secondaryAspiration.append (list[currentItem++].toInt ());

      do  {
        QString desc = list[currentItem++];
        desc.remove (0, 2);
        desc.chop (1);
        ltwDescriptions[ltwDescriptions.size () - 1].append (desc);

        ltwSteps[ltwSteps.size () - 1].append (list[currentItem++].toInt ());
        QString done = list[currentItem];
        if (done.endsWith ("))")) done.chop (2);
        else done.chop (1);
        ltwCompleted[ltwCompleted.size () - 1].append (done.toInt ());
      } while (!list[currentItem++].endsWith ("))"));
    }

    else  {
      list[currentItem].chop (1);
      secondaryAspiration.append (list[currentItem++].toInt ());
    }
  }

  household = list[currentItem++];

  while (!household.endsWith ("\""))
    household += "," + list[currentItem++];

  household.chop (1);
  household.remove (0, 1);

  youngAdult = (bool)(list[currentItem++].toInt ());

  major = list[currentItem++];

  while (!major.endsWith ("\""))
    major += "," + list[currentItem++];

  major.chop (1);
  major.remove (0, 1);

  semester = list[currentItem++].toInt ();
  alive = (bool)(list[currentItem++].toInt ());
  ghostColor = list[currentItem++].toInt ();
  hold = list[currentItem++].toInt ();

  int flags = list[currentItem++].toInt ();
  alien = flags & ALIEN;
  zombie = flags & ZOMBIE;
  vampire = flags & VAMPIRE;
  servo = flags & SERVO;
  werewolf = flags & WEREWOLF;
  plantsim = flags & PLANTSIM;
  bigfoot = flags & BIGFOOT;
  witch = flags & WITCH;

  species = HUMAN;
  if (flags & LARGE_DOG) species = LARGE_DOG;
  else if (flags & SMALL_DOG) species = SMALL_DOG;
  else if (flags & CAT) species = CAT;

  alignment = list[currentItem++].toInt ();

  imageFile = list[currentItem++];

  for (int x = currentItem; x < list.size (); x++)  {
    notes += list[x];
    if ((list.size () - 1) > x) notes += ",";
  }

  notes.replace (";;", "\r\n");
}

void Sim::readCSV0_30_4 (QString csv)  {
  QStringList list = csv.split (',');
  int currentItem = 0;

  name = list[currentItem++];

  while (!name.endsWith ("\""))
    name += "," + list[currentItem++];

  name.chop (1);
  name.remove (0, 1);

  age = list[currentItem++].toInt ();
  gender = list[currentItem++].toInt ();
  aspiration.append (list[currentItem++].toInt ());
  secondaryAspiration.append (list[currentItem++].toInt ());
  ltwDescriptions.append (QStringList ());
  ltwSteps.append (QList<int> ());
  ltwCompleted.append (QList<int> ());

  household = list[currentItem++];

  while (!household.endsWith ("\""))
    household += "," + list[currentItem++];

  household.chop (1);
  household.remove (0, 1);

  youngAdult = (bool)(list[currentItem++].toInt ());

  major = list[currentItem++];

  while (!major.endsWith ("\""))
    major += "," + list[currentItem++];

  major.chop (1);
  major.remove (0, 1);

  semester = list[currentItem++].toInt ();
  alive = (bool)(list[currentItem++].toInt ());
  ghostColor = list[currentItem++].toInt ();
  hold = list[currentItem++].toInt ();

  int flags = list[currentItem++].toInt ();
  alien = flags & ALIEN;
  zombie = flags & ZOMBIE;
  vampire = flags & VAMPIRE;
  servo = flags & SERVO;
  werewolf = flags & WEREWOLF;
  plantsim = flags & PLANTSIM;
  bigfoot = flags & BIGFOOT;

  species = HUMAN;
  if (flags & LARGE_DOG) species = LARGE_DOG;
  else if (flags & SMALL_DOG) species = SMALL_DOG;
  else if (flags & CAT) species = CAT;

  imageFile = list[currentItem++];

  for (int x = currentItem; x < list.size (); x++)  {
    notes += list[x];
    if ((list.size () - 1) > x) notes += ",";
  }

  notes.replace (";;", "\r\n");
}

void Sim::readCSV0_2 (QString csv)  {
  QStringList list = csv.split (',');
  int currentItem = 0;

  name = list[currentItem++];

  while (!name.endsWith ("\""))
    name += "," + list[currentItem++];

  name.chop (1);
  name.remove (0, 1);

  age = list[currentItem++].toInt ();
  gender = list[currentItem++].toInt ();
  aspiration.append (list[currentItem++].toInt ());
  secondaryAspiration.append (NONE);
  ltwDescriptions.append (QStringList ());
  ltwSteps.append (QList<int> ());
  ltwCompleted.append (QList<int> ());

  if (aspiration[0] == 1) aspiration[0] = GROW_UP;
  else aspiration[0]--;

  household = list[currentItem++];

  while (!household.endsWith ("\""))
    household += "," + list[currentItem++];

  household.chop (1);
  household.remove (0, 1);

  youngAdult = (bool)(list[currentItem++].toInt ());

  major = list[currentItem++];

  while (!major.endsWith ("\""))
    major += "," + list[currentItem++];

  major.chop (1);
  major.remove (0, 1);

  semester = list[currentItem++].toInt ();
  alive = (bool)(list[currentItem++].toInt ());
  ghostColor = list[currentItem++].toInt ();
  hold = list[currentItem++].toInt ();

  int flags = list[currentItem++].toInt ();
  alien = flags & ALIEN;
  zombie = flags & ZOMBIE;
  vampire = flags & VAMPIRE;
  servo = flags & SERVO;
  werewolf = flags & WEREWOLF;
  plantsim = flags & PLANTSIM;
  bigfoot = flags & BIGFOOT;

  species = HUMAN;
  if (flags & LARGE_DOG) species = LARGE_DOG;
  else if (flags & SMALL_DOG) species = SMALL_DOG;
  else if (flags & CAT) species = CAT;

  imageFile = list[currentItem++];

  for (int x = currentItem; x < list.size (); x++)  {
    notes += list[x];
    if ((list.size () - 1) > x) notes += ",";
  }

  notes.replace (";;", "\r\n");
}

void Sim::readCSV0_1 (QString csv)  {
  QStringList list = csv.split (',');
  int currentItem = 0;

  name = list[currentItem++];

  while (!name.endsWith ("\""))
    name += "," + list[currentItem++];

  name.chop (1);
  name.remove (0, 1);

  age = list[currentItem++].toInt ();
  gender = list[currentItem++].toInt ();
  aspiration.append (list[currentItem++].toInt ());
  secondaryAspiration.append (NONE);
  ltwDescriptions.append (QStringList ());
  ltwSteps.append (QList<int> ());
  ltwCompleted.append (QList<int> ());

  youngAdult = (bool)(list[currentItem++].toInt ());

  major = list[currentItem++];

  while (!major.endsWith ("\""))
    major += "," + list[currentItem++];

  major.chop (1);
  major.remove (0, 1);

  semester = list[currentItem++].toInt ();
  alive = (bool)(list[currentItem++].toInt ());
  hold = list[currentItem++].toInt ();

  int flags = list[currentItem++].toInt ();
  alien = flags & ALIEN;
  zombie = flags & ZOMBIE;
  vampire = flags & VAMPIRE;
  servo = flags & SERVO;
  werewolf = flags & WEREWOLF;
  plantsim = flags & PLANTSIM;
  bigfoot = flags & BIGFOOT;

  species = HUMAN;

  imageFile = list[currentItem++];

  for (int x = currentItem; x < list.size (); x++)  {
    notes += list[x];
    if ((list.size () - 1) > x) notes += ",";
  }

  notes.replace (";;", "\r\n");
}

void Sim::readFamilyInfo (QString csv, QList< QPointer<Sim> > simList)  {
  QStringList csvList = csv.split (",");

  if (simList.size () > csvList[0].toInt () && csvList[0].toInt () >= 0)
    setSpouse (simList[csvList[0].toInt ()]);

  for (int x = 1; x < csvList.size (); x++)
    if (simList.size () > csvList[x].toInt () && csvList[x].toInt () >= 0)
      addChild (simList[csvList[x].toInt ()]);
}

void Sim::update (QPointer<Sim> s, QList< QPointer<Sim> > list)  {
  bool foundParent1 = false;
  if (s->parent1)
    for (int x = 0; x < list.size (); x++)
      if (s->parent1->nid == list[x]->nid)  {
        parent1 = list[x];
        foundParent1 = true;
        break;
      }

  if (!foundParent1) parent1 = s->parent1;

  bool foundParent2 = false;
  if (s->parent2)
    for (int x = 0; x < list.size (); x++)
      if (s->parent2->nid == list[x]->nid)  {
        parent2 = list[x];
        foundParent2 = true;
        break;
      }

  if (!foundParent2) parent2 = s->parent2;

  for (int x = 0; x < s->children.size (); x++)  {
    bool foundChild = false;

    for (int y = 0; y < list.size (); y++)
      if (s->children[x]->nid == list[y]->nid)  {
        children.append (list[y]);
        foundChild = true;
        break;
      }

    if (!foundChild) children.append (s->children[x]);
  }

  bool foundSpouse = false;
  if (s->spouse)
    for (int x = 0; x < list.size (); x++)
      if (s->spouse->nid == list[x]->nid)  {
        spouse = list[x];
        foundSpouse = true;
       break;
      }

  if (!foundSpouse) spouse = s->spouse;

  name = s->name;
  species = s->species;
  age = s->age;
  household = s->household;
  semester = s->semester;
  gender = s->gender;
  aspiration.append (s->aspiration[s->aspiration.size () - 1]);
  secondaryAspiration.append 
    (s->secondaryAspiration[s->secondaryAspiration.size () - 1]);
  ltwDescriptions = s->ltwDescriptions;
  ltwSteps = s->ltwSteps;
  ltwCompleted = s->ltwCompleted;
  youngAdult = s->youngAdult;
  major = s->major;
  alive = s->alive;
}

QPointer<Sim> Sim::getParent1 ()  {
  return parent1;
}

QPointer<Sim> Sim::getParent2 ()  {
  return parent2;
}

QPointer<Sim> Sim::getChild (int i)  {
  if (children.size () > i) return children[i];
  else return NULL;
}

bool Sim::isChildOf (QPointer<Sim> p)  {
  if (parent1 == p) return true;
  if (parent2 == p) return true;
  return false;
}

QPointer<Sim> Sim::getSpouse ()  {
  return spouse;
}

bool Sim::onHold ()  {
  if (hold == 0) return false;

  else  {
    hold--;
    return true;
  }
}

QString Sim::getFullName ()  {
  return name;
}

int Sim::getNID ()  {
  return nid;
}

int Sim::getGender ()  {
  return gender;
}

int Sim::getAge ()  {
  return age;
}

bool Sim::canAge ()  {
  if (!alive) return false;
  if (household == "Adoption Queue" || household == "Townies and Dormies" ||
      household == "Downtown" || household == "Garden Club" ||
      household == "Service NPCs" || household == "Island Locals" ||
      household == "Mountain Locals" || household == "Asian Locals" ||
      household == "Tourists" || household == "Socialites" ||
      household == "Gearheads" || household == "Techies" ||
      household == "Jocks" || household == "Bohemians") return false;
  if (youngAdult) return true;
  if (vampire || zombie || servo) return false;
  return true;
}

QString Sim::getHousehold ()  {
  return household;
}

bool Sim::specialHousehold ()  {
  return (household == "Runaways" || household == "Adoption Queue" ||
          household == "Townies and Dormies" || household == "Downtown" || 
          household == "Garden Club" || household == "Service NPCs" || 
          household == "Island Locals" || household == "Mountain Locals" || 
          household == "Asian Locals" || household == "Tourists" || 
          household == "Socialites" || household == "Gearheads" || 
          household == "Techies" || household == "Jocks" || 
          household == "Bohemians" || household == "Hobby Mavens");
}

bool Sim::isTownie ()  {
  return (household == "Townies and Dormies" || household == "Downtown" || 
          household == "Garden Club" || household == "Service NPCs" || 
          household == "Island Locals" || household == "Mountain Locals" || 
          household == "Asian Locals" || household == "Tourists" || 
          household == "Socialites" || household == "Gearheads" || 
          household == "Techies" || household == "Jocks" || 
          household == "Bohemians" || household == "Hobby Mavens");
}

QString Sim::ageToQString ()  {
  QString text = "";

  if (servo) text += "Servo";

  else if (youngAdult)  {
    switch (semester)  {
      case 1:
      case 2: text += "Freshman"; break;
      case 3:
      case 4: text += "Sophomore"; break;
      case 5:
      case 6: text += "Junior"; break;
      case 7:
      case 8: text += "Senior"; break;
      default: text += "Bugged"; break;
    }

    text += " (" + major + " Major)";
  }

  else if (alive)  {
    if (plantsim && age >= PLANTSIM_ELDER) text += "Plantsim Elder";
    else if (plantsim && age >= PLANTSIM_ADULT) text += "Plantsim Adult";
    else if (plantsim) text += "Plantsim Toddler";
    else if (species == LARGE_DOG && age >= ELDER_LARGE_DOG)
      text += "Elder Large Dog";
    else if (species == LARGE_DOG && age >= ADULT_DOG) text += "Large Dog";
    else if (species == LARGE_DOG) text += "Puppy";
    else if (species == SMALL_DOG && age >= ELDER_SMALL_DOG)
      text += "Elder Small Dog";
    else if (species == SMALL_DOG && age >= ADULT_DOG) text += "Small Dog";
    else if (species == SMALL_DOG) text += "Puppy";
    else if (species == CAT && age >= ELDER_CAT) text += "Elder Cat";
    else if (species == CAT && age >= ADULT_CAT) text += "Cat";
    else if (species == CAT) text += "Kitten";
    else if (age >= ELDER) text += "Elder";
    else if (age >= ADULT) text += "Adult";
    else if (age >= TEEN) text += "Teen";
    else if (age >= CHILD) text += "Child";
    else if (age >= TODDLER) text += "Toddler";
    else if (species == HUMAN) text += "Baby";
  }

  if (!alive)  {
    switch (ghostColor)  {
      case PINK: text += "Pink"; break;
      case GREEN: text += "Green"; break;
      case YELLOW: text += "Yellow"; break;
      case BLUE: text += "Blue"; break;
      case ORANGE: text += "Orange"; break;
      case PURPLE: text += "Purple"; break;
      case RED: text += "Red"; break;
      case WHITE: text += "White"; break;
      case 8: text += "Transparent"; break;
      case BRIGHT_GREEN: text += "Bright Green"; break;
      default: break;
    }

    text += " Ghost";
  }

  if (alien) text += " Alien";
  if (zombie) text += " Zombie";
  if (vampire) text += " Vampire";
  if (werewolf) text += " Werewolf";

  if (witch)
    switch (alignment)  {
      case ATROCIOUSLYEVIL: text += " Atrociously Evil Witch"; break;
      case EVIL: text += " Evil Witch"; break;
      case MEAN: text += " Mean Witch"; break;
      case NEUTRAL: text += " Neutral Witch"; break;
      case NICE: text += " Nice Witch"; break;
      case GOOD: text += " Good Witch"; break;
      case INFALLIBLYGOOD: text += " Infallibly Good Witch"; break;
      default: text += " Witch"; break;
    }

  return text;
}


bool Sim::isParanormal (int para)  {
  switch (para)  {
    case ALIEN: return alien;
    case ZOMBIE: return zombie;
    case VAMPIRE: return vampire;
    case WEREWOLF: return werewolf;
    case SERVO: return servo;
    case PLANTSIM: return plantsim;
    case BIGFOOT: return bigfoot;
    case WITCH: return witch;
    case HUMAN:
      return !(alien || zombie || vampire || werewolf || servo || plantsim ||
               bigfoot || witch);
    default: return false;
  }
}

bool Sim::isSpecies (int spec)  {
  return species == spec;
}

int Sim::getAlignment ()  {
  return alignment;
}

QString Sim::getAspiration (int num)  {
  if (species != HUMAN) return "None";
  if (num == -1) num = aspiration.size () - 1;
  if (num < 0 || num >= aspiration.size ()) return "None";

  switch (aspiration[num])  {
    case PLEASURE: return "Pleasure";
    case ROMANCE: return "Romance";
    case KNOWLEDGE: return "Knowledge";
    case FAMILY: return "Family";
    case POPULARITY: return "Popularity";
    case FORTUNE: return "Fortune";
    case GRILLED_CHEESE: return "Grilled Cheese";
    case GROW_UP: return "Grow Up";
    default:
      if (age > TODDLER) return "Unknown";
      else return "None";
  }
}

QString Sim::getSecondaryAspiration (int num)  {
  if (species != HUMAN) return "None";
  if (num == -1) num = secondaryAspiration.size () - 1;
  if (num < 0 || num >= secondaryAspiration.size ()) return "None";

  switch (secondaryAspiration[num])  {
    case PLEASURE: return "Pleasure";
    case ROMANCE: return "Romance";
    case KNOWLEDGE: return "Knowledge";
    case FAMILY: return "Family";
    case POPULARITY: return "Popularity";
    case FORTUNE: return "Fortune";
    case GRILLED_CHEESE: return "Grilled Cheese";
    default:
      if (age > TEEN) return "Unknown";
      else return "None";
  }
}

int Sim::getAspirationCode (int num)  {
  if (num == -1) num = aspiration.size () - 1;
  if (num < 0 || num >= aspiration.size ()) return UNAVAILABLE;

  return aspiration[num];
}

int Sim::getSecondaryAspirationCode (int num)  {
  if (num == -1) num = secondaryAspiration.size () - 1;
  if (num < 0 || num >= secondaryAspiration.size ()) return NONE;

  return secondaryAspiration[num];
}

QString Sim::getLTW (int asp, int ltw)  {
  if (ltwDescriptions.size () > asp)
    if (ltwDescriptions[asp].size () > ltw)
      return ltwDescriptions[asp][ltw];

  return "";
}

int Sim::getLTWSteps (int asp, int ltw)  {
  if (ltwSteps.size () > asp)
    if (ltwSteps[asp].size () > ltw)
      return ltwSteps[asp][ltw];

  return 0;
}

int Sim::getLTWCompletedSteps (int asp, int ltw)  {
  if (ltwCompleted.size () > asp)
    if (ltwCompleted[asp].size () > ltw)
      return ltwCompleted[asp][ltw];

  return 0;
}

QString Sim::getMajor ()  {
  return major;
}

int Sim::getSemester ()  {
  return semester;
}

bool Sim::isAlive ()  {
  return alive;
}

int Sim::getGhostColor ()  {
  return ghostColor;
}

QString Sim::getNotes ()  {
  return notes;
}

QString Sim::getImageFile ()  {
  return imageFile;
}

QString Sim::getCSV ()  {
  QString csv = "\"" + name + "\"," + QString::number (nid) + "," +
                QString::number (age) + "," + QString::number (gender);

  for (int x = 0; x < aspiration.size (); x++)  {
    csv += ",(" + QString::number (aspiration[x]) + "," + 
           QString::number (secondaryAspiration[x]);

    for (int y = 0; y < ltwDescriptions[x].size (); y++)
      csv += ",(\"" + ltwDescriptions[x][y] + "\"," + 
             QString::number (ltwSteps[x][y]) + "," +
             QString::number (ltwCompleted[x][y]) + ")";

    csv += ")";
  }
                
  csv += ",\"" + household + "\"," + QString::number ((int)youngAdult) + ",\"" 
         + major + "\"," + QString::number (semester) + "," +
         QString::number ((int)alive) + "," + QString::number (ghostColor) + 
         "," + QString::number (hold);

  int flags = 0;
  if (alien) flags = flags | ALIEN;
  if (zombie) flags = flags | ZOMBIE;
  if (vampire) flags = flags | VAMPIRE;
  if (servo) flags = flags | SERVO;
  if (werewolf) flags = flags | WEREWOLF;
  if (plantsim) flags = flags | PLANTSIM;
  if (bigfoot) flags = flags | BIGFOOT;
  if (species == LARGE_DOG) flags = flags | LARGE_DOG;
  if (species == SMALL_DOG) flags = flags | SMALL_DOG;
  if (species == CAT) flags = flags | CAT;
  if (witch) flags = flags | WITCH;

  csv += "," + QString::number (flags) + "," + QString::number (alignment) +
         "," + imageFile + ",";

  QString fixedNotes = notes;
  fixedNotes.replace ("\r\n", ";;");
  fixedNotes.replace ("\n", ";;");

  csv += fixedNotes;

  return csv;
}

QString Sim::getFamilyInfo (QList< QPointer<Sim> > list)  {
  QString text = "";

  int spouseNum = -1;
  for (int x = 0; x < list.size (); x++)
    if (list[x] == spouse) spouseNum = x;

  text += QString::number (spouseNum);

  for (int y = 0; y < children.size (); y++)
    for (int z = 0; z < list.size (); z++)
      if (list[z] == children[y]) 
        text += "," + QString::number (z);

  return text;
}

void Sim::setParents (QPointer<Sim> p1, QPointer<Sim> p2)  {
  if ((parent1 == p1) && (parent2 == p2)) return;

  if (p1)  {
    parent1 = p1;
    parent1->addChild (this);
  }

  if (p2)  {
    parent2 = p2;
    parent2->addChild (this);
  }
}

void Sim::addChild (QPointer<Sim> ch)  {
  if (children.contains (ch)) return;

  if (ch)  {
    children.append (ch);

    if (!ch->isChildOf (this)) {
      if (!(ch->getParent1 ())) ch->setParents (this, spouse);
      else ch->setParents (NULL, this);
    }
  }
}

void Sim::clearParent1 ()  {
  parent1 = NULL;
}

void Sim::clearParent2 ()  {
  parent2 = NULL;
}

void Sim::clearChild (int i)  {
  if (children.size () > i) children.removeAt (i);
}

void Sim::setSpouse (QPointer<Sim> s)  {
  if (spouse == s) return;

  if (s)  {
    spouse = s;
    spouse->setSpouse (this);
  }
}

void Sim::clearSpouse ()  {
  spouse = NULL;
}

void Sim::setName (QString n)  {
  name = n;
}

void Sim::setNID (int n)  {
  nid = n;
}

void Sim::setGender (int g)  {
  gender = g;
}

void Sim::setAge (int a)  {
  age = a;
}

void Sim::setSpecies (QString s)  {
  if (s == "Human") species = HUMAN;
  else if (s == "LargeDog") species = LARGE_DOG;
  else if (s == "SmallDog") species = SMALL_DOG;
  else if (s == "Cat") species = CAT;
}

void Sim::setHold (int turns)  {
  hold = turns;
}

void Sim::setHousehold (QString h)  {
  household = h;
}

void Sim::newAspiration (int asp, int sasp)  {
  if (asp != NONE)  {
    aspiration.append (asp);
    secondaryAspiration.append (sasp);
    ltwDescriptions.append (QList<QString> ());
    ltwSteps.append (QList<int> ());
    ltwCompleted.append (QList<int> ());
  }
}

void Sim::updateAspiration (int asp, int sasp, int num)  {
  if (num == -1)
    num = aspiration.size () - 1;

  if (num < 0 || num >= aspiration.size ())
    return;

  if (asp != NONE) aspiration[num] = asp;
  if (sasp != NONE) secondaryAspiration[num] = sasp;
}

void Sim::updateAspiration (QString asp, QString sasp)  {
  if (asp == "Family") aspiration[aspiration.size () - 1] = FAMILY;
  else if (asp == "Fortune") aspiration[aspiration.size () - 1] = FORTUNE;
  else if (asp == "Romance") aspiration[aspiration.size () - 1] = ROMANCE;
  else if (asp == "Reputation") aspiration[aspiration.size () - 1] = POPULARITY;
  else if (asp == "Knowledge") aspiration[aspiration.size () - 1] = KNOWLEDGE;
  else if (asp == "Fun") aspiration[aspiration.size () - 1] = PLEASURE;
  else if (asp == "Chees") aspiration[aspiration.size () - 1] = GRILLED_CHEESE;
  else if (asp == "Growup") aspiration[aspiration.size () - 1] = GROW_UP;

  if (sasp == "Family") secondaryAspiration[aspiration.size () - 1] = FAMILY;
  else if (sasp == "Fortune") 
    secondaryAspiration[aspiration.size () - 1] = FORTUNE;
  else if (sasp == "Romance") 
    secondaryAspiration[aspiration.size () - 1] = ROMANCE;
  else if (sasp == "Reputation") 
    secondaryAspiration[aspiration.size () - 1] = POPULARITY;
  else if (sasp == "Knowledge") 
    secondaryAspiration[aspiration.size () - 1] = KNOWLEDGE;
  else if (sasp == "Fun") 
    secondaryAspiration[aspiration.size () - 1] = PLEASURE;
  else if (sasp == "Chees") 
    secondaryAspiration[aspiration.size () - 1] = GRILLED_CHEESE;
}

void Sim::addLTW (QString desc, int asp)  {
  if (asp == -1) asp = ltwDescriptions.size () - 1;

  if (ltwDescriptions.size () > asp && asp >= 0)
    ltwDescriptions[asp].append (desc);
}

void Sim::updateLTW (int asp, int ltw, QString desc, int done, int steps)  {
  if (ltwDescriptions.size () <= asp) ltwDescriptions.append (QStringList ());
  if (ltwDescriptions[asp].size () <= ltw) ltwDescriptions[asp].append (desc);
  else ltwDescriptions[asp][ltw] = desc;

  if (ltwSteps.size () <= asp) ltwSteps.append (QList<int> ());
  if (ltwSteps[asp].size () <= ltw) ltwSteps[asp].append (steps);
  else ltwSteps[asp][ltw] = steps;

  if (ltwCompleted.size () <= asp) ltwCompleted.append (QList<int> ());
  if (ltwCompleted[asp].size () <= ltw) ltwCompleted[asp].append (done);
  else ltwCompleted[asp][ltw] = done;
}

void Sim::deleteAspiration (int num)  {
  if (num < 0 || num >= aspiration.size ())
    return;

  aspiration.removeAt (num);
  secondaryAspiration.removeAt (num);
}

void Sim::setNotes (QString n)  {
  notes = n;
}

void Sim::setImageFile (QString i)  {
  imageFile = i;
}

void Sim::setCollege (bool c)  {
  youngAdult = c;
  age = ADULT;
  if (!c) setHold (1);

  if (c)  {
    major = "Undeclared";
    semester = 1;
  }
}

void Sim::setMajor (QString m)  {
  if (m != "")
    major = m;
}

void Sim::setSemester (int s)  {
  semester = s;
}

void Sim::increaseAge (int a)  {
  if (youngAdult && a > 0)  {
    semester += a;
    if (semester > 8)
      youngAdult = false;
  }

  else if (a < 0)  {
    if ((!plantsim) && (age > ELDER) && ((age + a) < ELDER))
      age = ELDER;
    else if (plantsim && (age > PLANTSIM_ELDER) && ((age + a) < PLANTSIM_ELDER))
      age = PLANTSIM_ELDER;
    else if ((!plantsim) && (age > ADULT) && ((age + a) < ADULT))
      age = ADULT;
    else if ((!plantsim) && (age > TEEN) && ((age + a) < TEEN))
      age = TEEN;
    else if ((!plantsim) && (age > CHILD) && ((age + a) < CHILD))
      age = CHILD;
    else if (plantsim && (age > PLANTSIM_ADULT) && ((age + a) < PLANTSIM_ADULT))
      age = PLANTSIM_ADULT;
    else if ((species == LARGE_DOG) && (age > ELDER_LARGE_DOG) && 
             ((age + a) < ELDER_LARGE_DOG))
      age = ELDER_LARGE_DOG;
    else if ((species == SMALL_DOG) && (age > ELDER_SMALL_DOG) &&
             ((age + a) < ELDER_SMALL_DOG))
      age = ELDER_SMALL_DOG;
    else if (((species == LARGE_DOG) || (species == SMALL_DOG)) && 
             (age > ADULT_DOG) && ((age + a) < ADULT_DOG))
      age = ADULT_DOG;
    else if ((species == CAT) && (age > ELDER_CAT) && ((age + a) < ELDER_CAT))
      age = ELDER_CAT;
    else if ((species == CAT) && (age > ADULT_CAT) && ((age + a) < ADULT_CAT))
      age = ADULT_CAT;
    else if ((age + a) < 0) age = 0;

    else age += a;
  }

  else age += a;

  if (age < TODDLER) aspiration[0] = NONE;
  else if (age < TEEN) aspiration[0] = GROW_UP;
}

void Sim::addParanormal (int flag)  {
  switch (flag)  {
    case VAMPIRE: vampire = true; break;
    case WEREWOLF: werewolf = true; break;
    case PLANTSIM: 
      plantsim = true;
      age -= 25;
      break;
    case WITCH:
      witch = true;
      alignment = NEUTRAL;
    default: break;
  }
}

void Sim::removeParanormal (int flag)  {
  switch (flag)  {
    case VAMPIRE: vampire = false; break;
    case WEREWOLF: werewolf = false; break;
    case PLANTSIM: 
      plantsim = false; 
      age += 25;
      break;
    case WITCH: witch = false;
    default: break;
  }
}

void Sim::setParaFlags (int flags)  {
  alien = flags & ALIEN;
  zombie = flags & ZOMBIE;
  vampire = flags & VAMPIRE;
  servo = flags & SERVO;
  werewolf = flags & WEREWOLF;
  plantsim = flags & PLANTSIM;
  bigfoot = flags & BIGFOOT;
  witch = flags & WITCH;
}

void Sim::setAlignment (int align)  {
  alignment = align;
}

void Sim::makeDead (int color)  {
  alive = false;
  ghostColor = color;
}

void Sim::resurrect (bool success)  {
  alive = true;
  zombie = success ? zombie : true;
}
