#ifndef SIM_H
#define SIM_H

#include <QObject>
#include <QPointer>
#include <QList>

class QString;
class QStringList;

class Sim : public QObject {
  friend class EditSimDialog;

  public:
    Sim (QObject* = 0);
    Sim (QString, int, int, int, int, bool, int, QString, QString);

    void setInitial (QString, int, int, int, bool, int, QString);
    void readCSV (QString, int);
    void readCSV0_30_4 (QString);
    void readCSV0_2 (QString);
    void readCSV0_1 (QString);
    void readFamilyInfo (QString, QList< QPointer<Sim> >);
    void update (QPointer<Sim>, QList< QPointer<Sim> >);

    QPointer<Sim> getParent1 ();
    QPointer<Sim> getParent2 ();
    QPointer<Sim> getChild (int);
    bool isChildOf (QPointer<Sim>);
    QPointer<Sim> getSpouse ();
    bool onHold ();
    bool canAge ();
    QString getFullName ();
    int getNID ();
    int getGender ();
    int getAge ();
    QString getHousehold ();
    bool specialHousehold ();
    bool isTownie ();
    bool isParanormal (int);
    bool isSpecies (int);
    int getAlignment ();
    QString ageToQString ();
    QString getAspiration (int = -1);
    QString getSecondaryAspiration (int = -1);
    int getAspirationCode (int = -1);
    int getSecondaryAspirationCode (int = -1);
    QString getLTW (int, int);
    int getLTWSteps (int, int);
    int getLTWCompletedSteps (int, int);
    QString getMajor ();
    int getSemester ();
    bool isAlive ();
    int getGhostColor ();
    QString getNotes ();
    QString getImageFile ();
    QString getCSV ();
    QString getFamilyInfo (QList< QPointer<Sim> >);

    void setParents (QPointer<Sim>, QPointer<Sim>);
    void addChild (QPointer<Sim>);
    void clearParent1 ();
    void clearParent2 ();
    void clearChild (int);
    void setSpouse (QPointer<Sim>);
    void clearSpouse ();
    void setName (QString);
    void setNID (int);
    void setGender (int);
    void setAge (int);
    void setSpecies (QString);
    void setHold (int);
    void setHousehold (QString);
    void newAspiration (int, int);
    void updateAspiration (int, int, int = -1);
    void updateAspiration (QString, QString);
    void addLTW (QString, int = -1);
    void updateLTW (int, int, QString, int, int);
    void deleteAspiration (int);
    void setNotes (QString);
    void setImageFile (QString);
    void setCollege (bool);
    void setMajor (QString);
    void setSemester (int);
    void increaseAge (int);
    void addParanormal (int);
    void removeParanormal (int);
    void setParaFlags (int);
    void setAlignment (int);
    void makeDead (int);
    void resurrect (bool);

  private:
    int hold;

    QPointer<Sim> parent1;
    QPointer<Sim> parent2;
    QList< QPointer<Sim> > children;
    QPointer<Sim> spouse;

    QString name;
    int nid;
    int species;
    int age;
    QString household;
    int semester;
    int gender;
    QList<int> aspiration;
    QList<int> secondaryAspiration;
    QList<QStringList> ltwDescriptions;
    QList< QList<int> > ltwSteps;
    QList< QList<int> > ltwCompleted;
    bool youngAdult;
    QString major;

    bool alien;
    bool zombie;
    bool vampire;
    bool servo;
    bool werewolf;
    bool plantsim;
    bool bigfoot;
    bool witch;
    int alignment;

    bool alive;
    int ghostColor;
    QString notes;
    QString imageFile;
};

#endif
