#include <QAbstractItemModel>
#include <QStandardItemModel>
#include <QModelIndex>
#include <QMainWindow>
#include <QTextStream>
#include <QFile>
#include <QMessageBox>
#include <QRegExp>

#include <iostream>
#include <cstdlib>
#include <ctime>
using namespace std;

#include "simsheet.h"
#include "agepage.h"
#include "householdpage.h"
#include "aspirationpage.h"
#include "familypage.h"
//#include "careerpage.h"
#include "achievementspage.h"
#include "updatepage.h"
#include "updateentry.h"
#include "sim.h"
#include "const.h"
#include "listmanagementdialog.h"
#include "addsimdialog.h"
#include "editsimdialog.h"
#include "filterdialog.h"
#include "settingsdialog.h"
#include "addnotesdialog.h"

SimSheet::SimSheet (QMainWindow *window)  { 
  setMinimumWidth (700);
  setMinimumHeight (400);

  listManagementDialog = NULL;
  addSimDialog = NULL;
  editSimDialog = NULL;
  settingsDialog = NULL;
  addNotesDialog = NULL;
  filterDialog = NULL;

  rotationTime = 1;
  rotateCollege = true;
  collegeRotationTime = 2;
  autoGoToCollege = false;
  autoGoToCollegeTime = 5;
  autoBirthdays = true;
  autoBirthdaysTime = 1;
  ageDisplay = 1;
  epFlags = NONE;
  simPELocation = "C:/Program Files/SimPE/";

  readSettings ();

  aliveFilterFlags = ALIVE_FILTER | DEAD_FILTER;
  typeFilterFlags = PLAYABLE_FILTER | TOWNIE_FILTER | NPC_FILTER |
                    RUNAWAY_FILTER | ADOPTION_FILTER | DOWNTOWNIE_FILTER |
                    GARDEN_CLUB_FILTER | ISLAND_FILTER | MOUNTAIN_FILTER |
                    ASIAN_FILTER | TOURIST_FILTER | HOBBY_FILTER | 
                    SOCIALITE_FILTER | BOHEMIAN_FILTER | JOCK_FILTER |
                    TECHIE_FILTER | GEARHEAD_FILTER;
  speciesFilterFlags = HUMAN_FILTER | LARGE_DOG_FILTER | SMALL_DOG_FILTER |
                       CAT_FILTER;
  genderFilterFlags = MALE_FILTER | FEMALE_FILTER;
  ageFilterFlags = BABY_FILTER | TODDLER_FILTER | CHILD_FILTER | TEEN_FILTER |
                   YA_FILTER | ADULT_FILTER | ELDER_FILTER;
  aspirationFilterFlags = NONE_FILTER | GROW_UP_FILTER | FORTUNE_FILTER |
                          FAMILY_FILTER | POPULARITY_FILTER | ROMANCE_FILTER |
                          KNOWLEDGE_FILTER | PLEASURE_FILTER |
                          GRILLED_CHEESE_FILTER;
  paranormalFilterFlags = NORMAL_FILTER | ALIEN_FILTER | ZOMBIE_FILTER |
                          VAMPIRE_FILTER | WEREWOLF_FILTER | SERVO_FILTER |
                          PLANTSIM_FILTER | BIGFOOT_FILTER | WITCH_FILTER;

  agePage = new AgePage (window);
  aspirationPage = new AspirationPage (epFlags);
  familyPage = new FamilyPage (simList);
  householdPage = new HouseholdPage ();
  careerPage = new QWidget ();
  achievementPage = new AchievementsPage ();
  updatePage = new UpdatePage (simList);

  addTab (agePage, "&List of Sims");
  addTab (householdPage, "&Households");
  addTab (familyPage, "&Family Ties");
  addTab (aspirationPage, "&Aspirations");
  addTab (careerPage, "&Careers");
  addTab (achievementPage, "Achie&vements");
  addTab (updatePage, "&Update");

  // Signals from agePage
  connect (agePage, SIGNAL (newListManagementDialog ()), this,
           SLOT (createListManagementDialog ()));
  connect (agePage, SIGNAL (newAddSimDialog ()), this,
           SLOT (createAddSimDialog ()));
  connect (agePage, SIGNAL (newEditSimDialog (int)), this,
           SLOT (createEditSimDialog (int)));
  connect (agePage, SIGNAL (newFilterDialog ()), this,
           SLOT (createFilterDialog ()));
  connect (agePage, SIGNAL (newSettingsDialog ()), this, 
           SLOT (createSettingsDialog ()));
  connect (agePage, SIGNAL (newAddNotesDialog (int)), this,
           SLOT (createAddNotesDialog (int)));
  connect (agePage, SIGNAL (assignImageFile (int, QString)), this,
           SLOT (assignImageFile (int, QString)));
  connect (agePage, SIGNAL (update (int)), this, SLOT (doUpdate (int)));

  // Signals from householdPage
  connect (householdPage, SIGNAL (showHousehold (QString)), this,
           SLOT (showHousehold (QString)));
  connect (householdPage, SIGNAL (replaceHouseholdName (QString, QString)),
           this, SLOT (replaceHouseholdName (QString, QString)));

  // Signals from updatePage
  connect (updatePage, SIGNAL (rotate ()), this, SLOT (doRotation ()));
  connect (updatePage, SIGNAL (holdAllBut (Sim*, int)), this,
           SLOT (holdAllBut (Sim*, int)));
  connect (updatePage, SIGNAL (holdHousehold (QString, int)), this,
           SLOT (holdHousehold (QString, int)));
  connect (updatePage, SIGNAL (holdAllHouseholdsBut (QString, int)), this,
           SLOT (holdAllHouseholdsBut (QString, int)));
  connect (updatePage, 
      SIGNAL (newSim (QString, int, int, int, int, QString, int, Sim*, Sim*)), 
           this, 
      SLOT (newSim (QString, int, int, int, int, QString, int, Sim*, Sim*)));
  connect (updatePage, 
      SIGNAL (newTowniePair (QString, int, QString, int, int, int, Sim*)),
           this, 
      SLOT (newTowniePair (QString, int, QString, int, int, int, Sim*)));
  connect (updatePage, SIGNAL (removeSim (Sim*)), this, 
           SLOT (removeSim (Sim*)));
  connect (updatePage, SIGNAL (addToAchievementsPage (QString, Sim*, Sim*)),
           achievementPage, SLOT (addEntry (QString, Sim*, Sim*)));
}

void SimSheet::setSimModel (QStandardItemModel *model)  {
  agePage->setSourceModel (model);
  updatePage->setSimList (simList);
}

void SimSheet::setHouseholdModel (QStandardItemModel *model)  {
  householdPage->setSourceModel (model);
}

void SimSheet::setFamilyModel (QStandardItemModel *model)  {
  familyPage->setSourceModel (model);
}

void SimSheet::saveSettings ()  {
  QFile conf ("simsheet.conf");
  QTextStream out (&conf);

  if (!conf.open (QFile::WriteOnly | QFile::Text))  {
    QMessageBox::warning (this, "", "Could not save settings.");
    return;
  }

  out << "# RotationTime: the number of sim days in the rotation.\r\n";
  out << "RotationTime=" << rotationTime << "\r\n\r\n";
  out << "# RotateCollege: 1 means time in college will increase "
      << "automatically, 0 means\r\n# it will not.\r\n";
  out << "RotateCollege=" << rotateCollege << "\r\n\r\n";
  out << "# CollegeRotationTime: the number of semesters that will pass "
      << "during the\r\n# rotation.  Not relevant if RotateCollege is off.\r\n";
  out << "CollegeRotationTime=" << collegeRotationTime << "\r\n\r\n";
  out << "# AutoGoToCollege: if 1, teens will automatically be sent to college"
      << " at a\r\n# certain age.\r\n";
  out << "AutoGoToCollege=" << autoGoToCollege << "\r\n\r\n";
  out << "# AutoGoToCollegeTime: the number of days until adulthood at which "
      << "teens will\r\n# be sent to college.  Not relevant if AutoGoToCollege"
      << " is off.\r\n";
  out << "AutoGoToCollegeTime=" << autoGoToCollegeTime << "\r\n\r\n";
  out << "# AutoBirthdays: if 1, birthday events will automatically be added "
      << "to the\r\n# update page.\r\n";
  out << "AutoBirthdays=" << autoBirthdays << "\r\n\r\n";
  out << "# AutoBirthdaysTime: if 1, automatically added birthdays will be set"
      << " to happen\r\n# after 6 PM; if 0 they will happen before.  Not "
      << "relevant if AutoBirthdays is\r\n# off.\r\n";
  out << "AutoBirthdaysTime=" << autoBirthdaysTime << "\r\n\r\n";
  out << "# AgeDisplay: if 1, ages will be displayed as \"## days until "
      << "<lifestage>\"; if\r\n# 0, they will be displayed as \"## days old\"."
      << "\r\n";
  out << "AgeDisplay=" << ageDisplay << "\r\n\r\n";
  out << "# EPFlags: to calculate the right value, sum the following:\r\n# +1 "
      << "(if University is installed)\r\n# +2 (if Nightlife is installed)"
      << "\r\n# +4 (if OFB is installed)\r\n# +8 (if Pets is installed)\r\n# "
      << "+16 (if Seasons is installed)\r\n# +32 (if Bon Voyage is installed)"
      << "\r\n# +64 (if Freetime is installed)\r\n# +128 (if Apartment Life is"
      << "installed)\r\n";
  out << "EPFlags=" << epFlags << "\r\n\r\n";
  out << "# SimPELocation: the default path to SimPE.  Use forward slashes (/)"
      << " and not\r\n# backslashes (\\).\r\n";
  out << "SimPELocation=" << simPELocation;

  conf.close ();
  return;
}

void SimSheet::doUpdate (int simIndex)  {
  // This function will call functions in all sim-specific tabs (Aspiration
  // tab, Family tab, Career tab, etc.) whenever a different sim is selected
  // on agePage.

  simIndex = filterMap[simIndex];

  agePage->setNotesDisplay (simList[simIndex]->getNotes ());
  agePage->setImageFile (simList[simIndex]->getImageFile ());

  familyPage->setSim (simList[simIndex]);

  aspirationPage->setSim (simList[simIndex]);
}

void SimSheet::doRotation ()  {
  QTextStream terminal (stdout);

  for (int x = 0; x < simList.size (); x++)  {
    // Increase ages of sims by the appropriate amount.
    if (!simList[x]->onHold () && simList[x]->canAge ())  {
      if (simList[x]->ageToQString ().contains ("Major"))  {
        simList[x]->increaseAge (1);

        for (int y = 1; y < collegeRotationTime; y++)
          if (!simList[x]->onHold ())
            simList[x]->increaseAge (1);
      }

      else  {
        simList[x]->increaseAge (1);

        for (int y = 1; y < rotationTime; y++)
          if (!simList[x]->onHold ())
            simList[x]->increaseAge (1);
      }
    }

    // Check for upcoming birthdays and add appropriate updateEntries.
    // Plantsims and pets have special aging, and thus special for loops.
    if (autoBirthdays && simList[x]->canAge () && !simList[x]->onHold ())  {
      if ((simList[x]->isParanormal (PLANTSIM)) == true)
        if ((simList[x]->getAge () >= (PLANTSIM_ADULT - rotationTime) &&
             simList[x]->getAge () < PLANTSIM_ADULT) ||
            (simList[x]->getAge () >= (PLANTSIM_ELDER - rotationTime) &&
             simList[x]->getAge () < PLANTSIM_ELDER))  {
          QString str = simList[x]->getFullName () + " grew up well ";
 
          str += autoBirthdaysTime ? "after" : "before";
          str += " six PM (eleven AM for pets). (Aspiration: Same)";

          UpdateEntry *temp = new UpdateEntry (str, simList[x], NULL);
          updatePage->addEvent (temp);
        }

      if ((simList[x]->isParanormal (PLANTSIM)) == false &&
          simList[x]->isSpecies (HUMAN))
        if ((simList[x]->getAge () >= (TODDLER - rotationTime) &&
             simList[x]->getAge () < TODDLER) || 
            (simList[x]->getAge () >= (CHILD - rotationTime) &&
             simList[x]->getAge () < CHILD) ||
            (simList[x]->getAge () >= (TEEN - rotationTime) &&
             simList[x]->getAge () < TEEN) ||
            (simList[x]->getAge () >= (ADULT - rotationTime) &&
             simList[x]->getAge () < ADULT) ||
            (simList[x]->getAge () >= (ELDER - rotationTime) &&
             simList[x]->getAge () < ELDER))  {
          QString str = simList[x]->getFullName () + " grew up well ";
          str += autoBirthdaysTime ? "after" : "before";
          str += " six PM (eleven AM for pets). (Aspiration: Same)";

          UpdateEntry *temp = new UpdateEntry (str, simList[x], NULL);
          updatePage->addEvent (temp);
        }

      if (simList[x]->isSpecies (LARGE_DOG))
        if ((simList[x]->getAge () >= (ADULT_DOG - rotationTime) &&
             simList[x]->getAge () < ADULT_DOG) ||
            (simList[x]->getAge () >= (ELDER_LARGE_DOG - rotationTime) &&
             simList[x]->getAge () < ELDER_LARGE_DOG))  {
          QString str = simList[x]->getFullName () + " grew up well ";
          str += autoBirthdaysTime ? "after" : "before";
          str += " six PM (eleven AM for pets). (Aspiration: Same)";

          UpdateEntry *temp = new UpdateEntry (str, simList[x], NULL);
          updatePage->addEvent (temp);
        }

      if (simList[x]->isSpecies (SMALL_DOG))
        if ((simList[x]->getAge () >= (ADULT_DOG - rotationTime) &&
             simList[x]->getAge () < ADULT_DOG) ||
            (simList[x]->getAge () >= (ELDER_SMALL_DOG - rotationTime)  &&
             simList[x]->getAge () < ELDER_SMALL_DOG))  {
          QString str = simList[x]->getFullName () + " grew up well ";
          str += autoBirthdaysTime ? "after" : "before";
          str += " six PM (eleven AM for pets). (Aspiration: Same)";

          UpdateEntry *temp = new UpdateEntry (str, simList[x], NULL);
          updatePage->addEvent (temp);
        }

      if (simList[x]->isSpecies (CAT))
        if ((simList[x]->getAge () >= (ADULT_CAT - rotationTime) &&
             simList[x]->getAge () < ADULT_CAT) ||
            (simList[x]->getAge () >= (ELDER_CAT - rotationTime)  &&
             simList[x]->getAge () < ELDER_CAT))  {
          QString str = simList[x]->getFullName () + " grew up well ";
          str += autoBirthdaysTime ? "after" : "before";
          str += " six PM (eleven AM for pets). (Aspiration: Same)";

          UpdateEntry *temp = new UpdateEntry (str, simList[x], NULL);
          updatePage->addEvent (temp);
        }
    }

    // Check for teens needing to go to college, and YAs ready to graduate.
    if (autoGoToCollege)  {
      if (simList[x]->isSpecies (HUMAN) &&
          !simList[x]->isParanormal (PLANTSIM) && 
          !simList[x]->isParanormal (SERVO) && 
          simList[x]->getAge () < ADULT &&
          (simList[x]->getAge () + rotationTime) > 
          (ADULT - autoGoToCollegeTime))  {
        QString str = simList[x]->getFullName () + " grew up well and went to";
        str += " college. (Major: Undeclared)";

        UpdateEntry *temp = new UpdateEntry (str, simList[x], NULL);
        updatePage->addEvent (temp);
      }

      if (simList[x]->ageToQString ().contains ("Major")  &&
          simList[x]->getSemester () >= (9 - collegeRotationTime))  {
        QString str = simList[x]->getFullName () + " grew up well and ";
        str += "graduated Summa cum Laude from college. (Major: " +
               simList[x]->getMajor () + ")";

        UpdateEntry *temp = new UpdateEntry (str, simList[x], NULL);
        updatePage->addEvent (temp);
      }
    }
  }

  updateModels ();
  householdPage->setNotPlayed ();
}

void SimSheet::holdAllBut (Sim *s, int r)  {
  for (int x = 0; x < simList.size (); x++)
    if (simList[x] != s)
      simList[x]->setHold (r);
}

void SimSheet::holdHousehold (QString house, int r)  {
  for (int x = 0; x < simList.size (); x++)
    if (simList[x]->getHousehold () == house)
      simList[x]->setHold (r);
}

void SimSheet::holdAllHouseholdsBut (QString house, int r)  {
  for (int x = 0; x < simList.size (); x++)
    if (simList[x]->getHousehold () != house)
      simList[x]->setHold (r);
}

void SimSheet::newSim (QString n, int g, int a, int asp, int holdDays,
                       QString house, int pn, Sim *parent1, Sim *parent2)  {
  QPointer<Sim> s = new Sim (n, a, g, asp, NONE, true, pn, "", "");
  simList.append (s);

  s->setHold (holdDays);
  if (house != "") s->setHousehold (house);
  s->setParents (parent1, parent2);

  QTextStream terminal (stdout);
  terminal << "New Sim made: " << n << " " << a << " " << g << " " << asp
           << endl;

  updateModels ();
}

void SimSheet::newSim (QString n, int g, int a, int asp, int sasp, bool al, 
                       int gc, QString house, int pn, int align, int sm, 
                       QString mj)  {
  QPointer<Sim> s = new Sim (n, a, g, asp, sasp, true, pn, "", "");
  s->setHousehold (house);
  s->setAlignment (align);

  if (sm)  {
    s->setCollege (true);
    for (int x = 0; x < sm; x++)
      s->increaseAge (1);
  }

  s->setMajor (mj);

  if (!al) s->makeDead (gc);

  simList.append (s);
  updateModels ();
}

void SimSheet::newSim (Sim *s)  {
  for (int x = 0; x < simsWithNIDs.size (); x++)  {
    if (simsWithNIDs[x]->getNID () == s->getNID ())  {
      simsWithNIDs[x]->update (s, simList);
      return;
    }

    else if (simsWithNIDs[x]->getNID () > s->getNID ()) break;
  }

  simList.append (s);
}

void SimSheet::newTowniePair (QString tn, int ta, QString bn, int bg, 
                              int holdDays, int sp, Sim *mother)  {
  // This function creates a male townie named tn, age ta, and his child,
  // name bn, gender bg, who was born beforeSix, to mother. Both are species
  // sp. This can't be done with newSim, because they're both called with
  // signals, and there's no way for the originator of this function
  // (UpdateEntry::go ()) to get the townie's information after he's created.
  // This won't actually be a problem until Family Ties, but I don't want to
  // rewrite the program when I get there.

  QPointer<Sim> townie = new Sim (tn, ta, MALE, NONE, NONE, true, sp, "", "");
  QPointer<Sim> baby = new Sim (bn, BABY, bg, NONE, NONE, true, sp, "", "");
  simList.append (townie);
  simList.append (baby);

  baby->setHold (holdDays);

  townie->setHousehold ("At Large");
  baby->setHousehold (mother->getHousehold ());
  baby->setParents (townie, mother);

  updateModels ();
}

void SimSheet::removeSim (Sim *s)  {
/*  // Another probably not needed bit of code.  Used to remove sims that got
  // taken by the Social Worker, but now they just go into the Adoption Queue
  // household.  There might still be reasons to delete sims, though.

  QAbstractItemModel *source = agePage->sourceModel ();

  for (int x = 0; x < simList.size (); x++)
    if (simList[x] == s)  {
      simList.removeAt (x);
      source->removeRow (x);

      delete s;
    }

  updateModels ();*/
}

void SimSheet::showHousehold (QString house)  {
  // Sends information about the contents of a household to householdPage, in
  // response to the household-has-been-clicked signal.

  QStringList names;

  for (int x = 0; x < simList.size (); x++)
    if (simList[x]->getHousehold () == house)
      names.append (simList[x]->getFullName ());

  householdPage->displayNames (names);
}

void SimSheet::replaceHouseholdName (QString before, QString after)  {
  for (int x = 0; x < simList.size (); x++)
    if (simList[x]->getHousehold () == before)
      simList[x]->setHousehold (after);

  updateModels ();
}

void SimSheet::createListManagementDialog ()  {
  if (!listManagementDialog)  {
    listManagementDialog = new ListManagementDialog;
    connect (listManagementDialog, SIGNAL (openList (QString, int)), this,
             SLOT (open (QString, int)));
    connect (listManagementDialog, SIGNAL (saveList (QString)), this,
             SLOT (save (QString)));
    connect (listManagementDialog, SIGNAL (setDebugList (int)), this,
             SLOT (setDebug (int)));
    connect (listManagementDialog, SIGNAL (clearList ()), this, 
             SLOT (clearSimList ()));
    connect (listManagementDialog, SIGNAL (addSim (Sim*)), this,
             SLOT (newSim (Sim*)));
    connect (listManagementDialog, SIGNAL (updateModels ()), this,
             SLOT (updateModels ()));
  }

  listManagementDialog->setSimPELocation (simPELocation);
  listManagementDialog->setEPFlags (epFlags);
  listManagementDialog->show ();
}

void SimSheet::createAddSimDialog ()  {
  if (addSimDialog) delete addSimDialog;

  QStringList houses;
  for (int x = 0; x < simList.size (); x++)
    if (!houses.contains (simList[x]->getHousehold ()))
      houses.append (simList[x]->getHousehold ());

  addSimDialog = new AddSimDialog (epFlags, houses);
  connect (addSimDialog, SIGNAL (newSim (QString, int, int, int, int, bool, 
                                         int, QString, int, int, int, QString)),
           this, SLOT (newSim (QString, int, int, int, int, bool, int, 
                               QString, int, int, int, QString)));

  addSimDialog->show ();
}

void SimSheet::createEditSimDialog (int s)  {
  if (editSimDialog)
    delete editSimDialog;

  QStringList houses;
  for (int x = 0; x < simList.size (); x++)
    if (!houses.contains (simList[x]->getHousehold ()))
      houses.append (simList[x]->getHousehold ());

  editSimDialog = new EditSimDialog (epFlags, houses, simList[filterMap[s]]);
  connect (editSimDialog, SIGNAL (simEdited ()), this, SLOT (updateModels ()));

  editSimDialog->show ();
}

void SimSheet::createFilterDialog ()  {
  if (!filterDialog)  {
    filterDialog = new FilterDialog ();

    connect (filterDialog, SIGNAL (setAliveFilter (bool)), this,
             SLOT (setAliveFilter (bool)));
    connect (filterDialog, SIGNAL (setDeadFilter (bool)), this,
             SLOT (setDeadFilter (bool)));

    connect (filterDialog, SIGNAL (setPlayableFilter (bool)), this,
             SLOT (setPlayableFilter (bool)));
    connect (filterDialog, SIGNAL (setTownieFilter (bool)), this,
             SLOT (setTownieFilter (bool)));
    connect (filterDialog, SIGNAL (setNPCFilter (bool)), this, 
             SLOT (setNPCFilter (bool)));
    connect (filterDialog, SIGNAL (setRunawayFilter (bool)), this,
             SLOT (setRunawayFilter (bool)));
    connect (filterDialog, SIGNAL (setAdoptionFilter (bool)), this,
             SLOT (setAdoptionFilter (bool)));
    connect (filterDialog, SIGNAL (setDowntownieFilter (bool)), this,
             SLOT (setDowntownieFilter (bool)));
    connect (filterDialog, SIGNAL (setGardenClubFilter (bool)), this,
             SLOT (setGardenClubFilter (bool)));
    connect (filterDialog, SIGNAL (setIslandFilter (bool)), this,
             SLOT (setIslandFilter (bool)));
    connect (filterDialog, SIGNAL (setMountainFilter (bool)), this,
             SLOT (setMountainFilter (bool)));
    connect (filterDialog, SIGNAL (setAsianFilter (bool)), this,
             SLOT (setAsianFilter (bool)));
    connect (filterDialog, SIGNAL (setTouristFilter (bool)), this,
             SLOT (setTouristFilter (bool)));
    connect (filterDialog, SIGNAL (setHobbyFilter (bool)), this,
             SLOT (setHobbyFilter (bool)));
    connect (filterDialog, SIGNAL (setSocialiteFilter (bool)), this,
             SLOT (setSocialiteFilter (bool)));
    connect (filterDialog, SIGNAL (setBohemianFilter (bool)), this,
             SLOT (setBohemianFilter (bool)));
    connect (filterDialog, SIGNAL (setJockFilter (bool)), this,
             SLOT (setJockFilter (bool)));
    connect (filterDialog, SIGNAL (setTechieFilter (bool)), this,
             SLOT (setTechieFilter (bool)));
    connect (filterDialog, SIGNAL (setGearheadFilter (bool)), this,
             SLOT (setGearheadFilter (bool)));

    connect (filterDialog, SIGNAL (setHumanFilter (bool)), this,
             SLOT (setHumanFilter (bool)));
    connect (filterDialog, SIGNAL (setLargeDogFilter (bool)), this,
             SLOT (setLargeDogFilter (bool)));
    connect (filterDialog, SIGNAL (setSmallDogFilter (bool)), this,
             SLOT (setSmallDogFilter (bool)));
    connect (filterDialog, SIGNAL (setCatFilter (bool)), this,
             SLOT (setCatFilter (bool)));

    connect (filterDialog, SIGNAL (setMaleFilter (bool)), this,
             SLOT (setMaleFilter (bool)));
    connect (filterDialog, SIGNAL (setFemaleFilter (bool)), this,
             SLOT (setFemaleFilter (bool)));

    connect (filterDialog, SIGNAL (setBabyFilter (bool)), this,
             SLOT (setBabyFilter (bool)));
    connect (filterDialog, SIGNAL (setToddlerFilter (bool)), this,
             SLOT (setToddlerFilter (bool)));
    connect (filterDialog, SIGNAL (setChildFilter (bool)), this,
             SLOT (setChildFilter (bool)));
    connect (filterDialog, SIGNAL (setTeenFilter (bool)), this,
             SLOT (setTeenFilter (bool)));
    connect (filterDialog, SIGNAL (setYAFilter (bool)), this,
             SLOT (setYAFilter (bool)));
    connect (filterDialog, SIGNAL (setAdultFilter (bool)), this,
             SLOT (setAdultFilter (bool)));
    connect (filterDialog, SIGNAL (setElderFilter (bool)), this,
             SLOT (setElderFilter (bool)));

    connect (filterDialog, SIGNAL (setNoneFilter (bool)), this,
             SLOT (setNoneFilter (bool)));
    connect (filterDialog, SIGNAL (setGrowUpFilter (bool)), this,
             SLOT (setGrowUpFilter (bool)));
    connect (filterDialog, SIGNAL (setFortuneFilter (bool)), this,
             SLOT (setFortuneFilter (bool)));
    connect (filterDialog, SIGNAL (setFamilyFilter (bool)), this,
             SLOT (setFamilyFilter (bool)));
    connect (filterDialog, SIGNAL (setPopularityFilter (bool)), this,
             SLOT (setPopularityFilter (bool)));
    connect (filterDialog, SIGNAL (setRomanceFilter (bool)), this,
             SLOT (setRomanceFilter (bool)));
    connect (filterDialog, SIGNAL (setKnowledgeFilter (bool)), this,
             SLOT (setKnowledgeFilter (bool)));
    connect (filterDialog, SIGNAL (setPleasureFilter (bool)), this,
             SLOT (setPleasureFilter (bool)));
    connect (filterDialog, SIGNAL (setGrilledCheeseFilter (bool)), this,
             SLOT (setGrilledCheeseFilter (bool)));

    connect (filterDialog, SIGNAL (setNormalFilter (bool)), this,
             SLOT (setNormalFilter (bool)));
    connect (filterDialog, SIGNAL (setAlienFilter (bool)), this,
             SLOT (setAlienFilter (bool)));
    connect (filterDialog, SIGNAL (setZombieFilter (bool)), this,
             SLOT (setZombieFilter (bool)));
    connect (filterDialog, SIGNAL (setVampireFilter (bool)), this,
             SLOT (setVampireFilter (bool)));
    connect (filterDialog, SIGNAL (setWerewolfFilter (bool)), this,
             SLOT (setWerewolfFilter (bool)));
    connect (filterDialog, SIGNAL (setServoFilter (bool)), this,
             SLOT (setServoFilter (bool)));
    connect (filterDialog, SIGNAL (setPlantsimFilter (bool)), this,
             SLOT (setPlantsimFilter (bool)));
    connect (filterDialog, SIGNAL (setBigfootFilter (bool)), this,
             SLOT (setBigfootFilter (bool)));
    connect (filterDialog, SIGNAL (setWitchFilter (bool)), this,
             SLOT (setWitchFilter (bool)));
  }

  filterDialog->show ();
}

void SimSheet::createSettingsDialog ()  {
  if (!settingsDialog)  {
    settingsDialog = new SettingsDialog (rotationTime, rotateCollege,
      collegeRotationTime, autoGoToCollege, autoGoToCollegeTime, autoBirthdays,
      autoBirthdaysTime, ageDisplay, epFlags, simPELocation);
    connect (settingsDialog, SIGNAL (setRotation (int)), this,
             SLOT (setRotationTime (int)));
    connect (settingsDialog, SIGNAL (setAutoCollegeRotation (bool)), this,
             SLOT (setCollegeRotation (bool)));
    connect (settingsDialog, SIGNAL (setCollegeRotation (int)), this,
             SLOT (setCollegeRotationTime (int)));
    connect (settingsDialog, SIGNAL (setAutoGoToCollege (bool)), this,
             SLOT (setAutoGoToCollege (bool)));
    connect (settingsDialog, SIGNAL (setGoToCollegeTime (int)), this,
             SLOT (setAutoGoToCollegeTime (int)));
    connect (settingsDialog, SIGNAL (setAutoBirthday (bool)), this,
             SLOT (setAutoBirthdays (bool)));
    connect (settingsDialog, SIGNAL (setAutoBirthdayTime (int)), this,
             SLOT (setAutoBirthdaysTime (int)));
    connect (settingsDialog, SIGNAL (setAgeDisplay (int)), this, 
             SLOT (setAgeDisplay (int)));
    connect (settingsDialog, SIGNAL (setEPFlags (int)), this,
             SLOT (setEPFlags (int)));
    connect (settingsDialog, SIGNAL (setSimPELocation (QString)), this,
             SLOT (setSimPELocation (QString)));
  }

  settingsDialog->show ();
}

void SimSheet::createAddNotesDialog (int simIndex)  {
  if (addNotesDialog) delete addNotesDialog;
  simIndex = filterMap[simIndex];

  addNotesDialog = new AddNotesDialog 
                                 (simList[simIndex]->getNotes (), simIndex);

  connect (addNotesDialog, SIGNAL (saveNotes (QString, int)), this, 
           SLOT (saveNotes (QString, int)));

  addNotesDialog->show ();
}

void SimSheet::assignImageFile (int simIndex, QString filename)  {
  // In response to the signal triggered by adding an image, probably.
  simIndex = filterMap[simIndex];

  simList[simIndex]->setImageFile (filename);
}

void SimSheet::open (QString filename, int format)  {
  clearSimList ();

  QFile file (filename + ".sls");
  QTextStream in (&file);

  if (!file.open (QFile::ReadOnly | QFile::Text))  {
    QMessageBox::warning (this, "", "Cannot read file.");
    return;
  }

  QString line = in.readLine ();
  QPointer<Sim> s;

  cout << "Reading sims..." << endl;
  while (line != "")  {
    s = new Sim ();
    s->readCSV (line, format);
    simList.append (s);
    line = in.readLine ();
  }

  agePage->sort ();

  cout << "Updating various tabs..." << endl;
  updatePage->setSimList (simList);
  updateModels ();

  line = in.readLine ();
  
  cout << "Reading updates..." << endl;
  while (line != "")  {
    updatePage->readCSV (line);
    line = in.readLine ();
  }

  line = in.readLine ();

  cout << "Reading achievements..." << endl;
  while (line != "")  {
    achievementPage->readCSV (simList, line);
    line = in.readLine ();
  }

  line = in.readLine ();
/*
  if (!settingsDialog)  {
    createSettingsDialog ();
    settingsDialog->hide ();
  }
  settingsDialog->readCSV (line, format);*/

  if (format >= V0_2)  {
    line = in.readLine ();
    cout << "Reading household info..." << endl;
    while (line != "")  {
      householdPage->readCSV (line);
      line = in.readLine ();
    }
  }

  if (format >= V0_4)  {
    line = in.readLine ();

    int x = 0;
    while (line != "" && x < simList.size ())  {
      if (!(simList[x].isNull ()))
        simList[x]->readFamilyInfo (line, simList);
      line = in.readLine ();
      x++;
    }
  }

  cout << "Saving backup..." << endl;
  save (filename + ".bak");
  file.close ();
}

void SimSheet::save (QString filename)  {
  filename += ".sls";

  QFile file (filename);
  QTextStream out (&file);

  if (!file.open (QFile::WriteOnly | QFile::Text))  {
    QMessageBox::warning (this, "", "Cannot write file.");
    return;
  }

  for (int x = 0; x < simList.size (); x++)
    out << simList[x]->getCSV () << "\r\n";

  out << "\r\n";

  out << updatePage->getCSV ();
  out << "\r\n";

  out << achievementPage->getCSV (simList);
  out << "\r\n";

  if (!settingsDialog)  {
    createSettingsDialog ();
    settingsDialog->hide ();
  }

  out << settingsDialog->getCSV ();
  out << "\r\n";

  out << householdPage->getCSV ();
  out << "\r\n\r\n";

  for (int x = 0; x < simList.size (); x++)
    out << simList[x]->getFamilyInfo (simList) << "\r\n";

  file.close ();
}

void SimSheet::setDebug (int debugList)  {
  // Someday there will be a random sims list... maybe?

  clearSimList ();

  if (debugList == 1)
    makeTestSims ();

  updateModels ();
}

void SimSheet::clearSimList ()  {
  agePage->reset ();
  householdPage->reset ();

  simList.clear ();

  cout << "Deleting sim model rows..." << endl;
  if (agePage->sourceModel ())
    agePage->sourceModel ()->removeRows 
                              (0, agePage->sourceModel ()->rowCount ());
  else cout << "Source model for agePage not found." << endl;

  cout << "Deleting household model rows.." << endl;
  if (householdPage->sourceModel ())  {
    cout << householdPage << endl;
    cout << householdPage->sourceModel () << endl;
    cout << householdPage->sourceModel ()->rowCount () << endl;

    (householdPage->sourceModel ())->removeRows 
                             (0, (householdPage->sourceModel ())->rowCount ());
  }
  else cout << "Source model for householdPage not found." << endl;

  cout << "Clearing updatePage..." << endl;
  updatePage->clear ();
  updatePage->setSimList (simList);
  cout << "Clearing achievementPage..." << endl;
  achievementPage->clear ();
}

void SimSheet::setRotationTime (int t)  {
  rotationTime = t;
  updatePage->updateRotationLength (rotationTime);
}

void SimSheet::setCollegeRotation (bool b)  {
  rotateCollege = b;
}

void SimSheet::setCollegeRotationTime (int t)  {
  collegeRotationTime = t;
}

void SimSheet::setAutoGoToCollege (bool b)  {
  autoGoToCollege = b;
}

void SimSheet::setAutoGoToCollegeTime (int t)  {
  autoGoToCollegeTime = t;
}

void SimSheet::setAutoBirthdays (bool b)  {
  autoBirthdays = b;
}

void SimSheet::setAutoBirthdaysTime (int t)  {
  autoBirthdaysTime = t;
}

void SimSheet::setAgeDisplay (int ad)  {
  ageDisplay = ad;
  updateModels ();
}

void SimSheet::setEPFlags (int flags)  {
  epFlags = flags;
  updateModels ();
  aspirationPage->setEPFlags (epFlags);
  updatePage->updateEPFlags (epFlags);
}

void SimSheet::setSimPELocation (QString loc)  {
  simPELocation = loc;
}

void SimSheet::setAliveFilter (bool a)  {
  if ((bool)(aliveFilterFlags & ALIVE_FILTER) ^ a) 
    aliveFilterFlags ^= ALIVE_FILTER;
  updateModels ();
}

void SimSheet::setDeadFilter (bool d)  {
  if ((bool)(aliveFilterFlags & DEAD_FILTER) ^ d)
    aliveFilterFlags ^= DEAD_FILTER;
  updateModels ();
}

void SimSheet::setPlayableFilter (bool p)  {
  if ((bool)(typeFilterFlags & PLAYABLE_FILTER) ^ p)
    typeFilterFlags ^= PLAYABLE_FILTER;
  updateModels ();
}

void SimSheet::setTownieFilter (bool t)  {
  if((bool)(typeFilterFlags & TOWNIE_FILTER) ^ t)
    typeFilterFlags ^= TOWNIE_FILTER;
  updateModels ();
}

void SimSheet::setNPCFilter (bool n)  {
  if ((bool)(typeFilterFlags & NPC_FILTER) ^ n)
    typeFilterFlags ^= NPC_FILTER;
  updateModels ();
}

void SimSheet::setRunawayFilter (bool r)  {
  if ((bool)(typeFilterFlags & RUNAWAY_FILTER) ^ r)
    typeFilterFlags ^= RUNAWAY_FILTER;
  updateModels ();
}

void SimSheet::setAdoptionFilter (bool a)  {
  if ((bool)(typeFilterFlags & ADOPTION_FILTER) ^ a)
    typeFilterFlags ^= ADOPTION_FILTER;
  updateModels ();
}

void SimSheet::setDowntownieFilter (bool d)  {
  if ((bool)(typeFilterFlags & DOWNTOWNIE_FILTER) ^ d)
    typeFilterFlags ^= DOWNTOWNIE_FILTER;
  updateModels ();
}

void SimSheet::setGardenClubFilter (bool gc)  {
  if ((bool)(typeFilterFlags & GARDEN_CLUB_FILTER) ^ gc)
    typeFilterFlags ^= GARDEN_CLUB_FILTER;
  updateModels ();
}

void SimSheet::setIslandFilter (bool i)  {
  if ((bool)(typeFilterFlags & ISLAND_FILTER) ^ i)
    typeFilterFlags ^= ISLAND_FILTER;
  updateModels ();
}

void SimSheet::setMountainFilter (bool m)  {
  if ((bool)(typeFilterFlags & MOUNTAIN_FILTER) ^ m)
    typeFilterFlags ^= MOUNTAIN_FILTER;
  updateModels ();
}

void SimSheet::setAsianFilter (bool a)  {
  if ((bool)(typeFilterFlags & ASIAN_FILTER) ^ a)
    typeFilterFlags ^= ASIAN_FILTER;
  updateModels ();
}

void SimSheet::setTouristFilter (bool t)  {
  if ((bool)(typeFilterFlags & TOURIST_FILTER) ^ t)
    typeFilterFlags ^= TOURIST_FILTER;
  updateModels ();
}

void SimSheet::setHobbyFilter (bool h)  {
  if ((bool)(typeFilterFlags & HOBBY_FILTER) ^ h)
    typeFilterFlags ^= HOBBY_FILTER;
  updateModels ();
}

void SimSheet::setSocialiteFilter (bool s)  {
  if ((bool)(typeFilterFlags & SOCIALITE_FILTER) ^ s)
    typeFilterFlags ^= SOCIALITE_FILTER;
  updateModels ();
}

void SimSheet::setBohemianFilter (bool b)  {
  if ((bool)(typeFilterFlags & BOHEMIAN_FILTER) ^ b)
    typeFilterFlags ^= BOHEMIAN_FILTER;
  updateModels ();
}

void SimSheet::setJockFilter (bool j)  {
  if ((bool)(typeFilterFlags & JOCK_FILTER) ^ j)
    typeFilterFlags ^= JOCK_FILTER;
  updateModels ();
}

void SimSheet::setTechieFilter (bool t)  {
  if ((bool)(typeFilterFlags & TECHIE_FILTER) ^ t)
    typeFilterFlags ^= TECHIE_FILTER;
  updateModels ();
}

void SimSheet::setGearheadFilter (bool g)  {
  if ((bool)(typeFilterFlags & GEARHEAD_FILTER) ^ g)
    typeFilterFlags ^= GEARHEAD_FILTER;
  updateModels ();
}

void SimSheet::setHumanFilter (bool h)  {
  if ((bool)(speciesFilterFlags & HUMAN_FILTER) ^ h)
    speciesFilterFlags ^= HUMAN_FILTER;
  updateModels ();
}

void SimSheet::setLargeDogFilter (bool ld)  {
  if ((bool)(speciesFilterFlags & LARGE_DOG_FILTER) ^ ld)
    speciesFilterFlags ^= LARGE_DOG_FILTER;
  updateModels ();
}

void SimSheet::setSmallDogFilter (bool sd)  {
  if ((bool)(speciesFilterFlags & SMALL_DOG_FILTER) ^ sd)
    speciesFilterFlags ^= SMALL_DOG_FILTER;
  updateModels ();
}

void SimSheet::setCatFilter (bool c)  {
  if ((bool)(speciesFilterFlags & CAT_FILTER) ^ c)
    speciesFilterFlags ^= CAT_FILTER;
  updateModels ();
}

void SimSheet::setMaleFilter (bool m)  {
  if ((bool)(genderFilterFlags & MALE_FILTER) ^ m)
    genderFilterFlags ^= MALE_FILTER;
  updateModels ();
}

void SimSheet::setFemaleFilter (bool f)  {
  if ((bool)(genderFilterFlags & FEMALE_FILTER) ^ f)
    genderFilterFlags ^= FEMALE_FILTER;
  updateModels ();
}

void SimSheet::setBabyFilter (bool b)  {
  if ((bool)(ageFilterFlags & BABY_FILTER) ^ b)
    ageFilterFlags ^= BABY_FILTER;
  updateModels ();
}

void SimSheet::setToddlerFilter (bool t)  {
  if ((bool)(ageFilterFlags & TODDLER_FILTER) ^ t)
    ageFilterFlags ^= TODDLER_FILTER;
  updateModels ();
}

void SimSheet::setChildFilter (bool c)  {
  if ((bool)(ageFilterFlags & CHILD_FILTER) ^ c)
    ageFilterFlags ^= CHILD_FILTER;
  updateModels ();
}

void SimSheet::setTeenFilter (bool t)  {
  if ((bool)(ageFilterFlags & TEEN_FILTER) ^ t)
    ageFilterFlags ^= TEEN_FILTER;
  updateModels ();
}

void SimSheet::setYAFilter (bool y)  {
  if ((bool)(ageFilterFlags & YA_FILTER) ^ y)
    ageFilterFlags ^= YA_FILTER;
  updateModels ();
}

void SimSheet::setAdultFilter (bool a)  {
  if ((bool)(ageFilterFlags & ADULT_FILTER) ^ a)
    ageFilterFlags ^= ADULT_FILTER;
  updateModels ();
}

void SimSheet::setElderFilter (bool e)  {
  if ((bool)(ageFilterFlags & ELDER_FILTER) ^ e)
    ageFilterFlags ^= ELDER_FILTER;
  updateModels ();
}

void SimSheet::setNoneFilter (bool n)  {
  if ((bool)(aspirationFilterFlags & NONE_FILTER) ^ n)
    aspirationFilterFlags ^= NONE_FILTER;
  updateModels ();
}

void SimSheet::setGrowUpFilter (bool gu)  {
  if ((bool)(aspirationFilterFlags & GROW_UP_FILTER) ^ gu)
    aspirationFilterFlags ^= GROW_UP_FILTER;
  updateModels ();
}

void SimSheet::setFortuneFilter (bool f)  {
  if ((bool)(aspirationFilterFlags & FORTUNE_FILTER) ^ f)
    aspirationFilterFlags ^= FORTUNE_FILTER;
  updateModels ();
}

void SimSheet::setFamilyFilter (bool f)  {
  if ((bool)(aspirationFilterFlags & FAMILY_FILTER) ^ f)
    aspirationFilterFlags ^= FAMILY_FILTER;
  updateModels ();
}

void SimSheet::setPopularityFilter (bool p)  {
  if ((bool)(aspirationFilterFlags & POPULARITY_FILTER) ^ p)
    aspirationFilterFlags ^= POPULARITY_FILTER;
  updateModels ();
}

void SimSheet::setRomanceFilter (bool r)  {
  if ((bool)(aspirationFilterFlags & ROMANCE_FILTER) ^ r)
    aspirationFilterFlags ^= ROMANCE_FILTER;
  updateModels ();
}

void SimSheet::setKnowledgeFilter (bool k)  {
  if ((bool)(aspirationFilterFlags & KNOWLEDGE_FILTER) ^ k)
    aspirationFilterFlags ^= KNOWLEDGE_FILTER;
  updateModels ();
}

void SimSheet::setPleasureFilter (bool p)  {
  if ((bool)(aspirationFilterFlags & PLEASURE_FILTER) ^ p)
    aspirationFilterFlags ^= PLEASURE_FILTER;
  updateModels ();
}

void SimSheet::setGrilledCheeseFilter (bool gc)  {
  if ((bool)(aspirationFilterFlags & GRILLED_CHEESE_FILTER) ^ gc)
    aspirationFilterFlags ^= GRILLED_CHEESE_FILTER;
  updateModels ();
}

void SimSheet::setNormalFilter (bool n)  {
  if ((bool)(paranormalFilterFlags & NORMAL_FILTER) ^ n)
    paranormalFilterFlags ^= NORMAL_FILTER;
  updateModels ();
}

void SimSheet::setAlienFilter (bool a)  {
  if ((bool)(paranormalFilterFlags & ALIEN_FILTER) ^ a)
    paranormalFilterFlags ^= ALIEN_FILTER;
  updateModels ();
}

void SimSheet::setZombieFilter (bool z)  {
  if ((bool)(paranormalFilterFlags & ZOMBIE_FILTER) ^ z)
    paranormalFilterFlags ^= ZOMBIE_FILTER;
  updateModels ();
}

void SimSheet::setVampireFilter (bool v)  {
  if ((bool)(paranormalFilterFlags & VAMPIRE_FILTER) ^ v)
    paranormalFilterFlags ^= VAMPIRE_FILTER;
  updateModels ();
}

void SimSheet::setWerewolfFilter (bool w)  {
  if ((bool)(paranormalFilterFlags & WEREWOLF_FILTER) ^ w)
    paranormalFilterFlags ^= WEREWOLF_FILTER;
  updateModels ();
}

void SimSheet::setServoFilter (bool s)  {
  if ((bool)(paranormalFilterFlags & SERVO_FILTER) ^ s)
    paranormalFilterFlags ^= SERVO_FILTER;
  updateModels ();
}

void SimSheet::setPlantsimFilter (bool p)  {
  if ((bool)(paranormalFilterFlags & PLANTSIM_FILTER) ^ p)
    paranormalFilterFlags ^= PLANTSIM_FILTER;
  updateModels ();
}

void SimSheet::setBigfootFilter (bool b)  {
  if ((bool)(paranormalFilterFlags & BIGFOOT_FILTER) ^ b)
    paranormalFilterFlags ^= BIGFOOT_FILTER;
  updateModels ();
}

void SimSheet::setWitchFilter (bool w)  {
  if ((bool)(paranormalFilterFlags & WITCH_FILTER) ^ w)
    paranormalFilterFlags ^= WITCH_FILTER;
  updateModels ();
}

void SimSheet::saveNotes (QString n, int s)  {
  simList[s]->setNotes (n);

  int modelRow = 0;

  for (int x = 0; x < filterMap.size (); x++)
    if (filterMap[x] == s)
      modelRow = x;

  doUpdate (modelRow);
}

void SimSheet::updateModels ()  {
  // Makes the models (simModel displayed on agePage, and householdModel
  // displayed on householdPage) current to the actual simList and filter
  // settings.

  QAbstractItemModel *model = agePage->sourceModel ();
  int modelRow = 0;

  agePage->reset ();
  model->removeRows (0, model->rowCount ());
  filterMap.clear ();
  simsWithNIDs.clear ();

  for (int x = 0; x < simList.size (); x++)  {
    if (filterAcceptsSim (simList[x]))  {
      int special = 0;
      if (simList[x]->isParanormal (PLANTSIM)) special = PLANTSIM;
      if (simList[x]->isSpecies (LARGE_DOG)) special = LARGE_DOG;
      if (simList[x]->isSpecies (SMALL_DOG)) special = SMALL_DOG;
      if (simList[x]->isSpecies (CAT)) special = CAT;

      model->insertRow (modelRow);

      model->setData (model->index (modelRow, 0), simList[x]->getFullName (), 
                      Qt::DisplayRole);
      model->setData (model->index (modelRow, 1), simList[x]->getHousehold (),
                      Qt::DisplayRole);
      model->setData (model->index (modelRow, 2), 
                      ageFormat(simList[x]->getAge (), special),  
                      Qt::DisplayRole);
      model->setData (model->index (modelRow, 3), simList[x]->ageToQString (), 
                      Qt::DisplayRole);

      if (epFlags & FREETIME)
        model->setData (model->index (modelRow, 4), 
                        simList[x]->getAspiration () + "/" + 
                        simList[x]->getSecondaryAspiration (), Qt::DisplayRole);
      else
        model->setData (model->index (modelRow, 4), 
                        simList[x]->getAspiration (), Qt::DisplayRole);

      filterMap.append (x);
      modelRow++;
    }

    if (simList[x]->getNID () >= 0)
      simsWithNIDs.append (simList[x]);
  }

  // Sort simsWithNIDs
  for (int x = 0; x < simsWithNIDs.size (); x++)  {
    int smallest = x;

    for (int y = x + 1; y < simsWithNIDs.size (); y++)
      if (!simsWithNIDs[smallest].isNull () && !simsWithNIDs[y].isNull ())
        if (simsWithNIDs[smallest]->getNID () > simsWithNIDs[y]->getNID ())
          smallest = y;

    simsWithNIDs.swap (smallest, x);
  }

  agePage->setSourceModel (model);
  agePage->sort ();
  updatePage->setSimList (simList);
  familyPage->setSimList (simList);

  cout << "Updating household model..." << endl;
  model = householdPage->sourceModel ();

  QTextStream terminal (stdout);

  QMap<QString, bool> yesNo;

  for (int x = 0; x < model->rowCount (); x++)  {
    QString houseName = model->index (x, 0).data ().toString ();
    bool played = ((((model->index (x, 1)).data ()).toString ()) == "Yes");
    terminal << houseName << ", " << played << endl;

    yesNo.insert (houseName, played);
  }

  QStringList households;

  cout << "Compiling list of households..." << endl;
  for (int x = 0; x < simList.size (); x++)
    if ((!households.contains (simList[x]->getHousehold ())) &&
        (simList[x]->getHousehold () != ""))
      households.append (simList[x]->getHousehold ());

  householdPage->reset ();
  model->removeRows (0, model->rowCount ());

  cout << "Adding households to model..." << endl;
  for (int y = 0; y < households.size (); y++)  {
    model->insertRow (y);
    model->setData (model->index (y, 0), households[y], Qt::DisplayRole);

    cout << "Added household name" << endl;
    terminal << households[y] << endl;

    if (households[y] != "Runaways" && households[y] != "Adoption Queue" &&
        households[y] != "Townies and Dormies" && households[y] != "Downtown"&& 
        households[y] != "Garden Club" && households[y] != "Service NPCs" && 
        households[y] != "Island Locals" && households[y] != "Mountain Locals" 
        && households[y] != "Asian Locals" && households[y] != "Tourists" && 
        households[y] != "Socialites" && households[y] != "Gearheads" && 
        households[y] != "Techies" && households[y] != "Jocks" && 
        households[y] != "Bohemians" && households[y] != "Hobby Mavens")  {
      cout << yesNo.value (households[y]);

      if (yesNo.value (households[y], false))
        model->setData (model->index (y, 1), "Yes", Qt::DisplayRole);
      else model->setData (model->index (y, 1), "No", Qt::DisplayRole);
    }

    else model->setData (model->index (y, 1), "", Qt::DisplayRole);
  }

  householdPage->setSourceModel (model);
}

void SimSheet::readSettings ()  {
  QFile conf ("simsheet.conf");

  if (!conf.exists ()) return;

  QTextStream in (&conf);

  if (!conf.open (QFile::ReadOnly | QFile::Text))  {
    QMessageBox::warning (this, "", "Could not read simsheet.conf file.");
    return;
  }

  QString line = "";
  QRegExp regExp ("");

  while (!in.atEnd ())  {
    line = in.readLine ();
    if (line == "" || line.startsWith ("#")) continue;

    regExp.setPattern ("RotationTime=(\\d+)");
    if (regExp.exactMatch (line))  {
      rotationTime = regExp.cap(1).toInt ();
      continue;
    }

    regExp.setPattern ("RotateCollege=(\\d)");
    if (regExp.exactMatch (line)) {
      rotateCollege = (bool)(regExp.cap(1).toInt ());
      continue;
    }

    regExp.setPattern ("CollegeRotationTime=(\\d+)");
    if (regExp.exactMatch (line)) {
      collegeRotationTime = regExp.cap(1).toInt ();
      continue;
    }

    regExp.setPattern ("AutoGoToCollege=(\\d)");
    if (regExp.exactMatch (line)) {
      autoGoToCollege = (bool)(regExp.cap(1).toInt ());
      continue;
    }

    regExp.setPattern ("AutoGoToCollegeTime=(\\d+)");
    if (regExp.exactMatch (line)) {
      autoGoToCollegeTime = regExp.cap(1).toInt ();
      continue;
    }

    regExp.setPattern ("AutoBirthdays=(\\d)");
    if (regExp.exactMatch (line)) {
      autoBirthdays = (bool)(regExp.cap(1).toInt ());
      continue;
    }

    regExp.setPattern ("AutoBirthdaysTime=(\\d)");
    if (regExp.exactMatch (line))  {
      autoBirthdaysTime = regExp.cap(1).toInt ();
      continue;
    }

    regExp.setPattern ("AgeDisplay=(\\d)");
    if (regExp.exactMatch (line))  {
      ageDisplay = regExp.cap(1).toInt ();
      continue;
    }

    regExp.setPattern ("EPFlags=(\\d+)");
    if (regExp.exactMatch (line))  {
      epFlags = regExp.cap(1).toInt ();
      continue;
    }

    regExp.setPattern ("SimPELocation=(.+)");
    if (regExp.exactMatch (line))
      simPELocation = regExp.cap(1);
  }
}

void SimSheet::makeTestSims ()  {
  // Makes one of every kind of sim possible.
  
  QStringList name;
  name << "Male Baby" << "Female Baby" << "Male Toddler" << "Female Toddler"
       << "Male Plantsim Toddler" << "Female Plantsim Toddler"
       << "Male Child" << "Female Child" << "Male Teenager" 
       << "Female Teenager" << "Male Vampire Teenager" 
       << "Female Vampire Teenager" << "Male Werewolf Teenager"
       << "Female Werewolf Teenager" << "Male Zombie Teenager"
       << "Female Zombie Teenager" << "Male College Student"
       << "Female College Student" << "Male Vampire College Student"
       << "Female Vampire College Student" << "Male Werewolf College Student"
       << "Female Werewolf College Student" << "Male Zombie College Student"
       << "Female Zombie College Student" << "Male Adult" << "Female Adult"
       << "Male Vampire Adult" << "Female Vampire Adult"
       << "Male Werewolf Adult" << "Female Werewolf Adult"
       << "Male Plantsim Adult" << "Female Plantsim Adult"
       << "Male Zombie Adult" << "Female Zombie Adult"
       << "Male Elder" << "Female Elder" << "Male Vampire Elder"
       << "Female Vampire Elder" << "Male Werewolf Elder" 
       << "Female Werewolf Elder" << "Male Plantsim Elder"
       << "Female Plantsim Elder" << "Male Zombie Elder" 
       << "Female Zombie Elder" << "Male Servo" << "Female Servo"
       << "Bigfoot" << "Female Puppy (Large)" << "Male Puppy (Large)" 
       << "Female Large Dog" << "Male Large Dog" << "Elder Female Large Dog" 
       << "Elder Male Large Dog" << "Female Puppy (Small)" 
       << "Male Puppy (Small)" << "Female Small Dog" << "Male Small Dog"
       << "Elder Female Small Dog" << "Elder Male Small Dog"
       << "Female Kitten" << "Male Kitten" << "Female Cat" << "Male Cat"
       << "Elder Female Cat" << "Elder Male Cat";

  QList<int> age;
  age << BABY << BABY << TODDLER << TODDLER << PLANTSIM_TODDLER
      << PLANTSIM_TODDLER << CHILD << CHILD << TEEN << TEEN << TEEN << TEEN
      << TEEN << TEEN << TEEN << TEEN << YOUNG_ADULT << YOUNG_ADULT
      << YOUNG_ADULT << YOUNG_ADULT << YOUNG_ADULT << YOUNG_ADULT
      << YOUNG_ADULT << YOUNG_ADULT << ADULT << ADULT << ADULT << ADULT
      << ADULT << ADULT << PLANTSIM_ADULT << PLANTSIM_ADULT << ADULT << ADULT 
      << ELDER << ELDER << ELDER << ELDER << ELDER << ELDER << PLANTSIM_ELDER 
      << PLANTSIM_ELDER << ELDER << ELDER << ADULT << ADULT 
      << ADULT << PUPPY << PUPPY << ADULT_DOG << ADULT_DOG << ELDER_LARGE_DOG
      << ELDER_LARGE_DOG << PUPPY << PUPPY << ADULT_DOG << ADULT_DOG 
      << ELDER_SMALL_DOG << ELDER_SMALL_DOG << KITTEN << KITTEN << ADULT_CAT 
      << ADULT_CAT << ELDER_CAT << ELDER_CAT;

  QList<int> paraFlag;
  paraFlag << 0 << 0 << 0 << 0 << PLANTSIM << PLANTSIM << 0 << 0 << 0 << 0
           << VAMPIRE << VAMPIRE << WEREWOLF << WEREWOLF << ZOMBIE << ZOMBIE
           << 0 << 0 << VAMPIRE << VAMPIRE << WEREWOLF << WEREWOLF << ZOMBIE
           << ZOMBIE << 0 << 0 << VAMPIRE << VAMPIRE << WEREWOLF << WEREWOLF
           << PLANTSIM << PLANTSIM << ZOMBIE << ZOMBIE << 0 << 0 << VAMPIRE
           << VAMPIRE << WEREWOLF << WEREWOLF << PLANTSIM << PLANTSIM
           << ZOMBIE << ZOMBIE << SERVO << SERVO << BIGFOOT << LARGE_DOG 
           << LARGE_DOG << LARGE_DOG << LARGE_DOG << LARGE_DOG << LARGE_DOG 
           << SMALL_DOG << SMALL_DOG << SMALL_DOG << SMALL_DOG << SMALL_DOG 
           << SMALL_DOG << CAT << CAT << CAT << CAT << CAT << CAT;
 
  for (int x = 0; x < name.size (); x++)  {
    int gender = MALE;
    if (x) gender = x % 2 + 1;

    int aspiration;
    int secondaryAspiration;

    if (paraFlag[x] == LARGE_DOG || paraFlag[x] == SMALL_DOG || 
        paraFlag[x] == CAT) 
      aspiration = secondaryAspiration = NONE;
    else if (age[x] < TODDLER && age[x] != YOUNG_ADULT && paraFlag[x] != SERVO 
             && paraFlag[x] != PLANTSIM) 
      aspiration = secondaryAspiration = NONE;
    else if (age[x] < TEEN && age[x] != YOUNG_ADULT && paraFlag[x] != SERVO && 
             paraFlag[x] != PLANTSIM)  {
      aspiration = GROW_UP;
      secondaryAspiration = NONE;
    }
    else if (age[x] < PLANTSIM_ADULT && paraFlag[x] == PLANTSIM)  {
      aspiration = GROW_UP;
      secondaryAspiration = NONE;
    }
    else  {
      aspiration = rand () % 7 + 1;
      secondaryAspiration = rand () % 7 + 1;
    }

    QPointer<Sim> newSim = new Sim (name[x], age[x], gender, aspiration, 
                                    secondaryAspiration, true, paraFlag[x], "", 
                                    "");
    simList.append (newSim);
  }
}

void SimSheet::addSim (QString name, QString age, QString lifeStage,
                       QString aspiration)  {
  // Probably not necessary, but still potentially useful.

  QAbstractItemModel *source = agePage->sourceModel ();

  source->insertRow (0);
 
  source->setData (source->index (0, 0), name, Qt::DisplayRole);
  source->setData (source->index (0, 1), age, Qt::DisplayRole);
  source->setData (source->index (0, 2), lifeStage, Qt::DisplayRole);
  source->setData (source->index (0, 3), aspiration, Qt::DisplayRole);
}

QString SimSheet::ageFormat (int age, int special)  {
  QString text = "";

  if (!ageDisplay)  {
    text.setNum (age);
    return text + " days old";
  }

  if (special == PLANTSIM)  {
    if (age >= PLANTSIM_ELDER)  {
      text.setNum (age);
      return text + " days old";
    }

    if (age >= PLANTSIM_ADULT)  {
      text.setNum (PLANTSIM_ELDER - age);
      return text + " days until Elder";
    }

    else  {
      text.setNum (PLANTSIM_ADULT - age);
      return text + " days until Adult";
    }
  }

  else if (special == LARGE_DOG)  {
    if (age >= ELDER_LARGE_DOG)  {
      text.setNum (age);
      return text + " days old";
    }

    if (age >= ADULT_DOG)  {
      text.setNum (ELDER_LARGE_DOG - age);
      return text + " days until Elder";
    }

    else {
      text.setNum (ADULT_DOG - age);
      return text + " days until Adult";
    }
  }

  else if (special == SMALL_DOG)  {
    if (age >= ELDER_SMALL_DOG)  {
      text.setNum (age);
      return text + " days old";
    }

    if (age >= ADULT_DOG)  {
      text.setNum (ELDER_SMALL_DOG - age);
      return text + " days until Elder";
    }

    else  {
      text.setNum (ADULT_DOG - age);
      return text + " days until Adult";
    }
  }

  else if (special == CAT)  {
    if (age >= ELDER_CAT)  {
      text.setNum (age);
      return text + " days old";
    }

    if (age >= ADULT_CAT)  {
      text.setNum (ELDER_CAT - age);
      return text + " days until Elder";
    }

    else  {
      text.setNum (ADULT_CAT - age);
      return text + " days until Adult";
    }
  }

  else  {
    if (age >= ELDER)  {
      text.setNum (age - 4);
      return text + " days old";
    }

    if (age >= ADULT)  {
      text.setNum (ELDER - age);
      return text + " days until Elder";
    }

    if (age >= TEEN)  {
      text.setNum (ADULT - age);
      return text + " days until Adult";
    }

    if (age >= CHILD)  {
      text.setNum (TEEN - age);
      return text + " days until Teen";
    }

    if (age >= TODDLER)  {
      text.setNum (CHILD - age);
      return text + " days until Child";
    }

    else  {
      text.setNum (TODDLER - age);
      return text + " days until Toddler";
    }
  }
}

bool SimSheet::filterAcceptsSim (Sim *s)  {
  // aliveFilterFlags
  if (s->isAlive () && !(aliveFilterFlags & ALIVE_FILTER)) 
    return false;
  if (!(s->isAlive ()) && !(aliveFilterFlags & DEAD_FILTER)) 
    return false;

  // typeFilterFlags
  if (!(s->specialHousehold ()) && 
      !(typeFilterFlags & PLAYABLE_FILTER)) return false;
  if (s->getHousehold () == "At Large" &&
      !(typeFilterFlags & TOWNIE_FILTER)) return false;
  if (s->getHousehold () == "Service NPCs" &&
      !(typeFilterFlags & NPC_FILTER)) return false;
  if (s->getHousehold () == "Runaways" &&
      !(typeFilterFlags & RUNAWAY_FILTER)) return false;
  if (s->getHousehold () == "Adoption Queue" &&
      !(typeFilterFlags & ADOPTION_FILTER)) return false;
  if (s->getHousehold () == "Downtown" &&
      !(typeFilterFlags & DOWNTOWNIE_FILTER)) return false;
  if (s->getHousehold () == "Garden Club" &&
      !(typeFilterFlags & GARDEN_CLUB_FILTER)) return false;
  if (s->getHousehold () == "Island Locals" &&
      !(typeFilterFlags & ISLAND_FILTER)) return false;
  if (s->getHousehold () == "Mountain Locals" &&
      !(typeFilterFlags & MOUNTAIN_FILTER)) return false;
  if (s->getHousehold () == "Asian Locals" &&
      !(typeFilterFlags & ASIAN_FILTER)) return false;
  if (s->getHousehold () == "Tourists" &&
      !(typeFilterFlags & TOURIST_FILTER)) return false;
  if (s->getHousehold () == "Hobby Mavens" &&
      !(typeFilterFlags & HOBBY_FILTER)) return false;
  if (s->getHousehold () == "Socialites" &&
      !(typeFilterFlags & SOCIALITE_FILTER)) return false;
  if (s->getHousehold () == "Bohemians" &&
      !(typeFilterFlags & BOHEMIAN_FILTER)) return false;
  if (s->getHousehold () == "Jocks" &&
      !(typeFilterFlags & JOCK_FILTER)) return false;
  if (s->getHousehold () == "Techies" &&
      !(typeFilterFlags & TECHIE_FILTER)) return false;
  if (s->getHousehold () == "Gearheads" &&
      !(typeFilterFlags & GEARHEAD_FILTER)) return false;

  // speciesFilterFlags
  if (s->isSpecies (HUMAN) && !(speciesFilterFlags & HUMAN_FILTER))
    return false;
  if (s->isSpecies (LARGE_DOG) && !(speciesFilterFlags & LARGE_DOG_FILTER))
    return false;
  if (s->isSpecies (SMALL_DOG) && !(speciesFilterFlags & SMALL_DOG_FILTER))
    return false;
  if (s->isSpecies (CAT) && !(speciesFilterFlags & CAT_FILTER))
    return false;

  // genderFilterFLags
  if ((s->getGender () == MALE) && !(genderFilterFlags & MALE_FILTER))
   return false;
  if ((s->getGender () == FEMALE) && !(genderFilterFlags & FEMALE_FILTER))
    return false;

  // ageFilterFlags
  if ((s->ageToQString ().contains ("Baby") || 
       s->ageToQString ().contains ("Puppy") ||
       s->ageToQString ().contains ("Kitten")) && 
      !(ageFilterFlags & BABY_FILTER))
    return false;
  if (s->ageToQString ().contains ("Toddler") && 
      !(ageFilterFlags & TODDLER_FILTER))
    return false;
  if (s->ageToQString ().contains ("Child") &&
      !(ageFilterFlags & CHILD_FILTER))
    return false;
  if (s->ageToQString ().contains ("Teen") &&
      !(ageFilterFlags & TEEN_FILTER))
    return false;
  if (s->ageToQString ().contains ("Major") &&
      !(ageFilterFlags & YA_FILTER))
    return false;
  if ((s->ageToQString ().contains ("Adult") ||
       s->ageToQString () == "Large Dog" ||
       s->ageToQString () == "Small Dog" ||
       s->ageToQString () == "Cat") && !(ageFilterFlags & ADULT_FILTER))
    return false;
  if (s->ageToQString ().contains ("Elder") &&
      !(ageFilterFlags & ELDER_FILTER))
    return false;

  // aspirationFilterFlags
  if ((s->getAspiration () == "None" || s->getAspiration () == "Unknown") &&
      !(aspirationFilterFlags & NONE_FILTER))
    return false;
  if (s->getAspiration () == "Grow Up" && 
      !(aspirationFilterFlags & GROW_UP_FILTER))
    return false;
  if (s->getAspiration () == "Fortune" &&
      !(aspirationFilterFlags & FORTUNE_FILTER))
    return false;
  if (s->getAspiration () == "Family" &&
      !(aspirationFilterFlags & FAMILY_FILTER))
    return false;
  if (s->getAspiration () == "Popularity" &&
      !(aspirationFilterFlags & POPULARITY_FILTER))
    return false;
  if (s->getAspiration () == "Romance" &&
      !(aspirationFilterFlags & ROMANCE_FILTER))
    return false;
  if (s->getAspiration () == "Knowledge" &&
      !(aspirationFilterFlags & KNOWLEDGE_FILTER))
    return false;
  if (s->getAspiration () == "Pleasure" &&
      !(aspirationFilterFlags & PLEASURE_FILTER))
    return false;
  if (s->getAspiration () == "Grilled Cheese" &&
      !(aspirationFilterFlags & GRILLED_CHEESE_FILTER))
    return false;

  // paranormalFilterFlags
  if (s->isParanormal (HUMAN) && !(paranormalFilterFlags & HUMAN_FILTER))
    return false;
  if (s->isParanormal (ALIEN) && !(paranormalFilterFlags & ALIEN_FILTER))
    return false;
  if (s->isParanormal (ZOMBIE) && !(paranormalFilterFlags & ZOMBIE_FILTER))
    return false;
  if (s->isParanormal (VAMPIRE) && !(paranormalFilterFlags & VAMPIRE_FILTER))
    return false;
  if (s->isParanormal (WEREWOLF) && !(paranormalFilterFlags & WEREWOLF_FILTER))
    return false;
  if (s->isParanormal (SERVO) && !(paranormalFilterFlags & SERVO_FILTER))
    return false;
  if (s->isParanormal (PLANTSIM) && !(paranormalFilterFlags & PLANTSIM_FILTER))
    return false;
  if (s->isParanormal (BIGFOOT) && !(paranormalFilterFlags & BIGFOOT_FILTER))
    return false;
  if (s->isParanormal (WITCH) && !(paranormalFilterFlags & WITCH_FILTER))
    return false;

  return true;
}
