#ifndef SIMSHEET_H
#define SIMSHEET_H

#include <QTabWidget>
#include <QList>
#include <QList>
#include <QPointer>

#include "const.h"
#include "sim.h"

class AgePage;
class HouseholdPage;
class AchievementsPage;
class UpdatePage;
class FamilyPage;
class AspirationPage;
class QAbstractItemModel;
class QStandardItemModel;
class QModelIndex;
class ListManagementDialog;
class AddSimDialog;
class EditSimDialog;
class FilterDialog;
class SettingsDialog;
class AddNotesDialog;
class QMainWindow;
class QCloseEvent;

class SimSheet : public QTabWidget {
  Q_OBJECT

  public:
    SimSheet (QMainWindow *window);

    void setSimModel (QStandardItemModel*);
    void setHouseholdModel (QStandardItemModel*);
    void setFamilyModel (QStandardItemModel*);
    void saveSettings ();

  private slots:
    void doUpdate (int);
    void doRotation ();
    void holdAllBut (Sim*, int);
    void holdHousehold (QString, int);
    void holdAllHouseholdsBut (QString, int);
    void newSim (QString, int, int, int, int, QString = "", int = NONE, 
                 Sim* = NULL, Sim* = NULL);
      // Used by updatePage
    void newSim (QString, int, int, int, int, bool, int, QString, int, int, 
                 int, QString);
      // Used by addSimDialog
    void newSim (Sim*);
      // Used by importDialog
    void newTowniePair (QString, int, QString, int, int, int, Sim*);
    void removeSim (Sim*);
    void showHousehold (QString);
    void replaceHouseholdName (QString, QString);

    void createListManagementDialog ();
    void createAddSimDialog ();
    void createEditSimDialog (int);
    void createFilterDialog ();
    void createSettingsDialog ();
    void createAddNotesDialog (int);
    void assignImageFile (int, QString);

    void open (QString, int);
    void save (QString);
    void setDebug (int);
    void clearSimList ();

    void setRotationTime (int);
    void setCollegeRotation (bool);
    void setCollegeRotationTime (int);
    void setAutoGoToCollege (bool);
    void setAutoGoToCollegeTime (int);
    void setAutoBirthdays (bool);
    void setAutoBirthdaysTime (int);
    void setAgeDisplay (int);
    void setEPFlags (int);
    void setSimPELocation (QString);

    // setting filters
    void setAliveFilter (bool);
    void setDeadFilter (bool);

    void setPlayableFilter (bool);
    void setTownieFilter (bool);
    void setNPCFilter (bool);
    void setRunawayFilter (bool);
    void setAdoptionFilter (bool);
    void setDowntownieFilter (bool);
    void setGardenClubFilter (bool);
    void setIslandFilter (bool);
    void setMountainFilter (bool);
    void setAsianFilter (bool);
    void setTouristFilter (bool);
    void setHobbyFilter (bool);
    void setSocialiteFilter (bool);
    void setBohemianFilter (bool);
    void setJockFilter (bool);
    void setTechieFilter (bool);
    void setGearheadFilter (bool);

    void setHumanFilter (bool);
    void setLargeDogFilter (bool);
    void setSmallDogFilter (bool);
    void setCatFilter (bool);

    void setMaleFilter (bool);
    void setFemaleFilter (bool);

    void setBabyFilter (bool);
    void setToddlerFilter (bool);
    void setChildFilter (bool);
    void setTeenFilter (bool);
    void setYAFilter (bool);
    void setAdultFilter (bool);
    void setElderFilter (bool);

    void setNoneFilter (bool);
    void setGrowUpFilter (bool);
    void setFortuneFilter (bool);
    void setFamilyFilter (bool);
    void setPopularityFilter (bool);
    void setRomanceFilter (bool);
    void setKnowledgeFilter (bool);
    void setPleasureFilter (bool);
    void setGrilledCheeseFilter (bool);

    void setNormalFilter (bool);
    void setAlienFilter (bool);
    void setZombieFilter (bool);
    void setVampireFilter (bool);
    void setWerewolfFilter (bool);
    void setServoFilter (bool);
    void setPlantsimFilter (bool);
    void setBigfootFilter (bool);
    void setWitchFilter (bool);

    void saveNotes (QString, int);

    void updateModels ();

  private:
    void readSettings ();
    void makeTestSims ();
    void addSim (QString, QString, QString, QString);
    QString ageFormat (int, int);
    bool filterAcceptsSim (Sim*);

    QList< QPointer<Sim> > simList;
    QList< QPointer<Sim> > simsWithNIDs;
    QList<int> filterMap;

    ListManagementDialog *listManagementDialog;
    AddSimDialog *addSimDialog;
    EditSimDialog *editSimDialog;
    FilterDialog *filterDialog;
    SettingsDialog *settingsDialog;
    AddNotesDialog *addNotesDialog;

    AgePage *agePage;
    AspirationPage *aspirationPage;
    FamilyPage *familyPage;
    HouseholdPage *householdPage;
    QWidget *careerPage;
    AchievementsPage *achievementPage;
    UpdatePage *updatePage;

    // settings
    int rotationTime;
    bool rotateCollege;
    int collegeRotationTime;
    bool autoGoToCollege;
    int autoGoToCollegeTime;
    bool autoBirthdays;
    int autoBirthdaysTime;
    int ageDisplay;
    int epFlags;
    QString simPELocation;

    // filters
    int aliveFilterFlags;
    int typeFilterFlags;
    int speciesFilterFlags;
    int genderFilterFlags;
    int ageFilterFlags;
    int aspirationFilterFlags;
    int paranormalFilterFlags;
};

#endif
