#include <QString>
#include <QStringList>
#include <QLabel>
#include <QComboBox>
#include <QSpinBox>
#include <QLineEdit>
#include <QPushButton>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QLayoutItem>
#include <QTextStream>

#include <iostream>
using namespace std;

#include "updatedialog.h"
#include "updateentry.h"
#include "sim.h"

UpdateDialog::UpdateDialog ()  {
  warningLabel = new QLabel ("");
  addButton = new QPushButton ("Add");
  addButton->setMaximumSize (addButton->sizeHint ());
  textLayout = new QVBoxLayout;
  mainLayout = new QVBoxLayout;

  mainLayout->addLayout (textLayout);
  mainLayout->addWidget (warningLabel);
  mainLayout->setAlignment (warningLabel, Qt::AlignCenter);
  mainLayout->addWidget (addButton);
  mainLayout->setAlignment (addButton, Qt::AlignCenter);

  setLayout (mainLayout);

  connect (addButton, SIGNAL (clicked ()), this, SLOT (makeUpdate ()));
}

void UpdateDialog::addText (QString text)  {
  QLabel *newText = new QLabel (text);

  makeSpace ()->addWidget (newText);
}

void UpdateDialog::addChoice (QStringList list)  {
  QComboBox *newChoice = new QComboBox;
  newChoice->setEditable (false);
  newChoice->addItems (list);
  newChoice->setMaximumSize (newChoice->sizeHint ());

  makeSpace ()->addWidget (newChoice);
}

void UpdateDialog::addSimChoice (QList< QPointer<Sim> > list)  {
  sortList (list);

  if (firstSimList.isEmpty ())
    firstSimList = list;
  else secondSimList = list;

  QComboBox *simChoice = new QComboBox;
  simChoice->setEditable (true);

  for (int x = 0; x < list.size (); x++)
    simChoice->addItem (list[x]->getFullName ());

  simChoice->setSizeAdjustPolicy (QComboBox::AdjustToContents);

  makeSpace ()->addWidget (simChoice);
}

void UpdateDialog::addNumberChoice (int start, int end)  {
  QSpinBox *numberChoice = new QSpinBox;
  numberChoice->setRange (start, end);
  makeSpace ()->addWidget (numberChoice);
}

void UpdateDialog::addEdit ()  {
  QLineEdit *newEdit = new QLineEdit;
  newEdit->setMinimumWidth (150);
  makeSpace ()->addWidget (newEdit);

  connect (newEdit, SIGNAL (textEdited (QString)), this, 
           SLOT (setWarning (QString)));
}

void UpdateDialog::reset ()  {
  for (int x = 0; x < textLayout->count (); x++)  {
    QLayout *lay = textLayout->itemAt (x)->layout ();

    for (int y = 0; y < lay->count (); y++)  {
      QLayoutItem *item = lay->itemAt (y);
      lay->removeItem (item);
      delete item;
    }

    textLayout->removeItem (lay);
    delete lay;
  }

  mainLayout->removeItem (textLayout);
  delete textLayout;
  textLayout = new QVBoxLayout;
  mainLayout->insertLayout (0, textLayout);
  repaint ();
  close ();
}

void UpdateDialog::setWarning (QString text)  {
  QString warning = (QString)"Please don't use a double-quote followed by a " +
                    (QString)"comma in the fields; it will mess up your save" +
                    (QString)" files.";

  if (text.contains ("\","))  {
    warningLabel->setText (warning);
    addButton->setEnabled (false);
  }
  else  {
    warningLabel->setText ("");
    addButton->setEnabled (true);
  }
}

void UpdateDialog::makeUpdate ()  {
  QString str = getAllText ();
  QPointer<Sim> sim1 = NULL;
  QPointer<Sim> sim2 = NULL;

  for (int x = 0; x < textLayout->count (); x++)  {
    QHBoxLayout *lay = qobject_cast<QHBoxLayout*>
                          (textLayout->itemAt (x)->layout ());

    for (int y = 0; y < lay->count (); y++)  {
      QComboBox *ptr = qobject_cast<QComboBox*>(lay->itemAt(y)->widget());

      if (ptr)  {
        if (ptr->isEditable () && !sim1)
          sim1 = firstSimList[ptr->currentIndex ()];
        else if (ptr->isEditable ())
          sim2 = secondSimList[ptr->currentIndex ()];
      }
    }
  }

  while (textLayout->count () > 0)  {
    QHBoxLayout *lay = qobject_cast<QHBoxLayout*>
                          (textLayout->itemAt (0)->layout ());

    while (lay->count () > 0)  {
      QLayoutItem *item = lay->itemAt (0);
      lay->removeItem (item);
      delete item;
    }

    textLayout->removeItem (lay);
    delete lay;
  }

  UpdateEntry *update = new UpdateEntry (str, sim1, sim2);

  emit newEvent (update);

  reset ();
}

QHBoxLayout *UpdateDialog::makeSpace ()  {
  if (textLayout->count () == 0)  {
    QHBoxLayout *lay = new QHBoxLayout;
    textLayout->addLayout (lay);
    textLayout->setAlignment (lay, Qt::AlignCenter);
    return lay;
  }

  else  {
    QHBoxLayout *currentLine = qobject_cast<QHBoxLayout*>
              (textLayout->itemAt (textLayout->count () - 1)->layout ());

    if (currentLine->minimumSize ().width () > 700)  {
      QHBoxLayout *newLine = new QHBoxLayout;
      textLayout->addLayout (newLine);
      textLayout->setAlignment (newLine, Qt::AlignCenter);
      return newLine;
    }

    return currentLine;
  }
}

QString UpdateDialog::getAllText ()  {
  QString text = "";

  for (int x = 0; x < textLayout->count (); x++)  {
    QLayout *lay = textLayout->itemAt (x)->layout ();

    for (int y = 0; y < lay->count (); y++)  {
      QWidget *w = lay->itemAt (y)->widget ();
  
      if (qobject_cast<QLabel*>(w))
        text += qobject_cast<QLabel*>(w)->text ();

      if (qobject_cast<QComboBox*>(w))  {
        if (qobject_cast<QComboBox*>(w)->currentText () == "Random")  {
          QStringList asp = QStringList () << "Fortune" << "Family" 
                                           << "Pleasure" << "Knowledge" 
                                           << "Popularity" << "Romance";

          int choice = rand () % 6;
          text += asp[choice];
        }

        else text += qobject_cast<QComboBox*>(w)->currentText ();
      }

      if (qobject_cast<QSpinBox*>(w))
        text += qobject_cast<QSpinBox*>(w)->cleanText ();

      if (qobject_cast<QLineEdit*>(w))
        text += qobject_cast<QLineEdit*>(w)->text ();
    }
  }

  return text;
}

void UpdateDialog::sortList (QList< QPointer<Sim> > &list)  {
  for (int x = 0; x < list.size (); x++)  {
    int smallest = x;

    for (int y = x + 1; y < list.size (); y++)
      if (!(list[smallest].isNull ()) && !(list[y].isNull ()))
        if (QString::localeAwareCompare (list[smallest]->getFullName (),
                                         list[y]->getFullName ()) > 0)
          smallest = y;

    list.swap (smallest, x);
  }
}
