#include <QLabel>
#include <QPushButton>
#include <QHBoxLayout>
#include <QPointer>
#include <QTextStream>
#include <QMessageBox>

#include <iostream>
using namespace std;

#include "updateentry.h"

UpdateEntry::UpdateEntry (QString t, Sim *s, Sim *s2)
{
  display = new QLabel ();
  display->setText (t);
  display->setWordWrap (true);
  display->setMinimumWidth (500);
  deleteButton = new QPushButton ("Delete");
  connect (deleteButton, SIGNAL (clicked ()), this, SLOT (deleteMe ()));

  layout = new QHBoxLayout ();
  layout->addWidget (display);
  layout->setStretchFactor (display, 1);
  layout->addWidget (deleteButton);
  layout->setSizeConstraint (QLayout::SetFixedSize);

  setLayout (layout);

  firstSim = s;
  secondSim = s2;
}

void UpdateEntry::go ()  {
  // The regExps from hell function.  The variable regExp gets set to various
  // patterns in an attempt to match the text of the updateEntry to something
  // that should have come from an updateDialog.  The regExps must match the
  // text of the dialog EXACTLY, or the update won't work and I'll spend hours
  // and hours chasing red herrings around trying to find out what went wrong.

  QString p = "";
  QRegExp regExp;

  // New Sims

  p = (QString)"^On day (\\d+) of the rotation, (.+) \\((male|female)\\), of" +
      (QString)" the (.+) household, joined the neighborhood (before|after) " +
      (QString)"six PM \\(eleven AM for pets\\) as an? (.+) \\((.+) " +
      (QString)"aspiration\\)\\.";
  regExp.setPattern (p);

  if (regExp.exactMatch (display->text ()))  {
    int gender = regExp.cap(3) == "male" ? MALE : FEMALE;

    int age = 0;
    if (regExp.cap(6) == "Toddler") age = TODDLER;
    else if (regExp.cap(6) == "Child") age = CHILD;
    else if (regExp.cap(6) == "Teenager") age = TEEN;
    else if (regExp.cap(6) == "Young Adult") age = YOUNG_ADULT;
    else if (regExp.cap(6) == "Adult") age = ADULT;
    else if (regExp.cap(6) == "Elder") age = ELDER;
    else if (regExp.cap(6) == "large dog" || regExp.cap(6) == "small dog") 
      age = ADULT_DOG;
    else if (regExp.cap(6) == "cat") age = ADULT_CAT;
    else if (regExp.cap(6) == "elder large dog") age = ELDER_LARGE_DOG;
    else if (regExp.cap(6) == "elder small dog") age = ELDER_SMALL_DOG;
    else if (regExp.cap(6) == "elder cat") age = ELDER_CAT;

    int asp;
    if (regExp.cap(6) == "Toddler" || regExp.cap(6) == "Child") 
      asp = GROW_UP;
    if (regExp.cap(6).contains ("dog") || regExp.cap(6).contains ("cat"))
      asp = NONE;
    else asp = convertAspiration (regExp.cap(7));

    int holdDays = (regExp.cap(5) == "before" ? regExp.cap(1).toInt () - 1
                                              : regExp.cap(1).toInt ());

    int species = HUMAN;
    if (regExp.cap(6).contains ("large dog"))
      species = LARGE_DOG;
    if (regExp.cap(6).contains ("small dog"))
      species = SMALL_DOG;
    if (regExp.cap(6).contains ("cat"))
      species = CAT;

    emit newSim (regExp.cap(2), gender, age, asp, holdDays, regExp.cap(4), 
                 species);
  }

  p = (QString)"^On day (\\d+) of the rotation, (.+) \\((male|female), (.+)" +
      (QString)" aspiration, (.+)\\) (moved in with|got married/joined to) " +
      (QString)".+ (before|after) six PM.";
  regExp.setPattern (p);

  if (regExp.exactMatch (display->text ()))  {
    cout << "Matched townie pattern." << endl;

    int gender = regExp.cap(3) == "male" ? MALE : FEMALE;
    int asp = convertAspiration (regExp.cap(4));
    int age = 0;
    if (regExp.cap(5) == "Adult") age = ADULT;
    if (regExp.cap(5) == "Young Adult") age = YOUNG_ADULT;
    if (regExp.cap(5) == "Elder") age = ELDER;
    int holdDays = (regExp.cap(7) == "before" ? regExp.cap(1).toInt () - 1
                                              : regExp.cap(1).toInt ());
    QString house = firstSim->getHousehold ();

    emit newSim (regExp.cap(2), gender, age, asp, holdDays, house);
  }

  p = (QString)"^On day (\\d+) of the rotation, .+ \\((.+) Aspiration\\) " +
      (QString)"(moved in with|got married/joined to) .+ (before|after) six" +
      (QString)" PM\\.";
  regExp.setPattern (p);

  if (regExp.exactMatch (display->text ()))  {
    firstSim->newAspiration (convertAspiration (regExp.cap(2)), 0);
    firstSim->setHousehold (secondSim->getHousehold ());
    if (regExp.cap(4) == "after") firstSim->setHold (regExp.cap(1).toInt ());
    else firstSim->setHold (regExp.cap(1).toInt () - 1);
  }
      
  p = (QString)"^On day (\\d+) of the rotation, .+ adopted a stray " +
      (QString)"(male|female) (large dog|small dog|cat) named (.+) " +
      (QString)"(before|after) eleven AM\\.";
  regExp.setPattern (p);

  if (regExp.exactMatch (display->text ()))  {
    int gender = regExp.cap(2) == "male" ? MALE : FEMALE;

    int species = HUMAN;
    if (regExp.cap(3) == "large dog") species = LARGE_DOG;
    if (regExp.cap(3) == "small dog") species = SMALL_DOG;
    if (regExp.cap(3) == "cat") species = CAT;

    int age;
    if (species == LARGE_DOG || species == SMALL_DOG) age = ADULT_DOG;
    else age = ADULT_CAT;

    int holdDays = (regExp.cap(6) == "before" ? regExp.cap(1).toInt () - 1
                                              : regExp.cap(1).toInt ());
    QString house = firstSim->getHousehold ();

    emit newSim (regExp.cap(4), gender, age, NONE, holdDays, house, species);
  }

  p = (QString)"^On day (\\d+) of the rotation, .+ adopted .+ (before|after)" +
      (QString)" eleven AM\\.";
  regExp.setPattern (p);

  if (regExp.exactMatch (display->text ()))  {
    secondSim->setHousehold (firstSim->getHousehold ());
    if (regExp.cap(2) == "after") secondSim->setHold (regExp.cap(1).toInt ());
    else secondSim->setHold (regExp.cap(1).toInt () - 1);
  }

  p = (QString)"^On day (\\d+) of the rotation, .+ and .+ had a (male|female)" +
      (QString)" baby named (.+) (before|after) six PM.";
  regExp.setPattern (p);

  if (regExp.exactMatch (display->text ()))  {
    cout << "Matched baby pattern." << endl;

    int gender = regExp.cap(2) == "male" ? MALE : FEMALE;
    int holdDays = (regExp.cap(4) == "before" ? regExp.cap(1).toInt () - 1
                                              : regExp.cap(1).toInt ());
    QString house = secondSim->getHousehold ();

    emit newSim (regExp.cap(3), gender, 0, NONE, holdDays, house, NONE, 
                 firstSim, secondSim);
  }

  p = (QString)"^On day (\\d+) of the rotation, .+ had a (male|female) baby " +
      (QString)"named (.+) by an (adult|elder) townie named (.+) " +
      (QString)"(before|after) six PM.";
  regExp.setPattern (p);

  if (regExp.exactMatch (display->text ()))  {
    int babyGender = regExp.cap(2) == "male" ? MALE : FEMALE;
    int holdDays = (regExp.cap(6) == "before" ? regExp.cap(1).toInt () - 1
                                              : regExp.cap(1).toInt ());
    int townieAge = regExp.cap(4) == "adult" ? ADULT : ELDER;

    emit newTowniePair (regExp.cap(5), townieAge, regExp.cap(3), babyGender,
                        holdDays, HUMAN, firstSim);
  }

  p = (QString)"^On day (\\d+) of the rotation, .+ and .+ had a (male|female)" +
      (QString)" (puppy|kitten) named (.+) (before|after) eleven AM\\.";
  regExp.setPattern (p);

  if (regExp.exactMatch (display->text ()))  {
    int gender = regExp.cap(2) == "male" ? MALE : FEMALE;
    int holdDays = (regExp.cap(5) == "before" ? regExp.cap(1).toInt () - 1
                                              : regExp.cap(1).toInt ());
    QString house = secondSim->getHousehold ();

    int species = regExp.cap(3) == "kitten" ? CAT : LARGE_DOG;
    if (firstSim->isSpecies (SMALL_DOG)) species = SMALL_DOG;

    emit newSim (regExp.cap(4), gender, 0, NONE, holdDays, house, species, 
                 firstSim, secondSim);
  }

  p = (QString)"^On day (\\d+) of the rotation, .+ had a (male|female) " +
      (QString)"(puppy|kitten) named (.+) by an (adult|elder) stray named " +
      (QString)"(.+) (before|after) eleven AM\\.";
  regExp.setPattern (p);

  if (regExp.exactMatch (display->text ()))  {
    int babyGender = regExp.cap(2) == "male" ? MALE : FEMALE;

    int species = regExp.cap(3) == "kitten" ? CAT : LARGE_DOG;
    if (firstSim->isSpecies (SMALL_DOG)) species = SMALL_DOG;

    int strayAge;
    if ((species == LARGE_DOG || species == SMALL_DOG) && 
        regExp.cap(5) == "adult")
      strayAge = ADULT_DOG;
    else if (regExp.cap(5) == "adult")
      strayAge = ADULT_CAT;
    else if (species == LARGE_DOG) 
      strayAge = ELDER_LARGE_DOG;
    else if (species == SMALL_DOG)
      strayAge = ELDER_SMALL_DOG;
    else strayAge = ELDER_CAT;

    int holdDays = (regExp.cap(7) == "before" ? regExp.cap(1).toInt () - 1
                                              : regExp.cap(1).toInt ());

    emit newTowniePair (regExp.cap(6), strayAge, regExp.cap(4), babyGender,
                        holdDays, species, firstSim);
  }

  p = (QString)"^On day (\\d+) of the rotation, .+ had a (male|female) alien" +
      (QString)" baby named (.+) (before|after) six PM.";
  regExp.setPattern (p);

  if (regExp.exactMatch (display->text ()))  {
    cout << "Matched alien baby pattern." << endl;

    int gender = regExp.cap(2) == "male" ? MALE : FEMALE;
    int holdDays = (regExp.cap(4) == "before" ? regExp.cap(1).toInt () - 1
                                              : regExp.cap(1).toInt ());
    QString house = firstSim->getHousehold ();

    emit newSim (regExp.cap(3), gender, 0, NONE, holdDays, house, ALIEN, 
                 firstSim);
  }

  p = (QString)"^On day (\\d+) of the rotation, .+ spawned a (male|female)" +
      (QString)" plantbaby named (.+) (before|after) six PM.";
  regExp.setPattern (p);

  if (regExp.exactMatch (display->text ()))  {
    cout << "Matched plantbaby pattern." << endl;

    int gender = regExp.cap(2) == "male" ? MALE : FEMALE;
    int holdDays = (regExp.cap(4) == "before" ? regExp.cap(1).toInt () - 1
                                              : regExp.cap(1).toInt ());
    QString house = firstSim->getHousehold ();

    emit newSim (regExp.cap(3), gender, 0, NONE, holdDays, house, PLANTSIM, 
                 firstSim);
  }

  p = (QString)"^On day (\\d+) of the rotation, .+ adopted a (male|female)" +
      (QString)" (.+) named (.+) (before|after) six PM " +
      (QString)"\\(eleven AM for pets\\)\\.";
  regExp.setPattern (p);

  if (regExp.exactMatch (display->text ()))  {
    cout << "Matched adoption pattern." << endl;

    int gender = regExp.cap(2) == "male" ? MALE : FEMALE;

    int age = 0;
    if (regExp.cap(3) == "baby" || regExp.cap(3).contains ("puppy") ||
        regExp.cap(3) == "kitten") 
      age = 0;
    else if (regExp.cap(3) == "toddler") age = TODDLER;
    else if (regExp.cap(3) == "child") age = CHILD;
    else if (regExp.cap(3) == "small dog" || regExp.cap(3) == "large dog")
      age = ADULT_DOG;
    else if (regExp.cap(3) == "cat") age = ADULT_CAT;
    else if (regExp.cap(3) == "elder large dog") age = ELDER_LARGE_DOG;
    else if (regExp.cap(3) == "elder small dog") age = ELDER_SMALL_DOG;
    else if (regExp.cap(3) == "elder cat") age = ELDER_CAT;

    int species = HUMAN;
    if (regExp.cap(3).contains ("large"))
      species = LARGE_DOG;
    if (regExp.cap(3).contains ("small"))
      species = SMALL_DOG;
    if (regExp.cap(3) == "kitten" || regExp.cap(3).contains ("cat"))
      species = CAT;

    int asp = NONE;
    if (species == HUMAN && age) asp = GROW_UP;
    int holdDays = (regExp.cap(5) == "before" ? regExp.cap(1).toInt () - 1
                                              : regExp.cap(1).toInt ());
    QString house = firstSim->getHousehold ();

    if (regExp.cap(3) == "baby" || regExp.cap(3) == "toddler" ||
        regExp.cap(3) == "child")
      emit newSim (regExp.cap(4), gender, age, asp, holdDays, house, species, 
                   firstSim);
    else emit newSim (regExp.cap(4), gender, age, asp, holdDays, house, 
                      species);
  }

  p = (QString)"^On day (\\d+) of the rotation, .+ adopted .+, whose name was" +
      (QString)" changed to (.+), (before|after) six PM " +
      (QString)"\\(eleven AM for pets\\)\\.";
  regExp.setPattern (p);

  if (regExp.exactMatch (display->text ()))  {
    int holdDays = (regExp.cap(3) == "before" ? regExp.cap(1).toInt () - 1
                                              : regExp.cap(1).toInt ());

    secondSim->setName (regExp.cap(2));
    secondSim->setHousehold (firstSim->getHousehold ());
    if (secondSim->isSpecies (HUMAN)) 
      secondSim->setParents (firstSim, firstSim->getSpouse ());
    secondSim->setHold (holdDays);
  }

  regExp.setPattern ("^.+ activated a (male|female) servo named (.+)\\.");

  if (regExp.exactMatch (display->text ()))  {
    cout << "Matched servo pattern." << endl;

    int gender = regExp.cap(1) == "male" ? MALE : FEMALE;
    QString house = firstSim->getHousehold ();

    emit newSim (regExp.cap(2), gender, ADULT, 
                 convertAspiration (firstSim->getAspiration()), 0,
                 firstSim->getHousehold (), SERVO);
  }

  p = (QString)"^On day (\\d+) of the rotation, .+ went on a vacation and" +
      (QString)" brought back Bigfoot (before|after) six PM.";
  regExp.setPattern (p);

  if (regExp.exactMatch (display->text ()))  {
    int holdDays = (regExp.cap(2) == "before" ? regExp.cap(1).toInt () - 1 
                                              : regExp.cap(1).toInt ());

    emit newSim ("Bigfoot", MALE, ADULT, NONE, holdDays, 
                 firstSim->getHousehold (), BIGFOOT);
  }

  // Aging

  regExp.setPattern ("^Put .+ on hold for (\\d+) days.");

  if (regExp.exactMatch (display->text ()))  {
    int rotations = regExp.cap(1).toInt ();
    cout << "rotations:" << rotations << endl;

    p = "^Put .*the (.+) household on hold for (\\d+) days.";
    regExp.setPattern (p);
    if (regExp.exactMatch (display->text ()))  {
      if (display->text ().contains ("all households except"))
        emit holdAllHouseholdsBut (regExp.cap(1), rotations);
      else emit holdHousehold (regExp.cap(1), rotations);
    }

    else if (display->text ().contains ("all sims and pets except"))
      emit holdAllBut (firstSim, rotations);
    else firstSim->setHold (rotations);
  }

  p = ".+ grew up (well|badly) (before|after) six PM \\(eleven AM for pets\\).";
  regExp.setPattern (p);

  if (regExp.exactMatch (display->text ()))  {
//    bool well = (regExp.cap (1) == "well");
    bool beforeSix = (regExp.cap (2) == "before");
    int asp = NONE;

    regExp.setPattern (".*\\(Aspiration: (.+)\\)");
    if (regExp.exactMatch (display->text ()))
      asp = convertAspiration (regExp.cap (1));

    firstSim->updateAspiration (asp, NONE);

    if (beforeSix) 
      firstSim->increaseAge (1);
  }

  p = "^.+ was taken away by the Social Worker\\.";
  regExp.setPattern (p);

  if (regExp.exactMatch (display->text ()))  {
    firstSim->setHousehold ("Adoption Queue");
    firstSim->setParents (NULL, NULL);
  }

  regExp.setPattern ("^.+ was removed by the police\\.");

  if (regExp.exactMatch (display->text ()))  {
    firstSim->setHousehold ("Adoption Queue");
    firstSim->setParents (NULL, NULL);
  }

  regExp.setPattern ("^.+ was given or sold to .+\\.");

  if (regExp.exactMatch (display->text ()))
    firstSim->setHousehold (secondSim->getHousehold ());

  regExp.setPattern ("^.+ was put up for adoption\\.");

  if (regExp.exactMatch (display->text ()))
    firstSim->setHousehold ("Adoption Queue");

  regExp.setPattern ("^.+ ran away from home\\.");

  if (regExp.exactMatch (display->text ()))
    firstSim->setHousehold ("Runaways");

  regExp.setPattern ("^.+ returned home to .+\\.");

  if (regExp.exactMatch (display->text ()))
    firstSim->setHousehold (secondSim->getHousehold ());

  regExp.setPattern ("^.+ was found and returned to .+\\.");

  if (regExp.exactMatch (display->text ()))
    firstSim->setHousehold (secondSim->getHousehold ());

  p = ".+ grew up (well|badly) and went to college\\. \\(Major: (.+)\\)";
  regExp.setPattern (p);

  if (regExp.exactMatch (display->text ()))  {
//    bool well = regExp.cap(1) == "well";

    firstSim->setCollege (true);
    firstSim->setMajor (regExp.cap(2));
    firstSim->setHold (1);
  }

  p = (QString)"^.+ grew up (well|badly) and (.+) (of|from) college\\. " +
      (QString)"\\(Major: (.*)\\)";
  regExp.setPattern (p);

  if (regExp.exactMatch (display->text ()))  {
//    bool well = regExp.cap(1) == "well";(.+)\\)");
    firstSim->setCollege (false);
    if (regExp.cap(2).contains ("graduated"))
      firstSim->setMajor (regExp.cap(4));
    else firstSim->setMajor ("");
  }

  regExp.setPattern ("^.+ died of old age\\.");

  if (regExp.exactMatch (display->text ()))
    firstSim->makeDead (WHITE);

  p = (QString)"^.+ (starved to death|drowned|burned to death|was " +
      (QString)"electrocuted by an appliance|was hit by a satelite|died of " +
      (QString)"disease|was killed by a swarm of Africanized flies|had an " +
      (QString)"accident while running with scissors|was eaten by a hungry " +
      (QString)"cowplant|turned to dust in the sunlight|was electrocuted by " +
      (QString)"a rampaging servo|was electrocuted by a lightning bolt|froze" +
      (QString)" to death|spontaneously combusted|withered away for lack of " +
      (QString)"sun, water, and love|was frightened to death by a ghost|" +
      (QString)"overextended him/herself envigorating the employees)\\.";
  regExp.setPattern (p);

  if(regExp.exactMatch (display->text ()))  {
    if (regExp.cap(1) == "starved to death") 
      firstSim->makeDead (8);
    else if (regExp.cap(1) == "drowned") 
      firstSim->makeDead (BLUE);
    else if (regExp.cap(1) == "burned to death")
      firstSim->makeDead (RED);
    else if (regExp.cap(1).startsWith ("electrocuted by a"))
      firstSim->makeDead (YELLOW);
    else if (regExp.cap(1) == "was killed by a swarm of Africanized flies")
      firstSim->makeDead (PURPLE);
    else if (regExp.cap(1) == "had an accident while running with scissors")
      firstSim->makeDead (PINK);
    else if (regExp.cap(1) == "was eaten by a hungry cowplant")
      firstSim->makeDead (BRIGHT_GREEN);
    else if (regExp.cap(1) == "turned to dust in the sunlight") 
      firstSim->makeDead (8);
    else if (regExp.cap(1) == "froze to death")
      firstSim->makeDead (BLUE);
    else if (regExp.cap(1) == "spontaneously combusted")
      firstSim->makeDead (RED);
    else if (regExp.cap(1) == "withered away for lack of sun, water, and love")
      firstSim->makeDead (8);
    else if (regExp.cap(1) == "was frightened to death by a ghost")
      firstSim->makeDead (PINK);
    else if (regExp.cap(1) == 
               "overextended him/herself envigorating the employees")
      firstSim->makeDead (8);
  }

  p = "^.+ was brought back to life (successfully|as a zombie) by .+\\.";
  regExp.setPattern (p);

  if (regExp.exactMatch (display->text ()))  {
    bool succ = regExp.cap(1) == "successfully";

    firstSim->resurrect (succ);
  }

  p = "^.+ drank Elixer of Life and (gained|lost) three sim days\\.";
  regExp.setPattern (p);

  if (regExp.exactMatch (display->text ()))  {
    int change = 3 * (regExp.cap(1) == "gained" ? -1 : 1);

    firstSim->increaseAge (change);
  }

  p = "^.+ drank a glass of Cowplant milk and gained five sim days\\.";
  regExp.setPattern (p);

  if (regExp.exactMatch (display->text ()))
    firstSim->increaseAge (-5);

  regExp.setPattern ("^.+ ate the Kibble of Life and (.+) three sim days\\.");

  if (regExp.exactMatch (display->text ()))  {
    int change = 3 * (regExp.cap(1) == "gained" ? -1 : 1);

    firstSim->increaseAge (change);
  }

  // Household and Family

  regExp.setPattern ("^.+ moved out to form the (.+) household\\.");

  if (regExp.exactMatch (display->text ()))  {
    cout << "Matched Household RegExp" << endl;
    QTextStream terminal (stdout);
    terminal << regExp.cap(1) << endl;
    terminal << firstSim << endl;

    firstSim->setHousehold (regExp.cap(1));

    cout << "Successfully set household" << endl;
  }

  regExp.setPattern ("^.+ moved to the (.+) household\\.");

  if (regExp.exactMatch (display->text ()))
    firstSim->setHousehold (regExp.cap(1));

  regExp.setPattern ("^.+'s grave was moved to the (.+) household\\.");

  if (regExp.exactMatch (display->text ()))
    firstSim->setHousehold (regExp.cap(1));

  p = "^.+ got married/joined to .+ and changed his or her name to (.+)\\.";
  regExp.setPattern (p);

  if (regExp.exactMatch (display->text ()))  {
    firstSim->setName (regExp.cap(1));
    firstSim->setSpouse (secondSim);
  }

  regExp.setPattern ("^.+ broke up with .+\\.");

  if (regExp.exactMatch (display->text ()))  {
    firstSim->clearSpouse ();
    secondSim->clearSpouse ();
  }

  // Changes

  regExp.setPattern ("^.+ changed his or her name to (.+)\\.");

  if (regExp.exactMatch (display->text ()))
    firstSim->setName (regExp.cap(1));

  regExp.setPattern ("^.+ changed his or her aspiration to (.+)\\.");

  if (regExp.exactMatch (display->text ()))
    firstSim->newAspiration (convertAspiration (regExp.cap(1)), NONE);

  regExp.setPattern ("^.+ changed his or her major to (.+)\\.");

  if (regExp.exactMatch (display->text ()))
    firstSim->setMajor (regExp.cap(1));

  // Supernatural
  
  regExp.setPattern ("^.+ was bitten by a vampire\\.");
 
  if (regExp.exactMatch (display->text ()))
    firstSim->addParanormal (VAMPIRE);

  regExp.setPattern ("^.+ was bitten by a werewolf\\.");
  
  if (regExp.exactMatch (display->text ()))
    firstSim->addParanormal (WEREWOLF);

  regExp.setPattern("^.+ was mutated into a plantsim by a cloud of pesticide.");

  if (regExp.exactMatch (display->text ()))
    firstSim->addParanormal (PLANTSIM);

  regExp.setPattern ("^.+ was given the powers of (.+)\\.");

  if (regExp.exactMatch (display->text ()))  {
    firstSim->addParanormal (WITCH);
    if (regExp.cap(1) == "Light") firstSim->setAlignment (NICE);
    if (regExp.cap(1) == "Neutrality") firstSim->setAlignment (NEUTRAL);
    if (regExp.cap(1) == "Darkness") firstSim->setAlignment (MEAN);
  }

  regExp.setPattern ("^.+ changed alignment to (.+)\\.");

  if (regExp.exactMatch (display->text ()))  {
    if (regExp.cap(1) == "Atrociously Evil")
      firstSim->setAlignment (ATROCIOUSLYEVIL);
    else if (regExp.cap(1) == "Evil")
      firstSim->setAlignment (EVIL);
    else if (regExp.cap(1) == "Mean")
      firstSim->setAlignment (MEAN);
    else if (regExp.cap(1) == "Neutral")
      firstSim->setAlignment (NEUTRAL);
    else if (regExp.cap(1) == "Nice")
      firstSim->setAlignment (NICE);
    else if (regExp.cap(1) == "Good")
      firstSim->setAlignment (GOOD);
    else if (regExp.cap(1) == "Infallibly Good")
      firstSim->setAlignment (INFALLIBLYGOOD);
  }

  regExp.setPattern ("^.+ drank Vamprocillin and became human again\\.");

  if (regExp.exactMatch (display->text ()))
    firstSim->removeParanormal (VAMPIRE);

  regExp.setPattern ("^.+ was cured of Lycanthropy\\.");

  if (regExp.exactMatch (display->text ()))
    firstSim->removeParanormal (WEREWOLF);

  regExp.setPattern ("^.+ was cured of Plantsiminism\\.");
 
  if (regExp.exactMatch (display->text ()))
    firstSim->removeParanormal (PLANTSIM);

  regExp.setPattern ("^.+ was cured of Witchiness\\.");

  if (regExp.exactMatch (display->text ()))
    firstSim->removeParanormal (WITCH);

  emit addToAchievementsPage (display->text (), firstSim, secondSim);
  emit deleteMe (this);
}

QString UpdateEntry::getText ()  {
  return display->text ();
}

int UpdateEntry::getFirstSimNum (QList< QPointer<Sim> > list)  {
  if (!firstSim) return -1;

  for (int x = 0; x < list.size (); x++)
    if (firstSim == list[x])
      return x;

  return -1;
}

int UpdateEntry::getSecondSimNum (QList< QPointer<Sim> > list)  {
  if (!secondSim) return -1;

  for (int x = 0; x < list.size (); x++)
    if (secondSim == list[x])
      return x;

  return -1;
}

int UpdateEntry::convertAspiration (QString name)  {
  if (name == "Romance") return ROMANCE;
  if (name == "Knowledge") return KNOWLEDGE;
  if (name == "Family") return FAMILY;
  if (name == "Popularity") return POPULARITY;
  if (name == "Fortune") return FORTUNE;
  if (name == "Pleasure") return PLEASURE;
  if (name == "Grilled Cheese") return GRILLED_CHEESE;
  if (name == "Grow Up") return GROW_UP;

  return NONE;
}


void UpdateEntry::deleteMe ()  {
  // Hehe >_>
  emit deleteMe (this);
}
