#include <QScrollArea>
#include <QPushButton>
#include <QComboBox>
#include <QLabel>
#include <QString>
#include <QStringList>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QTextStream>

#include <iostream>
using namespace std;

#include "const.h"
#include "sim.h"
#include "updateentry.h"
#include "updatedialog.h"
#include "updatepage.h"

UpdatePage::UpdatePage (QList< QPointer<Sim> > list)  {
  simList = list;
  setLivingSimsList ();
  epFlags = NONE;
  rotationLength = 1;

  dialog = NULL;

  catagoryBox = new QComboBox ();
  catagoryBox->addItem ("Select a Catagory");
  catagoryBox->addItem ("New Sims and Pets");
  catagoryBox->addItem ("Aging");
  catagoryBox->addItem ("Household and Family");
  catagoryBox->addItem ("Changes");
  catagoryBox->addItem ("Supernatural");
  catagoryBox->addItem ("Other");
  specificBox = new QComboBox ();
  specificBox->setSizeAdjustPolicy (QComboBox::AdjustToContents);

  connect (catagoryBox, SIGNAL (currentIndexChanged (const QString&)),
           this, SLOT (setSpecificBox (const QString&)));

  addButton = new QPushButton ("Add Event");
  addButton->setEnabled (false);
  addButton->setMaximumSize (addButton->sizeHint ());

  connect (addButton, SIGNAL (clicked ()), this, SLOT (createUpdateDialog ()));

  comboBoxLayout = new QHBoxLayout ();
  comboBoxLayout->addWidget (catagoryBox);
  comboBoxLayout->addWidget (specificBox);
  comboBoxLayout->addWidget (addButton);

  // This is a QScrollArea containing a single QWidget laid out with a
  // QVBoxLayout.  UpdateEntries are added to the QVBoxLayout, and in this way
  // arrange themselves nicely in the QScrollArea without me having to do
  // anything. Qt = <3
  eventLayout = new QVBoxLayout ();
  eventList = new QWidget ();
  eventList->setLayout (eventLayout);
  eventList->setMaximumWidth (700);
  eventList->show ();
  scrollArea = new QScrollArea ();
  scrollArea->setWidget (eventList);
  scrollArea->show ();

  updateButton = new QPushButton ("Apply Updates");
  updateButton->setMaximumSize (updateButton->sizeHint ());

  connect (updateButton, SIGNAL (clicked ()), this, SLOT (doUpdate ()));

  mainLayout = new QVBoxLayout ();
  mainLayout->addLayout (comboBoxLayout);
  mainLayout->setAlignment (comboBoxLayout, Qt::AlignCenter);
  mainLayout->addWidget (scrollArea);
  mainLayout->addWidget (updateButton);
  mainLayout->setAlignment (updateButton, Qt::AlignCenter);

  setLayout (mainLayout);
}

void UpdatePage::setSimList (QList< QPointer<Sim> > list)  {
  // The updatePage needs to have access to the simList so that when it makes
  // updates, they actually happen with the real, actual sims in the simList
  // and not copies.
  simList = list;
  setLivingSimsList ();
}

void UpdatePage::updateEPFlags (int flags)  {
  epFlags = flags;
}

void UpdatePage::updateRotationLength (int length)  {
  rotationLength = length;
}

QString UpdatePage::getCSV ()  {
  QString text = "";

  for (int x = 0; x < eventLayout->count (); x++)  {
    UpdateEntry *entry = 
                qobject_cast<UpdateEntry*>(eventLayout->itemAt (x)->widget ());

    int firstSimNum = entry->getFirstSimNum (simList);
    int secondSimNum = entry->getSecondSimNum (simList);

    text += "\"" + entry->getText () + "\"," + QString::number (firstSimNum) + 
            "," + QString::number (secondSimNum) + "\r\n";
  }

  return text;
}

void UpdatePage::readCSV (QString csv)  {
  QStringList list = csv.split (',');

  QString text = list[0];
  int currentSection = 1;

  while (!text.endsWith ("\""))
    text += "," + list[currentSection++];

  text.chop (1);
  text.remove (0, 1);

  int first = list[currentSection++].toInt ();
  int second = list[currentSection++].toInt ();

  Sim *s1 = (first >= 0) ? simList[first] : NULL;

  Sim *s2 = (second >= 0) ? simList[second] : NULL;

  UpdateEntry *newEntry = new UpdateEntry (text, s1, s2);
  addEvent (newEntry);
}

void UpdatePage::clear ()  {
  while (eventLayout->count () > 0)  {
    UpdateEntry *ptr = qobject_cast<UpdateEntry*>
                                        (eventLayout->itemAt (0)->widget ());
    eventLayout->removeWidget (ptr);
    delete ptr;
  }

  eventList->resize (eventLayout->minimumSize ());
}

void UpdatePage::setSpecificBox (const QString &catagory)  {
  specificBox->clear ();

  if (catagory == "New Sims and Pets")  {
    specificBox->addItem ("From CAS");
    specificBox->addItem ("Former Townies/NPCs");
    specificBox->addItem ("At Large Townies/NPCs");
    if (epFlags & PETS) specificBox->addItem ("Stray Pets");
    if (epFlags & PETS) specificBox->addItem ("At Large Stray Pets");
    specificBox->addItem ("Ordinary Babies");
    specificBox->addItem ("Babies by Townies");
    if (epFlags & PETS) specificBox->addItem ("Pet Babies");
    if (epFlags & PETS) specificBox->addItem ("Pet Babies by Strays");
    specificBox->addItem ("Alien Babies");
    if (epFlags & SEASONS) specificBox->addItem ("Plant Babies");
    if (epFlags & PETS) specificBox->addItem ("Adopted New Children and Pets");
    else specificBox->addItem ("Adopted New Children");
    if (epFlags & PETS) specificBox->addItem ("Readopted Children and Pets");
    else specificBox->addItem ("Readopted Children");
    if (epFlags & OFB) specificBox->addItem ("Activated Servos");
    if (epFlags & BONVOYAGE) specificBox->addItem ("Bigfoot");
  }

  else if (catagory == "Aging")  {
    specificBox->addItem ("Individual Holds");
    specificBox->addItem ("Household Holds");
    specificBox->addItem ("Growing up");
    specificBox->addItem ("Social Worker");
    if (epFlags & PETS) specificBox->addItem ("Removed Pets");
    if (epFlags & PETS) specificBox->addItem ("Given or sold Pets");
    if (epFlags & PETS) specificBox->addItem ("Pets put up for adoption");
    specificBox->addItem ("Runaway Teens");
    if (epFlags & PETS) specificBox->addItem ("Runaway Pets");
    specificBox->addItem ("Returned Teens");
    if (epFlags & PETS) specificBox->addItem ("Found Pets");
    if (epFlags & UNIVERSITY) specificBox->addItem ("Going to college");
    if (epFlags & UNIVERSITY) specificBox->addItem ("Leaving college");
    specificBox->addItem ("Natural Death");
    specificBox->addItem ("Unnatural Death");
    if (epFlags & UNIVERSITY) specificBox->addItem ("Resurrection");
    specificBox->addItem ("Drinking Elixer");
    if (epFlags & UNIVERSITY) specificBox->addItem ("Drinking Cowplant milk");
    if (epFlags & PETS) specificBox->addItem ("Kibble of Life");
  }

  else if (catagory == "Household and Family")  {
    specificBox->addItem ("Moving out");
    specificBox->addItem ("Moving house");
    specificBox->addItem ("Moving graves");
    specificBox->addItem ("Going steady");
    specificBox->addItem ("Getting engaged");
    specificBox->addItem ("Getting married/joined");
    specificBox->addItem ("Breaking up");
  }

  else if (catagory == "Changes")  {
    specificBox->addItem ("Changing names");
    if (!(epFlags & FREETIME))
      specificBox->addItem ("Changing aspirations");
    if (epFlags & FREETIME) 
      specificBox->addItem ("Changing primary and secondary aspirations");
    if (epFlags & UNIVERSITY) specificBox->addItem ("Changing majors");
  }

  else if (catagory == "Supernatural")  {
    if (epFlags & NIGHTLIFE) specificBox->addItem ("Becoming a Vampire");
    if (epFlags & PETS) specificBox->addItem ("Becoming a Werewolf");
    if (epFlags & SEASONS) specificBox->addItem ("Becoming a Plantsim");
    if (epFlags & APARTMENTLIFE) specificBox->addItem ("Becoming a Witch");
    if (epFlags & APARTMENTLIFE) specificBox->addItem ("Changing alignment");
    if (epFlags & NIGHTLIFE) specificBox->addItem ("Being cured of Vampirism");
    if (epFlags & PETS) specificBox->addItem ("Being cured of Lycanthropy");
    if (epFlags & SEASONS) specificBox->addItem("Being cured of Plantsiminism");
    if (epFlags & APARTMENTLIFE) specificBox->addItem 
      ("Being cured of Witchiness");
  }

  else if (catagory == "Other")
    specificBox->addItem ("Custom Event");

  addButton->setEnabled (specificBox->count ());
}

void UpdatePage::createUpdateDialog ()  {
  // Massive function of DOOM.  This consults the current state of the various
  // combo boxes to determine how to lay out the updateDialog.  UpdateDialog
  // is a beautiful, beautiful thing and no doubt has saved me hours, nay ages
  // of coding, but this function is still godawful long.
  //
  // UpdateDialog is a customizable dialog.  AddText adds a QLabel containing
  // some text, addEdit adds a QLineEdit, addChoice adds a QComboBox containing
  // entries from a QStringList.  AddSimChoice takes a list of sim pointers,
  // makes a QComboBox containing their names, and keeps track of which one
  // got selected.

  if (dialog) delete dialog;

  dialog = new UpdateDialog ();
  connect (dialog, SIGNAL (newEvent (UpdateEntry*)), this, 
           SLOT (addEvent (UpdateEntry*)));

  // Useful QStringLists for adding as choices.

  QStringList gender = QStringList () << "male" << "female";
  QStringList beforeSix = QStringList () << "before" << "after";

  QStringList households;

  for (int x = 0; x < simList.size (); x++)
    if (!(households.contains (simList[x]->getHousehold ())) &&
        !simList[x]->specialHousehold ())
      households.append (simList[x]->getHousehold ());

  // I would go through and document each of these if statements, explaining
  // why I made it so that certain sims can and cannot do certain things, but
  // I am so sick of this function that my eyes are rolling back into my head
  // just thinking about it.  I keep thinking I'll come back to it sooner or
  // later, but it just never stops being humongous.

  // New Sims
  if (specificBox->currentText () == "From CAS")  {
    dialog->addText ("On day ");
    dialog->addNumberChoice (1, rotationLength);
    dialog->addText (" of the rotation, ");
    dialog->addEdit ();
    dialog->addText (" (");
    dialog->addChoice (gender);
    dialog->addText ("), of the ");
    dialog->addEdit ();
    dialog->addText (" household, joined the neighborhood ");
    dialog->addChoice (beforeSix);
    dialog->addText (" six PM (eleven AM for pets) as ");

    QStringList age = QStringList () << "a Toddler" << "a Child" 
                                     << "a Teenager" << "a Young Adult"
                                     << "an Adult" << "an Elder";
    if (epFlags & PETS) age << "a large dog" << "a small dog" << "a cat" 
                            << "an elder large dog" << "an elder small dog" 
                            << "an elder cat";

    dialog->addChoice (age);
    dialog->addText (" (");

    QStringList asp = QStringList () << "No" << "Grow Up" << "Fortune" 
                                     << "Family" << "Popularity" << "Knowledge" 
                                     << "Romance";
    if (epFlags & NIGHTLIFE) asp << "Pleasure";
    asp << "Random";

    dialog->addChoice (asp);
    dialog->addText (" aspiration).");
  }

  else if (specificBox->currentText () == "Former Townies/NPCs")  {
    dialog->addText ("On day ");
    dialog->addNumberChoice (1, rotationLength);
    dialog->addText (" of the rotation, ");
    dialog->addEdit ();
    dialog->addText (" (");
    dialog->addChoice (gender);
    dialog->addText (", ");

    QStringList asp = QStringList () << "Fortune" << "Family" << "Knowledge"
                                     << "Popularity" << "Romance";
    if (epFlags & NIGHTLIFE) asp << "Pleasure";

    dialog->addChoice (asp);
    dialog->addText (" aspiration, ");

    QStringList a = QStringList () << "Adult";
    if (epFlags & UNIVERSITY) a << "Young Adult"; 
    a << "Elder";

    dialog->addChoice (a);
    dialog->addText (") ");

    QStringList means = QStringList () << "moved in with" 
                                       << "got married/joined to";

    dialog->addChoice (means);
    dialog->addText (" ");

    QList< QPointer<Sim> > marriageable;

    for (int x = 0; x < livingSims.size (); x++)
      if (livingSims[x]->isSpecies (HUMAN) &&
          !livingSims[x]->specialHousehold () &&
          ((livingSims[x]->ageToQString ().contains ("Adult")) ||
           (livingSims[x]->ageToQString ().contains ("Elder")) ||
           (livingSims[x]->ageToQString ().contains ("Major"))))
        marriageable.append (livingSims[x]);

    dialog->addSimChoice (marriageable);
    dialog->addText (" ");
    dialog->addChoice (beforeSix);
    dialog->addText (" six PM.");
  }

  else if (specificBox->currentText () == "At Large Townies/NPCs")  {
    QList< QPointer<Sim> > atlarge;

    for (int x = 0; x < livingSims.size (); x++)
      if (livingSims[x]->isSpecies (HUMAN) && livingSims[x]->isTownie ())
        atlarge.append (livingSims[x]);

    dialog->addText ("On day ");
    dialog->addNumberChoice (1, rotationLength);
    dialog->addText (" of the rotation, ");
    dialog->addSimChoice (atlarge);
    dialog->addText (" (");

    QStringList asp = QStringList () << "Fortune" << "Family" << "Knowledge"
                                     << "Popularity" << "Romance";
    if (epFlags & NIGHTLIFE) asp << "Pleasure";
    dialog->addChoice (asp);
    dialog->addText (" Aspiration) ");

    QStringList means = QStringList () << "moved in with" 
                                       << "got married/joined to";

    dialog->addChoice (means);
    dialog->addText (" ");

    QList< QPointer<Sim> > marriageable;

    for (int x = 0; x < livingSims.size (); x++)
      if (livingSims[x]->isSpecies (HUMAN) && 
          !livingSims[x]->specialHousehold () &&
          ((livingSims[x]->ageToQString ().contains ("Adult")) ||
           (livingSims[x]->ageToQString ().contains ("Elder")) ||
           (livingSims[x]->ageToQString ().contains ("Major"))))
        marriageable.append (livingSims[x]);

    dialog->addSimChoice (marriageable);
    dialog->addText (" ");
    dialog->addChoice (beforeSix);
    dialog->addText (" six PM.");
  }

  else if (specificBox->currentText () == "Stray Pets")  {
    QList< QPointer<Sim> > canadopt;

    for (int x = 0; x < livingSims.size (); x++)
      if (livingSims[x]->isSpecies (HUMAN) && 
          !livingSims[x]->specialHousehold () &&
          ((livingSims[x]->ageToQString ().contains ("Adult")) ||
           (livingSims[x]->ageToQString ().contains ("Elder"))))
        canadopt.append (livingSims[x]);

    dialog->addText ("On day ");
    dialog->addNumberChoice (1, rotationLength);
    dialog->addText (" of the rotation, ");
    dialog->addSimChoice (canadopt);
    dialog->addText (" adopted a stray ");
    dialog->addChoice (gender);
    dialog->addText (" ");

    QStringList species = QStringList () << "large dog" << "small dog" << "cat";

    dialog->addChoice (species);
    dialog->addText (" named ");
    dialog->addEdit ();
    dialog->addText (" ");
    dialog->addChoice (beforeSix);
    dialog->addText (" eleven AM.");
  }

  else if (specificBox->currentText () == "At Large Stray Pets")  {
    QList< QPointer<Sim> > canadopt;

    for (int x = 0; x < livingSims.size (); x++)
      if (livingSims[x]->isSpecies (HUMAN) && 
          !livingSims[x]->specialHousehold () &&
          ((livingSims[x]->ageToQString ().contains ("Adult")) ||
           (livingSims[x]->ageToQString ().contains ("Elder"))))
        canadopt.append (livingSims[x]);

    dialog->addText ("On day ");
    dialog->addNumberChoice (1, rotationLength);
    dialog->addText (" of the rotation, ");
    dialog->addSimChoice (canadopt);
    dialog->addText (" adopted ");

    QList< QPointer<Sim> > atlarge;

    for (int x = 0; x < livingSims.size (); x++)
      if (!livingSims[x]->isSpecies (HUMAN) && livingSims[x]->isTownie ())
        atlarge.append (livingSims[x]);

    dialog->addSimChoice (atlarge);
    dialog->addText (" ");
    dialog->addChoice (beforeSix);
    dialog->addText (" eleven AM.");
  }

  else if (specificBox->currentText () == "Ordinary Babies")  {
    QList< QPointer<Sim> > mSims;
    QList< QPointer<Sim> > fSims;

    for (int x = 0; x < livingSims.size (); x++) {
      if (livingSims[x]->isSpecies (HUMAN) && 
          livingSims[x]->getHousehold () != "Runaways" &&
          !(livingSims[x]->isParanormal (ZOMBIE)) &&
          !(livingSims[x]->isParanormal (BIGFOOT)) &&
          !(livingSims[x]->isParanormal (SERVO)))  {
        if (livingSims[x]->getGender () == MALE && 
            ((livingSims[x]->ageToQString ().contains ("Adult")) ||
             (livingSims[x]->ageToQString ().contains ("Elder")) ||
             (livingSims[x]->ageToQString ().contains ("Major"))))
          mSims.append (livingSims[x]);
        else if (livingSims[x]->getGender () == FEMALE &&
                 !livingSims[x]->specialHousehold () &&
                 (livingSims[x]->ageToQString ().contains ("Adult")))
          fSims.append (livingSims[x]);
      }
    }

    dialog->addText ("On day ");
    dialog->addNumberChoice (1, rotationLength);
    dialog->addText (" of the rotation, ");
    dialog->addSimChoice (mSims);
    dialog->addText (" and ");
    dialog->addSimChoice (fSims);
    dialog->addText (" had a ");
    dialog->addChoice (gender);
    dialog->addText (" baby named ");
    dialog->addEdit ();
    dialog->addText (" ");
    dialog->addChoice (beforeSix);
    dialog->addText (" six PM.");
  }

  else if (specificBox->currentText () == "Babies by Townies")  {
    QList< QPointer<Sim> > fSims;

    for (int x = 0; x < livingSims.size (); x++)
      if (livingSims[x]->isSpecies (HUMAN) && 
          !livingSims[x]->specialHousehold () &&
          livingSims[x]->getGender () == FEMALE &&
          (livingSims[x]->ageToQString ().contains ("Adult")))
        fSims.append (livingSims[x]);

    dialog->addText ("On day ");
    dialog->addNumberChoice (1, rotationLength);
    dialog->addText (" of the rotation, ");
    dialog->addSimChoice (fSims);
    dialog->addText (" had a ");
    dialog->addChoice (gender);
    dialog->addText (" baby named ");
    dialog->addEdit ();
    dialog->addText (" by an ");

    QStringList age = QStringList () << "adult" << "elder";

    dialog->addChoice (age);
    dialog->addText (" townie named ");
    dialog->addEdit ();
    dialog->addText (" ");
    dialog->addChoice (beforeSix);
    dialog->addText (" six PM.");
  }

  else if (specificBox->currentText () == "Pet Babies")  {
    QList< QPointer<Sim> > mPets;
    QList< QPointer<Sim> > fPets;

    for (int x = 0; x < livingSims.size (); x++) {
      if (!livingSims[x]->isSpecies (HUMAN) && 
          livingSims[x]->getHousehold () != "Runaways" &&
          livingSims[x]->getHousehold () != "Adoption Queue")  {
        if (livingSims[x]->getGender () == MALE &&
            (!(livingSims[x]->ageToQString ().contains ("Kitten")) ||
             !(livingSims[x]->ageToQString ().contains ("Puppy"))))
          mPets.append (livingSims[x]);

        else if (livingSims[x]->getGender () == FEMALE &&
                 !livingSims[x]->specialHousehold () &&
                 !(livingSims[x]->ageToQString ().contains ("Kitten")) &&
                 !(livingSims[x]->ageToQString ().contains ("Puppy")) &&
                 !(livingSims[x]->ageToQString ().contains ("Elder")))
          fPets.append (livingSims[x]);
      }
    }

    dialog->addText ("On day ");
    dialog->addNumberChoice (1, rotationLength);
    dialog->addText (" of the rotation, ");
    dialog->addSimChoice (mPets);
    dialog->addText (" and ");
    dialog->addSimChoice (fPets);
    dialog->addText (" had a ");
    dialog->addChoice (gender);
    dialog->addText (" ");

    QStringList spec = QStringList () << "puppy" << "kitten";

    dialog->addChoice (spec);
    dialog->addText (" named ");
    dialog->addEdit ();
    dialog->addText (" ");
    dialog->addChoice (beforeSix);
    dialog->addText (" eleven AM.");
  }

  else if (specificBox->currentText () == "Pet Babies by Strays")  {
    QList< QPointer<Sim> > fPets;

    for (int x = 0; x < livingSims.size (); x++)
      if (!livingSims[x]->isSpecies (HUMAN) && 
          livingSims[x]->getGender () == FEMALE &&
          !livingSims[x]->specialHousehold () &&
          !(livingSims[x]->ageToQString ().contains ("Kitten")) &&
          !(livingSims[x]->ageToQString ().contains ("Puppy")) &&
          !(livingSims[x]->ageToQString ().contains ("Elder")))
        fPets.append (livingSims[x]);

    dialog->addText ("On day ");
    dialog->addNumberChoice (1, rotationLength);
    dialog->addText (" of the rotation, ");
    dialog->addSimChoice (fPets);
    dialog->addText (" had a ");
    dialog->addChoice (gender);
    dialog->addText (" ");

    QStringList spec = QStringList () << "puppy" << "kitten";

    dialog->addChoice (spec);
    dialog->addText (" named ");
    dialog->addEdit ();
    dialog->addText (" by an ");

    QStringList age = QStringList () << "adult" << "elder";

    dialog->addChoice (age);
    dialog->addText (" stray named ");
    dialog->addEdit ();
    dialog->addText (" ");
    dialog->addChoice (beforeSix);
    dialog->addText (" eleven AM.");
  }

  else if (specificBox->currentText () == "Alien Babies")  {
    QList< QPointer<Sim> > mSims;

    for (int x = 0; x < livingSims.size (); x++)
      if (livingSims[x]->isSpecies (HUMAN) && 
          livingSims[x]->getGender () == MALE && 
          !livingSims[x]->isParanormal (ZOMBIE) && 
          !livingSims[x]->specialHousehold () &&
          (livingSims[x]->ageToQString ().contains ("Adult")))
        mSims.append (livingSims[x]);

    dialog->addText ("On day ");
    dialog->addNumberChoice (1, rotationLength);
    dialog->addText (" of the rotation, ");
    dialog->addSimChoice (mSims);
    dialog->addText (" had a ");
    dialog->addChoice (gender);
    dialog->addText (" alien baby named ");
    dialog->addEdit ();
    dialog->addText (" ");
    dialog->addChoice (beforeSix);
    dialog->addText (" six PM.");
  }

  else if (specificBox->currentText () == "Plant Babies")  {
    QList< QPointer<Sim> > pltSims;

    for (int x = 0; x < livingSims.size (); x++)
      if (livingSims[x]->isParanormal (PLANTSIM) &&  
          !livingSims[x]->specialHousehold () &&
          !(livingSims[x]->ageToQString ().contains ("Toddler")))
        pltSims.append (livingSims[x]);

    dialog->addText ("On day ");
    dialog->addNumberChoice (1, rotationLength);
    dialog->addText (" of the rotation, ");
    dialog->addSimChoice (pltSims);
    dialog->addText (" spawned a ");
    dialog->addChoice (gender);
    dialog->addText (" plantbaby named ");
    dialog->addEdit ();
    dialog->addText (" ");
    dialog->addChoice (beforeSix);
    dialog->addText (" six PM.");
  }

  else if ((specificBox->currentText () == "Adopted New Children and Pets") ||
           (specificBox->currentText () == "Adopted New Children"))  {
    QList< QPointer<Sim> > canadopt;

    for (int x = 0; x < livingSims.size (); x++)
      if (livingSims[x]->isSpecies (HUMAN) && 
          !livingSims[x]->specialHousehold () &&
          ((livingSims[x]->ageToQString ().contains ("Adult")) ||
           (livingSims[x]->ageToQString ().contains ("Elder"))))
        canadopt.append (livingSims[x]);

    dialog->addText ("On day ");
    dialog->addNumberChoice (1, rotationLength);
    dialog->addText (" of the rotation, ");
    dialog->addSimChoice (canadopt);
    dialog->addText (" adopted a ");
    dialog->addChoice (gender);
    dialog->addText (" ");

    QStringList age = QStringList () << "baby" << "toddler" << "child";
    if (epFlags & PETS) age << "puppy (small)" << "puppy (large)" << "large dog"
                            << "small dog" << "elder large dog" 
                            << "elder small dog" << "kitten" << "cat" 
                            << "elder cat";

    dialog->addChoice (age);
    dialog->addText (" named ");
    dialog->addEdit ();
    dialog->addText (" ");
    dialog->addChoice (beforeSix);
    dialog->addText (" six PM (eleven AM for pets).");
  }

  else if ((specificBox->currentText () == "Readopted Children and Pets") ||
           (specificBox->currentText () == "Readopted Children"))  {
    QList< QPointer<Sim> > canadopt;

    for (int x = 0; x < livingSims.size (); x++)
      if (livingSims[x]->isSpecies (HUMAN) && 
          !livingSims[x]->specialHousehold () &&
          ((livingSims[x]->ageToQString ().contains ("Adult")) ||
           (livingSims[x]->ageToQString ().contains ("Elder"))))
        canadopt.append (livingSims[x]);

    dialog->addText ("On day ");
    dialog->addNumberChoice (1, rotationLength);
    dialog->addText (" of the rotation, ");
    dialog->addSimChoice (canadopt);
    dialog->addText (" adopted ");

    QList< QPointer<Sim> > adoptable;

    for (int x = 0; x < livingSims.size (); x++)
      if (livingSims[x]->getHousehold () == "Adoption Queue")
        adoptable.append (livingSims[x]);

    dialog->addSimChoice (adoptable);
    dialog->addText (", whose name was changed to ");
    dialog->addEdit ();
    dialog->addText (", ");
    dialog->addChoice (beforeSix);
    dialog->addText (" six PM (eleven AM for pets).");
  }

  else if (specificBox->currentText () == "Activated Servos")  {
    QList< QPointer<Sim> > canbuild;

    for (int x = 0; x < livingSims.size (); x++)
      if (livingSims[x]->isSpecies (HUMAN) && 
          !livingSims[x]->specialHousehold () &&
          !(livingSims[x]->ageToQString ().contains ("Baby")) &&
          !(livingSims[x]->ageToQString ().contains ("Toddler")))
        canbuild.append (livingSims[x]);

    dialog->addSimChoice (canbuild);
    dialog->addText (" activated a ");
    dialog->addChoice (gender);
    dialog->addText (" servo named ");
    dialog->addEdit ();
    dialog->addText (".");
  }

  else if (specificBox->currentText () == "Bigfoot")  {
    QList< QPointer<Sim> > canbring;

    for (int x = 0; x < livingSims.size (); x++)
      if (livingSims[x]->isSpecies (HUMAN) && 
          !livingSims[x]->specialHousehold () &&
          ((livingSims[x]->ageToQString ().contains ("Adult")) ||
           (livingSims[x]->ageToQString ().contains ("Elder"))))
        canbring.append (livingSims[x]);

    dialog->addText ("On day ");
    dialog->addNumberChoice (1, rotationLength);
    dialog->addText (" of the rotation, ");
    dialog->addSimChoice (canbring);
    dialog->addText (" went on a vacation and brought back Bigfoot ");
    dialog->addChoice (beforeSix);
    dialog->addText (" six PM.");
  }

  // Aging
  else if (specificBox->currentText () == "Individual Holds")  {
    dialog->addText ("Put");

    QStringList options;
    options << "" << " all sims and pets except";

    dialog->addChoice (options);
    dialog->addText (" ");
    dialog->addSimChoice (livingSims);
    dialog->addText (" on hold for ");
    dialog->addNumberChoice (0, 20);
    dialog->addText (" days.");
  }

  else if (specificBox->currentText () == "Household Holds")  {
    dialog->addText ("Put ");

    QStringList options;
    options << "the " << "all households except the ";

    dialog->addChoice (options);
    dialog->addChoice (households);
    dialog->addText (" household on hold for ");
    dialog->addNumberChoice (0, 20);
    dialog->addText (" days.");
  }

  else if (specificBox->currentText () == "Growing up")  {
    QList< QPointer<Sim> > nonElders;

    for (int x = 0; x < livingSims.size (); x++)
      if (!(livingSims[x]->ageToQString ().contains ("Elder")) &&
          !(livingSims[x]->ageToQString ().contains ("Major")) &&
          !livingSims[x]->isParanormal (VAMPIRE) &&
          !livingSims[x]->isParanormal (ZOMBIE))
        nonElders.append (livingSims[x]);

    dialog->addSimChoice (nonElders);
    dialog->addText (" grew up ");

    QStringList well = QStringList () << "well" << "badly";

    dialog->addChoice (well);
    dialog->addText (" ");
    dialog->addChoice (beforeSix);
    dialog->addText (" six PM (eleven AM for pets). (Aspiration: ");

    QStringList asp = QStringList () << "Same" << "Family" << "Popularity" 
                                     << "Fortune" << "Knowledge" << "Romance";
    if (epFlags & NIGHTLIFE) asp << "Pleasure" << "Grilled Cheese"; 
    asp << "Random";

    dialog->addChoice (asp);
    dialog->addText (")");
  }

  else if (specificBox->currentText () == "Social Worker")  {
    QList< QPointer<Sim> > underTeen;

    for (int x = 0; x < livingSims.size (); x++)
      if (!livingSims[x]->specialHousehold () &&
          ((livingSims[x]->ageToQString ().contains ("Baby")) ||
           (livingSims[x]->ageToQString ().contains ("Toddler")) ||
           (livingSims[x]->ageToQString ().contains ("Child"))))
        underTeen.append (livingSims[x]);

    dialog->addSimChoice (underTeen);
    dialog->addText (" was taken away by the Social Worker.");
  }

  else if (specificBox->currentText () == "Removed Pets")  {
    QList< QPointer<Sim> > pets;

    for (int x = 0; x < livingSims.size (); x++)
      if (!livingSims[x]->isSpecies (HUMAN) && 
          !livingSims[x]->specialHousehold ())
        pets.append (livingSims[x]);

    dialog->addSimChoice (pets);
    dialog->addText (" was removed by the police.");
  }

  else if (specificBox->currentText () == "Given or sold Pets")  {
    QList< QPointer<Sim> > pets;

    for (int x = 0; x < livingSims.size (); x++)
      if (!livingSims[x]->isSpecies (HUMAN) && 
          !(livingSims[x])->specialHousehold ())
        pets.append (livingSims[x]);

    dialog->addSimChoice (pets);
    dialog->addText (" was given or sold to ");

    QList< QPointer<Sim> > receivers;

    for (int x = 0; x < livingSims.size (); x++)
      if (livingSims[x]->isSpecies (HUMAN) &&  
          !livingSims[x]->specialHousehold ())
        receivers.append (livingSims[x]);

    dialog->addSimChoice (receivers);
    dialog->addText (".");
  }

  else if (specificBox->currentText () == "Pets put up for adoption")  {
   QList< QPointer<Sim> > pets;

    for (int x = 0; x < livingSims.size (); x++)
      if (!livingSims[x]->isSpecies (HUMAN) && 
          !livingSims[x]->specialHousehold ())
        pets.append (livingSims[x]);

    dialog->addSimChoice (pets);
    dialog->addText (" was put up for adoption.");
  }

  else if (specificBox->currentText () == "Runaway Teens")  {
    QList< QPointer<Sim> > teens;

    for (int x = 0; x < livingSims.size (); x++)
      if (livingSims[x]->ageToQString ().contains ("Teen") &&
          !livingSims[x]->specialHousehold ())
        teens.append (livingSims[x]);

    dialog->addSimChoice (teens);
    dialog->addText (" ran away from home.");
  }

  else if (specificBox->currentText () == "Runaway Pets")  {
    QList< QPointer<Sim> > pets;

    for (int x = 0; x < livingSims.size (); x++)
      if (!livingSims[x]->isSpecies (HUMAN) && 
          !livingSims[x]->specialHousehold ())
        pets.append (livingSims[x]);

    dialog->addSimChoice (pets);
    dialog->addText (" ran away from home.");
  }

  else if (specificBox->currentText () == "Returned Teens")  {
    QList< QPointer<Sim> > teens;

    for (int x = 0; x < livingSims.size (); x++)
      if (livingSims[x]->isSpecies (HUMAN) && 
          livingSims[x]->getHousehold () == "Runaways")
        teens.append (livingSims[x]);

    dialog->addSimChoice (teens);
    dialog->addText (" returned home to ");

    QList< QPointer<Sim> > accepters;

    for (int x = 0; x < livingSims.size (); x++)
      if (!livingSims[x]->specialHousehold ())
        accepters.append (livingSims[x]);

    dialog->addSimChoice (accepters);
    dialog->addText (".");
  }

  else if (specificBox->currentText () == "Found Pets")  {
    QList< QPointer<Sim> > pets;

    for (int x = 0; x < livingSims.size (); x++)
      if (!livingSims[x]->isSpecies (HUMAN) && 
          livingSims[x]->getHousehold () == "Runaways")
        pets.append (livingSims[x]);

    dialog->addSimChoice (pets);
    dialog->addText (" was found and returned to ");

    QList< QPointer<Sim> > accepters;

    for (int x = 0; x < livingSims.size (); x++)
      if (!livingSims[x]->specialHousehold ())
        accepters.append (livingSims[x]);

    dialog->addSimChoice (accepters);
    dialog->addText (".");
  }

  else if (specificBox->currentText () == "Going to college")  {
    QList< QPointer<Sim> > teens;

    for (int x = 0; x < livingSims.size (); x++)
      if (livingSims[x]->ageToQString ().contains ("Teen") &&
          !livingSims[x]->specialHousehold ())
        teens.append (livingSims[x]);

    dialog->addSimChoice (teens);
    dialog->addText (" grew up ");

    QStringList well = QStringList () << "well" << "badly";

    dialog->addChoice (well);
    dialog->addText (" and went to college. (Major: ");
    dialog->addEdit ();
    dialog->addText (")");
  }

  else if (specificBox->currentText () == "Leaving college")  {
    QList< QPointer<Sim> > yadults;

    for (int x = 0; x < livingSims.size (); x++)
      if (livingSims[x]->ageToQString ().contains ("Major"))
        yadults.append (livingSims[x]);

    dialog->addSimChoice (yadults);
    dialog->addText (" grew up ");

    QStringList well = QStringList () << "well" << "badly";

    dialog->addChoice (well);
    dialog->addText (" and ");

    QStringList thing = QStringList () << "graduated Summa cum Laude from" 
                                       << "graduated Magna cum Laude from" 
                                       << "graduated without honors from"
                                       << "dropped out of" 
                                       << "was expelled from";

    dialog->addChoice (thing);
    dialog->addText (" college. (Major: ");
    dialog->addEdit ();
    dialog->addText (")");
  }

  else if (specificBox->currentText () == "Natural Death")  {
    QList< QPointer<Sim> > elders;

    for (int x = 0; x < livingSims.size (); x++)
      if ((livingSims[x]->ageToQString ().contains ("Elder")) &&
          !livingSims[x]->specialHousehold () && 
          !livingSims[x]->isParanormal (VAMPIRE) &&
          !livingSims[x]->isParanormal (ZOMBIE))
        elders.append (livingSims[x]);

    dialog->addSimChoice (elders);
    dialog->addText (" died of old age.");
  }

  else if (specificBox->currentText () == "Unnatural Death")  {
    QList< QPointer<Sim> > candie;

    for (int x = 0; x < livingSims.size (); x++)
      if (!livingSims[x]->specialHousehold () &&
          !(livingSims[x]->ageToQString ().contains ("Baby")) &&
          !(livingSims[x]->ageToQString ().contains ("Toddler")))
        candie.append (livingSims[x]);

    dialog->addSimChoice (candie);
    dialog->addText (" ");

    QStringList deadFlag;

    deadFlag << "starved to death." << "drowned." 
             << "burned to death." << "was electrocuted by an appliance."
             << "was hit by a saltelite." << "died of disease."
             << "was killed by a swarm of Africanized flies."
             << "was frightened to death by a ghost."
             << "had an accident while running with scissors.";
    if (epFlags & UNIVERSITY) deadFlag << "was eaten by a hungry cowplant.";
    if (epFlags & NIGHTLIFE) deadFlag << "turned to dust in the sunlight.";
    if (epFlags & OFB) deadFlag << "was electrocuted by a rampaging servo."
                     << "overextended him/herself envigorating the employees.";
    if (epFlags & SEASONS) deadFlag << "was electrocuted by a lightning bolt."
                                    << "froze to death." 
                                    << "spontaneously combusted."
                     << "withered away for lack of sun, water, and love.";

    dialog->addChoice (deadFlag);
  }

  else if (specificBox->currentText () == "Resurrection")  {
    QList< QPointer<Sim> > dead;

    for (int x = 0; x < simList.size (); x++)
      if (!simList[x]->isAlive ())
        dead.append (simList[x]);

    dialog->addSimChoice (dead);
    dialog->addText (" was brought back to life ");

    QStringList success = QStringList () << "successfully" << "as a zombie";

    dialog->addChoice (success);
    dialog->addText (" by ");
  
    QList< QPointer<Sim> > resurrectors;

    for (int x = 0; x < livingSims.size (); x++)
      if ((livingSims[x]->ageToQString ().contains ("Teen")) ||
          (livingSims[x]->ageToQString ().contains ("Adult")) ||
          (livingSims[x]->ageToQString ().contains ("Elder")) ||
          (livingSims[x]->ageToQString ().contains ("Servo")))
        resurrectors.append (livingSims[x]);

    dialog->addSimChoice (resurrectors);
    dialog->addText (".");
  }

  else if (specificBox->currentText () == "Drinking Elixer")  {
    QList< QPointer<Sim> > candrink;

    for (int x = 0; x < livingSims.size (); x++)
      if (livingSims[x]->isSpecies (HUMAN) && 
          !livingSims[x]->specialHousehold () &&
          ((livingSims[x]->ageToQString ().contains ("Teen")) ||
           (livingSims[x]->ageToQString ().contains ("Adult")) ||
           (livingSims[x]->ageToQString ().contains ("Elder"))))
        candrink.append (livingSims[x]);

    dialog->addSimChoice (candrink);
    dialog->addText (" drank Elixer of Life and ");

    QStringList gain = QStringList () << "gained" << "lost";

    dialog->addChoice (gain);
    dialog->addText (" three sim days.");
  }

  else if (specificBox->currentText () == "Drinking Cowplant milk")  {
    QList< QPointer<Sim> > candrink;

    for (int x = 0; x < livingSims.size (); x++)
      if (livingSims[x]->isSpecies (HUMAN) && 
          !livingSims[x]->specialHousehold () &&
          !(livingSims[x]->ageToQString ().contains ("Baby")) &&
          !(livingSims[x]->ageToQString ().contains ("Toddler")) &&
          !(livingSims[x]->isParanormal (SERVO)))
        candrink.append (livingSims[x]);

    dialog->addSimChoice (candrink);
    dialog->
         addText (" drank a glass of Cowplant milk and gained five sim days.");
  }

  else if (specificBox->currentText ()  == "Kibble of Life")  {
    QList< QPointer<Sim> > pets;

    for (int x = 0; x < livingSims.size (); x++)
      if (!livingSims[x]->isSpecies (HUMAN) && 
          !livingSims[x]->specialHousehold () &&
          (!(livingSims[x]->ageToQString ().contains ("Kitten")) &&
           !(livingSims[x]->ageToQString ().contains ("Puppy"))))
        pets.append (livingSims[x]);

    dialog->addSimChoice (pets);
    dialog->addText (" ate the Kibble of Life and ");

    QStringList gain = QStringList () << "gained" << "lost";

    dialog->addChoice (gain);
    dialog->addText (" three sim days.");
  }

  // Household and Family
  else if (specificBox->currentText () == "Moving out")  {
    dialog->addSimChoice (livingSims);
    dialog->addText (" moved out to form the ");
    dialog->addEdit ();
    dialog->addText (" household.");
  }

  else if (specificBox->currentText () == "Moving house")  {
    dialog->addSimChoice (livingSims);
    dialog->addText (" moved to the ");
    dialog->addChoice (households);
    dialog->addText (" household.");
  }

  else if (specificBox->currentText () == "Moving graves")  {
    QList< QPointer<Sim> > deadPeople;

    for (int x = 0; x < simList.size (); x++)
      if (!simList[x]->isAlive ())
        deadPeople.append (simList[x]);

    dialog->addSimChoice (deadPeople);
    dialog->addText ("'s grave was moved to the ");
    dialog->addChoice (households);
    dialog->addText (" household.");
  }

  else if (specificBox->currentText () == "Going steady")  {
    QList< QPointer<Sim> > teens;

    for (int x = 0; x < livingSims.size (); x++)
      if (!livingSims[x]->specialHousehold () &&
          livingSims[x]->ageToQString ().contains ("Teen"))
        teens.append (livingSims[x]);

    dialog->addSimChoice (teens);
    dialog->addText (" went steady with ");
    dialog->addSimChoice (teens);
    dialog->addText (".");
  }

  else if (specificBox->currentText () == "Getting engaged")  {
    QList< QPointer<Sim> > engageable;

    for (int x = 0; x < livingSims.size (); x++)
      if (livingSims[x]->isSpecies (HUMAN) && 
          livingSims[x]->getHousehold () != "Runaways" &&
          ((livingSims[x]->ageToQString ().contains ("Major")) ||
           (livingSims[x]->ageToQString ().contains ("Adult")) ||
           (livingSims[x]->ageToQString ().contains ("Elder"))))
        engageable.append (livingSims[x]);

    dialog->addSimChoice (engageable);
    dialog->addText (" got engaged to ");
    dialog->addSimChoice (engageable);
    dialog->addText (".");
  }

  else if (specificBox->currentText () == "Getting married/joined")  {
    QList< QPointer<Sim> > marriageable;

    for (int x = 0; x < livingSims.size (); x++)
      if (livingSims[x]->isSpecies (HUMAN) && 
          livingSims[x]->getHousehold () != "Runaways" &&
          ((livingSims[x]->ageToQString ().contains ("Adult")) ||
           (livingSims[x]->ageToQString ().contains ("Elder"))) &&
          !(livingSims[x]->getSpouse ()))
        marriageable.append (livingSims[x]);

    dialog->addSimChoice (marriageable);
    dialog->addText (" got married/joined to ");
    dialog->addSimChoice (marriageable);
    dialog->addText (" and changed his or her name to ");
    dialog->addEdit ();
    dialog->addText (".");
  }

  else if (specificBox->currentText () == "Breaking up")  {
    QList< QPointer<Sim> > breakable;

    for (int x = 0; x < livingSims.size (); x++)
      if (livingSims[x]->isSpecies (HUMAN) && 
          livingSims[x]->getHousehold () != "Runaways" &&
          !(livingSims[x]->ageToQString ().contains ("Baby")) &&
          !(livingSims[x]->ageToQString ().contains ("Toddler")) &&
          !(livingSims[x]->ageToQString ().contains ("Child")))
        breakable.append (livingSims[x]);

    dialog->addSimChoice (breakable);
    dialog->addText (" broke up with ");
    dialog->addSimChoice (breakable);
    dialog->addText (".");
  }

  // Changes
  else if (specificBox->currentText () == "Changing names")  {
    dialog->addSimChoice (simList);
    dialog->addText (" changed his or her name to ");
    dialog->addEdit ();
    dialog->addText (".");
  }

  else if (specificBox->currentText () == "Changing aspirations")  {
    QList< QPointer<Sim> > hasasp;

    for (int x = 0; x < livingSims.size (); x++)
      if (livingSims[x]->isSpecies (HUMAN) &&
          !(livingSims[x]->ageToQString ().contains ("Baby")) &&
          !(livingSims[x]->ageToQString ().contains ("Toddler")) &&
          !(livingSims[x]->ageToQString ().contains ("Child")))
        hasasp.append (livingSims[x]);

    dialog->addSimChoice (hasasp);
    dialog->addText (" changed his or her aspiration to ");

    QStringList asp = QStringList () << "Fortune" << "Family" << "Popularity"
                                     << "Romance" << "Knowledge";
    if (epFlags & NIGHTLIFE) asp << "Pleasure" << "Grilled Cheese";
    asp << "Random";

    dialog->addChoice (asp);
    dialog->addText (".");
  }

  else if (specificBox->currentText () == "Changing majors")  {
    QList< QPointer<Sim> > yas;

    for (int x = 0; x < livingSims.size (); x++)
      if (livingSims[x]->ageToQString ().contains ("Major"))
        yas.append (livingSims[x]);

    dialog->addSimChoice (yas);
    dialog->addText (" changed his or her major to ");
    dialog->addEdit ();
    dialog->addText (".");
  }

  // Supernatural
  else if (specificBox->currentText () == "Becoming a Vampire")  {
    QList< QPointer<Sim> > vampable;

    for (int x = 0; x < livingSims.size (); x++)
      if (livingSims[x]->isSpecies (HUMAN) && 
          !(livingSims[x]->specialHousehold ()) &&
          ((livingSims[x]->ageToQString ().contains ("Teen")) ||
           (livingSims[x]->ageToQString ().contains ("Adult")) ||
           (livingSims[x]->ageToQString ().contains ("Elder"))) &&
          !(livingSims[x]->isParanormal (VAMPIRE)))
        vampable.append (livingSims[x]);

    dialog->addSimChoice (vampable);
    dialog->addText (" was bitten by a vampire.");
  }

  else if (specificBox->currentText () == "Becoming a Werewolf")  {
    QList< QPointer<Sim> > lycanthropable;

    for (int x = 0; x < livingSims.size (); x++)
      if (livingSims[x]->isSpecies (HUMAN) && 
          !(livingSims[x]->specialHousehold ()) &&
          ((livingSims[x]->ageToQString ().contains ("Teen")) ||
           (livingSims[x]->ageToQString ().contains ("Adult")) ||
           (livingSims[x]->ageToQString ().contains ("Elder"))) &&
          !(livingSims[x]->isParanormal (WEREWOLF)))
        lycanthropable.append (livingSims[x]);

    dialog->addSimChoice (lycanthropable);
    dialog->addText (" was bitten by a werewolf.");
  }

  else if (specificBox->currentText () == "Becoming a Plantsim")  {
    QList< QPointer<Sim> > possplts;

    for (int x = 0; x < livingSims.size (); x++)
      if (livingSims[x]->isSpecies (HUMAN) && 
          !(livingSims[x]->specialHousehold ()) &&
          ((livingSims[x]->ageToQString ().contains ("Adult")) ||
           (livingSims[x]->ageToQString ().contains ("Elder"))) &&
          !(livingSims[x]->isParanormal (PLANTSIM))) 
        possplts.append (livingSims[x]);

    dialog->addSimChoice (possplts);
    dialog->addText (" was mutated into a plantsim by a cloud of pesticide.");
  }

  else if (specificBox->currentText () == "Becoming a Witch")  {
    QList< QPointer<Sim> > posswitches;

    for (int x = 0; x < livingSims.size (); x++)
      if (livingSims[x]->isSpecies (HUMAN) && 
          !(livingSims[x]->specialHousehold ()) &&
          (!(livingSims[x]->ageToQString ().contains ("Baby")) &&
           !(livingSims[x]->ageToQString ().contains ("Toddler")) &&
           !(livingSims[x]->ageToQString ().contains ("Child"))) &&
          !(livingSims[x]->isParanormal (WITCH)))
        posswitches.append (livingSims[x]);

    dialog->addSimChoice (posswitches);
    dialog->addText (" was given the powers of ");

    QStringList alignment;
    alignment << "Light" << "Neutrality" << "Darkness";

    dialog->addChoice (alignment);
    dialog->addText (".");
  }

  else if (specificBox->currentText () == "Changing alignment")  {
    QList< QPointer<Sim> > witches;

    for (int x = 0; x < livingSims.size (); x++)
      if (!livingSims[x]->specialHousehold () && 
          livingSims[x]->isParanormal (WITCH))
        witches.append (livingSims[x]);

    dialog->addSimChoice (witches);
    dialog->addText (" changed alignment to ");

    QStringList alignment;
    alignment << "Atrociously Evil" << "Evil" << "Mean" << "Neutral" << "Nice"
              << "Good" << "Infallibly Good";

    dialog->addChoice (alignment);
    dialog->addText (".");
  }

  else if (specificBox->currentText () == "Being cured of Vampirism")  {
    QList< QPointer<Sim> > vamps;

    for (int x = 0; x < livingSims.size (); x++)
      if (!livingSims[x]->specialHousehold () && 
          livingSims[x]->isParanormal (VAMPIRE))
        vamps.append (livingSims[x]);

    dialog->addSimChoice (vamps);
    dialog->addText (" drank Vamprocillin and became human again.");
  }

  else if (specificBox->currentText () == "Being cured of Lycanthropy")  {
    QList< QPointer<Sim> > weres;

    for (int x = 0; x < livingSims.size (); x++)
      if (!livingSims[x]->specialHousehold () && 
          livingSims[x]->isParanormal (WEREWOLF))
        weres.append (livingSims[x]);

    dialog->addSimChoice (weres);
    dialog->addText (" was cured of Lycanthropy.");
  }

  else if (specificBox->currentText () == "Being cured of Plantsiminism")  {
    QList< QPointer<Sim> > plants;

    for (int x = 0; x < livingSims.size (); x++)
      if (!livingSims[x]->specialHousehold () &&  
          livingSims[x]->isParanormal (PLANTSIM) &&
          !(livingSims[x]->ageToQString ().contains ("Toddler")))
        plants.append (livingSims[x]);

    dialog->addSimChoice (plants);
    dialog->addText (" was cured of Plantsiminism.");
  }

  else if (specificBox->currentText () == "Being cured of Witchiness")  {
    QList< QPointer<Sim> > witches;

    for (int x = 0; x < livingSims.size (); x++)
      if (!livingSims[x]->specialHousehold () && 
          livingSims[x]->isParanormal (WITCH))
        witches.append (livingSims[x]);

    dialog->addSimChoice (witches);
    dialog->addText (" was cured of Witchiness.");
  }

  // Custom
  else if (specificBox->currentText () == "Custom Event")  {
    dialog->addSimChoice (simList);
    dialog->addText (" ");
    dialog->addEdit ();
  }

  dialog->show ();
}

void UpdatePage::addEvent (UpdateEntry *update)  {
  eventLayout->addWidget (update);
  update->show ();
  eventList->resize (eventLayout->minimumSize ());

  connect (update, SIGNAL (holdAllBut (Sim*, int)), this, 
           SIGNAL (holdAllBut (Sim*, int)));
  connect (update, SIGNAL (holdHousehold (QString, int)), this,
           SIGNAL (holdHousehold (QString, int)));
  connect (update, SIGNAL (holdAllHouseholdsBut (QString, int)), this,
           SIGNAL (holdAllHouseholdsBut (QString, int)));
  connect (update, 
     SIGNAL (newSim (QString, int, int, int, int, QString, int, Sim*, Sim*)), 
           this, 
     SIGNAL (newSim (QString, int, int, int, int, QString, int, Sim*, Sim*)));
  connect (update, 
         SIGNAL (newTowniePair (QString, int, QString, int, int, int, Sim*)),
           this, 
         SIGNAL (newTowniePair (QString, int, QString, int, int, int, Sim*)));
  connect (update, SIGNAL (removeSim (Sim*)), this, SIGNAL (removeSim (Sim*)));
  connect (update, SIGNAL (addToAchievementsPage (QString, Sim*, Sim*)), this,
           SIGNAL (addToAchievementsPage (QString, Sim*, Sim*)));
  connect (update, SIGNAL (deleteMe (UpdateEntry*)), this, 
           SLOT (deleteEvent (UpdateEntry*)));
}

void UpdatePage::doUpdate ()  {
  // Where the magic happens.

  while (eventLayout->count () > 0)  {
    UpdateEntry *ptr = qobject_cast<UpdateEntry*>
                                        (eventLayout->itemAt (0)->widget ());
    eventLayout->removeWidget (ptr);
    ptr->go ();
  }

  eventList->resize (eventLayout->minimumSize ());

  emit addToAchievementsPage ("----------------------------------------", NULL,
                              NULL);
  emit rotate ();
}

void UpdatePage::setLivingSimsList ()  {
  livingSims.clear ();

  for (int x = 0; x < simList.size (); x++)
    if (simList[x]->isAlive ()) 
      livingSims.append (simList[x]);
}

void UpdatePage::deleteEvent (UpdateEntry *event)  {
  // This slot is triggered by a signal in updateEntry called deleteMe, which
  // sends a pointer to itself.  It's like saying "delete this" but l33ter in
  // that it doesn't kill the program.  Sweet.

  eventLayout->removeWidget (event);
  delete event;
  eventList->resize (eventLayout->minimumSize ());
}
